/**********************************************************************//**
***************************************************************************
*** @file    flib.h
***
*** @brief   Basic Functions, Small but Fast.
***
*** @author  Copyright (c) 2014 Vision Components.
*** @author  All rights reserved.
*** @author  This software embodies materials and concepts which are
***          confidential to Vision Components.
***
***
*** @revisionHistory
***
***    Date          Version    Author  Changes
***    28.02.2014    1.0        MBE     Added Header.
***
*** @endRevisionHistory
***************************************************************************
***************************************************************************/
#ifndef _FLIB_h
#define _FLIB_h

#ifndef _VCTYPES_h
    #include <vctypes.h>
#endif


#ifdef __cplusplus
extern "C" {
#endif

/* lib version number   */

#define FASTLIB_VERSION     (314)
#define FASTLIB_SUBVERSION  (6)

#if defined __TI_COMPILER_VERSION__

  #define FLIB_API

#endif

#if defined __GNUC__
  #define restrict  __restrict
  #define FLIB_API
  #define _add2(a, b)       ((a & 0xFFFF0000) + (b & 0xFFFF0000) + ((a + b) & 0xFFFF))
#endif

#ifndef FLIB_API
  #define restrict                          /* the restrict keyword is only necessary for TI CCS    */
  #define FLIB_API __declspec(dllexport)    /* FLIB API FOR MICROSOFT                               */
                                            /* if a Microsoft C++ DLL is needed, the macro FLIB_API */
                                            /* must be placed before the function declaration, e.g. */
                                            /* void FLIB_API function(void);                        */
  #define _add2(a, b)       ((a & 0xFFFF0000) + (b & 0xFFFF0000) + ((a + b) & 0xFFFF))
#endif

/* lib version number   */

void GetLibVer_FASTLIB(int *Version, int *SubVersion);
char *GetLibVerString_FASTLIB(void);


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  2x2Flib           2x2 Flib
*
*  Functions in this group process @times{2,2} using image lines as input.
*  @todo weitere Doku
*
* @{
*/
 
/* 2x2lib       */
void FL_2x2_AVG_U8P_U8P(U32 count, U8 *src1, U8 *src2, U8 *dest);   /* moving average           */
U32  FL_2x2_FOC_U8P_U8P(U32 count, U8 *src1, U8 *src2, I32 sh);     /* 2x2 focus function       */
void FL_2x2_GRAD(U32 cnt, U8 *src1, U8 *src2, U8 *g, U8 *d, U8 *t); /* 2x2 gradient             */
void FL_2x2_GRAD2(U32 count, U8 *p1, U8 *p2, U8 *g, U8 *d, U8 *t);  /* 2x2 precise gradient     */
void FL_2x2_ROB_U8P_U8P(U32 count, U8 *src1, U8 *src2, U8 *dest);   /* robert's cross           */
void FL_2x2_PAV_U8P_U8P(U32 count, U8 *src1, U8 *src2, U8 *dest);   /* pyramid average          */
void FL_2x2_PMX_U8P_U8P(U32 count, U8 *src1, U8 *src2, U8 *dest);   /* pyramid maximum          */
void FL_2x2_PMN_U8P_U8P(U32 count, U8 *src1, U8 *src2, U8 *dest);   /* pyramid minimum          */
void FL_2x2_PAC_U8P_U8P(U32 count, U8 *src1, U8 *src2, U8 *dest);   /* pyramid mean bayer vers. */
void FL_2x2_SUM_U16P_U16P(U32 count, U16 *s1, U16 *s2, U16 *dst);	/* moving sum, U16			*/
U32  FL_2x2_SumMax_U16P_U16P(U32 cnt, U16 *src1, U16 *src2, I32 *x);/* max of moving sum        */
U32  FL_2x2_SMULMAX_U16P_U16P(U32 cnt, U16 *s11, U16 *s12, 			/* maxmul of moving sum     */
                                        U16 *s21, U16 *s22, I32 *x);

/**@}@endcond*//*---End of 2x2Flib--------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  3x3Flib           3x3 Flib
*
*  Functions in this group process @times{3,3} using image lines as input.
*  @todo weitere Doku
*
* @{
*/
 
/* 3x3lib       */
#define BELOW_THRESHOLD  (128)

void FL_3x3_AVG_U8P(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);                      /* moving average           */
void FL_3x3_MAX_U8P(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);                      /* maximum operator         */
void FL_3x3_MIN_U8P(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);                      /* minimum operator         */
void FL_3x3_MED_U8P(I32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);                      /* median filter            */
void FL_3x3_LAP_U8P(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);                      /* laplace operator         */
void FL_3x3_SOB_U8P(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);                      /* sobel operator           */
void FL_3x3_SOBH_U8P(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);                     /* hor. sobel with offset   */
void FL_3x3_SOBV_U8P(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);                     /* ver. sobel with offset   */
void FL_3x3_GRAD(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *g, U8 *d, U8 *t);            /* 3x3 gradient             */
void FL_3x3_GRADT(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *d, U32 thr, U8 *t);   /* 3x3 gradient w. thresh.  */
void FL_3x3_f3f_U8P(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr, I32 c[3][3], I32 sh); /* 3x3 filter, abs          */
void FL_3x3_f3g_U8P(U32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr, I32 c[3][3], I32 sh); /* 3x3 filter, saturation   */
void FL_3x1_f3g_U8P(U32 dx, U8 *p1, U8 *pr, I32 c[3], I32 sh);                       /* 3x1 filter, saturation   */
void FL_3x3_NONMAX(U32 count, U8 *g0, U8 *g1, U8 *g2, U8 *d, U8 *pr, I32 *adtbl);	 /* nonmaximum supression    */
U32  FL_3x3_SumMax_U16P_U16P(U32 count, U16 *src1, U16 *src2, U16 *src3, I32 *x);    /* max of moving sum        */
                                 
/**@}@endcond*//*---End of 3x3Flib--------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  5x5Flib           5x5 Flib
*
*  Functions in this group process @times{5,5} using image lines as input.
*  @todo weitere Doku
*
* @{
*/
 
/* 5x5lib       */
void FL_5x5_f5f_U8P(U32 dx, U8 *p1, U8 *p2, U8 *p3, U8 *p4, U8 *p5, U8 *pr, I32 c[5][5], I32 sh);       /* 5x5 filter               */
void FL_5x5_f5hf_U8P_I16P(U32 dx, U8 *px, I16 *pr, I32 h[], I32 sh);                                    /* 5x1 filter               */
void FL_5x5_f5hx_U8P_I16P(U32 dx, U8 *px, I16 *pr, I32 h[], I32 sh);                                    /* 5x1 symmetric filter     */
void FL_5x5_f5vf_I16P_U8P(U32 dx, I16 *p1, I16 *p2, I16 *p3, I16 *p4, I16 *p5, U8 *pr, I32 v[],I32 sh); /* 1x5 filter               */
void FL_5x5_f5vx_I16P_U8P(U32 dx, I16 *p1, I16 *p2, I16 *p3, I16 *p4, I16 *p5, U8 *pr, I32 v[],I32 sh); /* 1x5 symmetric filter     */
void FL_1x5_MED_U8P(I32 dx, U8 *ps, U8 *pr);                       									    /* 1x5 median filter        */
void FL_5x1_MED_U8P(I32 dx, U8 *p1, U8 *p2, U8 *p3, U8 *p4, U8 *p5, U8 *pr);							/* 5x1 median filter        */
                                                                       

/**@}@endcond*//*---End of 5x5Flib--------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  7x7Flib           7x7 Flib
*
*  Functions in this group process @times{7,7} using image lines as input.
*  @todo weitere Doku
*
* @{
*/
 
/* 7x7lib       */                                                  									/* 7x7 filter               */
void FL_7x7_f7hf_U8P_I16P(U32 dx, U8 *px, I16 *pr, I32 h[], I32 sh);									/* 7x1 filter               */
void FL_7x7_f7hx_U8P_I16P(U32 dx, U8 *px, I16 *pr, I32 h[], I32 sh);									/* 7x1 symmetric filter     */
void FL_7x7_f7vf_I16P_U8P(U32 dx, I16 *p1, I16 *p2, I16 *p3,        									/* 1x7 filter               */
      I16 *p4, I16 *p5, I16 *p6, I16 *p7, U8 *pr, I32 v[],I32 sh);
void FL_7x7_f7vx_I16P_U8P(U32 dx, I16 *p0, I16 *p1, I16 *p2,        									/* 1x7 symmetric filter     */
      I16 *p3, I16 *p4, I16 *p5, I16 *p6, U8 *pr, I32 v[],I32 sh);
void FL_1x7_MED_U8P(I32 dx, U8 *ps, U8 *pr);                       										/* 1x7 median filter        */
void FL_7x1_MED_U8P(I32 dx, U8 *p1, U8 *p2, U8 *p3, U8 *p4, U8 *p5, U8 *p6, U8 *p7, U8 *pr);			/* 7x1 median filter        */
                                                          


/**@}@endcond*//*---End of 7x7Flib--------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  accuFlib          Accu Flib
*
*  @a FL_accu_() functions in this group add the corresponding 
*  elements of the @p src vector to the @p accu vector provided, like:
*  
*  @code
*   accu[i] += src[i];
*  @endcode
*  
*  @a FL_accusq_() functions in this group add the square of the corresponding 
*  elements of the @p src vector to the @p accu vector provided, like:
*  
*  @code
*   accu[i] += src[i] * src[i];
*  @endcode
*  
*  @a FL_accusub_() functions in this group subtract the corresponding 
*  elements of the @p src vector to the @p accu vector provided, like:
*  
*  @todo Warum accusub in accuFlib und nicht in subFlib?
*  
*  @code
*   accu[i] -= src[i];
*  @endcode
*
* @{
*/
 
/* acculib      */
void FL_accu_I8P_I16P(U32 count, I8 *src, I16 *accu);               /* vector accumulate        */
void FL_accu_I8P_I32P(U32 count, I8 *src, I32 *accu);               /* vector accumulate        */
void FL_accu_I16P_I32P(U32 count, I16 *src, I32 *accu);             /* vector accumulate        */
void FL_accu_U8P_U16P(U32 count, U8 *src, U16 *accu);               /* vector accumulate        */
void FL_accu_U8P_U32P(U32 count, U8 *src, U32 *accu);               /* vector accumulate        */
void FL_accu_U16P_U16P(U32 count, U16 *src, U16 *accu);             /* vector accumulate        */
void FL_accu_U16P_U32P(U32 count, U16 *src, U32 *accu);             /* vector accumulate        */
void FL_accu_U32P_U32P(U32 count, U32 *src, U32 *accu);             /* vector accumulate        */
void FL_accuSq_U8P_I32P(U32 count, U8 *src, I32 *accu);             /* vector square accumulate */
void FL_accuSq_U8P_U32P(U32 count, U8 *src, U32 *accu);             /* vector square accumulate */
void FL_accusub_U8P_U16P(U32 count, U8 *src, U16 *accu);            /* vector accumulate subtr. */
void FL_accusub_U8P_U32P(U32 count, U8 *src, U32 *accu);            /* vector accumulate subtr. */

/**@}@endcond*//*---End of 7x7Flib--------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  addFlib       Add Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* addlib       */
void FL_add_U8P_U8P  (I32 count, U8  *src1, U8  *src2, U8  *dst);   /* vector add               */
void FL_add_U16P_U16P(U32 count, U16 *src1, U16 *src2, U16 *dst);   /* vector add               */
void FL_add_I16P_I16P(U32 count, I16 *src1, I16 *src2, I16 *dst);   /* vector add               */
void FL_add_U32P_U32P(U32 count, U32 *src1, U32 *src2, U32 *dst);   /* vector add               */
void FL_add_I32P_I32P(U32 count, I32 *src1, I32 *src2, I32 *dst);   /* vector add               */

void FL_addn_U16P_U16P(U32 count, U16 *src1, U16 *src2, U8 *dst);   /* add and normalize        */

void FL_add_I8P_I8(U32 count, I8 *src, I8 *dst, I8 offset);         /* add constant to vector   */
void FL_add_I16P_I16(U32 count, I16 *src, I16 *dst, I16 offset);    /* add constant to vector   */
void FL_add_I32P_I32(U32 count, I32 *src, I32 *dst, I32 offset);    /* add constant to vector   */

U32  FL_sum_U8P(U32 count, U8 *src);                                /* sum up vector elements   */
I32  FL_sum_I16P(U32 count, I16 *src);                              /* sum up vector elements   */
U32  FL_sum_U16P(U32 count, U16 *src);                              /* sum up vector elements   */
I32  FL_sum_I32P(U32 count, I32 *src);                              /* sum up vector elements   */
U32  FL_sum_U32P(U32 count, U32 *src);                              /* sum up vector elements   */
I32  FL_sumabs_I32P(U32 count, I32 *src);                           /* sum up vector elements   */
U32  FL_sum_inc_U8P(U32 count, U8 *src, U32 inc_src);               /* sum with source increment*/
I32  FL_sum_inc_I16P(U32 count, I16 *src, U32 inc_src);             /* sum with source increment*/
U32  FL_sum_inc_U16P(U32 count, U16 *src, U32 inc_src);             /* sum with source increment*/
U32  FL_orabs_I16P(U32 count, I16 *src);                            /* check vector for zero    */
I32  FL_sumL_U8P(U32 count, U8 *src, I32 offs, I32 tol);            /* I32 sum with offset and  */
 
/**@}@endcond*//*---End of addFlib       -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  alphaFlib Alpha Flib
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/
                                                                    /* tolerance clipping       */
/* alpha        */
#define  TMUL       (256)                                           /* scaling multiplier       */
#define  TADD       (127)                                           /* rounding constant        */
#define  TSHIFT     (8)                                             /* scaling shift            */
#define  TADD2      (32767)                                         /* rounding constant        */
#define  TSHIFT2    (16)                                            /* scaling shift            */
#define  MAXINT     (32768)                                         /* maximum signed short     */

void FL_alpha_U8P_U8P(U32 count, U32 alpha, U8 *src, U8 *dst);      /* horizontal alphablending */
void FL_alpha_U8P_U8P_U8P(U32 count, U32 alpha, U8 *src1,           /* vertical alphablending   */
                                            U8 *src2, U8 *dst);
void FL_beta_U8P_U8P_U8P(U32 count, U32 alpha, U32 beta,            /* vert. alpha/beta blending*/
                                U8 *src1, U8 *src2, U8 *dst);
void FL_beta_U8P_U16P_U16P(U32 count, U32 alpha, U32 beta,          /* vert. alpha/beta blending*/
                         U8 *src1, U16 *src2, U16 *dst);            /* sp. ISEF version         */
void FL_alpha2D_U8P_U8P_U8P(U32 count, U32 alphaX, U32 alphaY,      /* 2D alphablending         */
                                  U8 *src1, U8 *src2, U8 *dst);
void alpha2D_BG(U32 alphaX, U32 alphaY,                             /* 2D alpha with background */
                         U8 *src1, U8 *src2, U8 *dst, U8 bgnd);

void FL_beta2_U8P_I16P_I16P(U32 count, I16 a, I16 b1, I16 b2,        /* betablending for ISEF   */
            I16 b3, U8 *x, I16 *AA0, I16 *AA1, I16 *AA2, I16 *AA3);  /* and GAUSS                */
void FL_beta2_I16P_I16P_I16P(U32 count, I16 a, I16 b1, I16 b2,       /* betablending for ISEF   */
           I16 b3, I16 *x, I16 *AA0, I16 *AA1, I16 *AA2, I16 *AA3);  /* and GAUSS                */

/**@}@endcond*//*---End of alphaFlib -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  bitnFlib         Bitn Flib
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* bitn         */
#ifndef _VCLIB_h
#define MAXPREC 5                                                   /* maximum number of words  */

typedef struct
  {
  I32 precision;
  U32 v[MAXPREC];
  }  BITN;
#endif

void   BN_set(BITN *x, I32 y);                                      /* set BITN value           */
void   BN_cpy(BITN *x, BITN *z);                                    /* copy BITN numbers        */
void   BN_prt(BITN *x);                                             /* BITN HEX print           */
float  BN_tof(BITN *x);                                             /* BITN to float conversion */
double BN_tod(BITN *x);                                             /* BITN to double conversion*/
void   BN_add(BITN *x, BITN *y, BITN *z);                           /* add two BITN numbers     */
void   BN_sub(BITN *x, BITN *y, BITN *z);                           /* subtract BITN numbers    */
void   BN_mul(BITN *x, BITN *y, BITN *z);                           /* BITN signed multiply     */
void   BN_mulu(BITN *x, BITN *y, BITN *z);                          /* unsigned multiplication  */
void   BN_div(BITN *x, BITN *y, BITN *z);                           /* signed division          */
void   BN_divu(BITN *x, BITN *y, BITN *z);                          /* unsigned division        */
void   BN_abs(BITN *x, BITN *z);                                    /* absolute value           */
void   BN_neg(BITN *x, BITN *z);                                    /* two's complement         */
void   BN_lsl(BITN *x, BITN *z, I32 shift);                         /* logical shift left       */
void   BN_lsr(BITN *x, BITN *z);                                    /* logical shift right BITN */
void   BN_asr(BITN *x, BITN *z);                                    /* arithm. shift right BITN */
I32    BN_chk(BITN *x);                                             /* check for sign and zero  */
I32    BN_cpu(BITN *x);                                             /* check precision unsigned */
I32    BN_cps(BITN *x);                                             /* check precision signed   */
I32    BN_lmb(BITN *x);                                             /* number of leading bits   */
void   BN_ext(BITN *x, I32 precision);                              /* sign extend BITN value   */

/* bitn macros  */

#define BN_SET(x, y)    {(x)->v[0] = (y); (x)->precision=1;}
#define BN_STL(x, y)    {(x)->v[0] = (U32)(y); (x)->v[1] = (U32)(y>>32); (x)->precision=2;}
#define BN_ISNEG(x)     (((BITN *)(x))->v[((BITN *)(x))->precision-1] & 0x80000000)

#define BN_ADD1(x, y, z)    {(z)->v[0] = (x)->v[0] + (y)->v[0]; (z)->precision=1;}
#define BN_MULU1(x, y, z)   {(z)->v[0] = (U16)(x)->v[0] * (U16)(y)->v[0]; (z)->precision=1;}

/* BN_MAC1 precision x: 16bit y: 16bit z: 64bit (accumulator)                               */
#define BN_MAC1(x, y, z)    { U40 accu   = (U40)((U16)(x)->v[0] * (U16)(y)->v[0]);          \
                              accu       = accu + (U40)(z)->v[0];                           \
                              (z)->v[0]  = accu;                                            \
                              (z)->v[1] += (accu>>32); (z)->precision=2; }

/* BN_MAC2 precision x: 16bit y: 32bit z: 64bit (accumulator)                               */
#define BN_MAC2(x, y, z)    { U40 accu   = (U40)((U16)(x)->v[0] * (U16)(y)->v[0]);          \
                              U32 vz     = (U16)(x)->v[0] * (U16)((y)->v[0]>>16);           \
                              accu       = accu + (U40)(vz<<16) + (U40)(z)->v[0];           \
                              (z)->v[0]  = accu;                                            \
                              (z)->v[1] += (accu>>32) + (vz>>16); (z)->precision=2;}

/* BN_ADD2 precision x: 64bit y:32bit z: 64bit                                              */
#define BN_ADD2(x, y, z)    {U40 accu; (z)->precision=2;                                    \
                            accu = (U40)(x)->v[0] + (U40)(y)->v[0];                         \
                            (z)->v[0] = (U32)accu;                                          \
                            (z)->v[1] = (x)->v[1] + (U32)(accu>>32);}

#ifndef __TI_COMPILER_VERSION__
    int _lmbd(int y, int x);                                        /* number of bits of x         */
#endif
/**@}@endcond*//*---bitnFlib         -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  colorFlib     Color Flib     
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/
 
/* colorlib     */
void FL_GreyToRGBO(U32 count, U8 *grey, U8 *ovl, U32 *rgbo);        /* RGBO output conversion   */
void FL_RGB_ToGrey(I32 dx, U8 *red, U8 *grn, U8 *blu, U8 *grey);    /* simple RGB to Y conv.    */
void FL_RGBToRGBO(U32 count, U8 *red, U8 *grn,                      /* RGBO output conversion   */
                               U8 *blu, U8 *ovl, U32 *rgbo);
void FL_RGBToGreyRGBO(I32 dx, U8 *red, U8 *grn, U8 *blu,            /* simple conversion from   */
                                            U8 *ovl, U32 *rgbo);    /* RGB to Grey + overlay    */
void FL_RGBToBMP(U32 count, U8 *r, U8 *g, U8 *b, U8 *rgb);          /* BMP output conversion    */
void FL_RGBToIHS(U32 count, U8 *red, U8 *grn,                       /* RGB to IHS conversion    */
                U8 *blu, U8 *I, U8 *H, U8 *S, U32 *reciprocal);
void FL_RGBToYCbCr(U32 count, U8 *red, U8 *grn,                     /* RGB to YCbCr conversion  */
                            U8 *blu, U8 *y, U8 *cb, U8 *cr);
void FL_RGBToYCbCr2(U32 count, U8 *red, U8 *grn,                    /* RGB to YCbCr conversion  */
                             U8 *blu, U8 *y, U8 *cb, U8 *cr);       /* decimation by 2          */
void FL_YCbCrToRGB(U32 count,  U8 *y, U8 *cb, U8 *cr,               /* YCbCr to RGB  conversion */
                                 U8 *red, U8 *grn, U8 *blu);
void FL_YCbCrToRGB2(U32 count, U8 *y, U8 *cb, U8 *cr,               /* YCbCr to RGB conversion  */
                           U8 *red, U8 *grn, U8 *blu);              /* decimation by 2          */
void FL_YCbCrToRGBO(U32 count, U8 *yy, U8 *ccb,                     /* RGBO output conversion   */
                               U8 *ccr, U8 *ovl, U32 *rgbo);
void FL_YCbCrToYUVNORM(U32 count, U8 *yy, U8 *cb, U8 *cr, U8 *x,    /* YCbCr to normalized YUV  */
                             U8 *nb, U8 *nr, U16 *recb, U16 *recr);
void FL_YUVNORMToYCbCr(U32 count, U8 *x, U8 *nb, U8 *nr,            /* normalized YUV to YCbCr  */
                                      U8 *yy, U8 *cb, U8 *cr);
void FL_BayerToRGB_5x5(I32 dx2, U8 *BayerPage, I32 pitch,           /* Bayer to RGB 5x5 conv.   */
            U8 *red, U8 *grn, U8 *blu, U8 *gtrail, U8 *ghead);      /* main routine             */
void FL_clookf(U32 count, U8 *u, U8 *v, U8 *r, U8 table[]);         /* LUT for color classify   */

/**@}@endcond*//*---End of colorFlib     -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  copyFlib      Copy Flib      
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* copylib      */
void FL_CPY_U8P_U8P(I32 count, U8 *src, U8 *dst);                   /* U8 fast copy function    */
void FL_CPS_U8P_U8P(I32 count, U8 *src, U8 *dst);                   /* U8 slow but simple       */
void FL_CPY_U16P_U16P(U32 count, U16 *src, U16 *dst);               /* U16 copy function        */
void FL_CPY_U32P_U32P(U32 count, U32 *src, U32 *dst);               /* U32 copy function        */
void FL_CPY_ID_U8P_U8P(U32 count, I32 inc, U8 *src, U8 *dst);       /* U8  copy dst increment   */
void FL_CPY_IS_U8P_U8P(U32 count, I32 inc, U8 *src, U8 *dst);       /* U8  copy src increment   */
void FL_CPY_ID_U16P_U16P(U32 count, I32 inc, U16 *src, U16 *dst);   /* U16 copy dst increment   */
void FL_CPY_ID_U32P_U32P(U32 count, I32 inc, U32 *src, U32 *dst);   /* U32 copy dst increment   */
void FL_RVS_U8P_U8P(U32 count, U8 *src, U8 *dst);                   /* U8 reverse elements      */
void FL_EXC_U8P_U8P(U32 count, U8 *src, U8 *dst);                   /* U8 exchange              */
void FL_REXC_U8P_U8P(U32 count, U8 *src, U8 *dst);                  /* U8 exchange and reverse  */
void FL_mem_cpy(I8 *src, I8 *dst, U32 count);                       /* memcopy routine          */

/**@}@endcond*//*---End of copyFlib      -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  dotFlib       Dot Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/
 
/* dotlib       */
I32 FL_DotProduct_I8P_I8P(U32 count, I8 *src1, I8 *src2);           /* vector dotproduct        */
U32 FL_DotProduct_U8P_U8P(U32 count, U8 *src1, U8 *src2);           /* vector dotproduct        */
I32 FL_DotProduct_I16P_I16P(U32 count, I16 *src1, I16 *src2);       /* vector dotproduct        */
U32 FL_DotProduct_U16P_U16P(U32 count, U16 *src1, U16 *src2);       /* vector dotproduct        */
I32 FL_DotProduct_I32P_I32P(U32 Count, I32 *src1, I32 *src2);       /* vector dotproduct        */
U40 FL_DotProduct_U32P_U32P(U32 count, U32 *src1, U32 *src2);       /* vector dotproduct        */
F32 FL_DotProduct_F32P_F32P(U32 count, F32 *src1, F32 *src2);       /* vector dotproduct float  */

/**@}@endcond*//*---End of dotFlib       -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  fftFlib       FFT Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* fftlib       */
void FL_fft2(I32 n, I16 *xy, const I16 *w);                         /* radix 2 FFT              */
void FL_bitrev(I16 *xy, I16 *index, I32 n);                         /* complex bitreversal      */
void FL_scale(U32 count, I16 *src, I32 shift);                      /* complex shift            */
void FL_CMPLX_U8P_U32P(U32 count, U8 *src, U32 *dst, I32 Mean);     /* type conversion complex  */
void FL_CMPLX_I16P_U32P(U32 count, I16 *src, U32 *dst, I32 Mean);   /* type conversion complex  */

/**@}@endcond*//*---End of fftFlib       -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  histoFlib        Histo Flib
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* histo        */
void FL_lookf(U32 count, U8 *src, U8 *dst, U32 table[]);
void FL_lookf_inc(U32 count, U8 *p, U8 *q, U32 inc1, U32 inc2, U8 table[]);
void FL_histo_U8P(U32 n, U8 *p, U16 *t_hist0, U16 *t_hist1, U16 *t_hist2, U16 *t_hist3);
void FL_histo_U8P_X(U32 n, U8 * p, U16 *t0, U16 *t1, U16 *t2, U16 *t3, U16 *t4, U16 *t5, U16 *t6, U16 *t7);
void FL_histo32_U8P(U32 n, U8 *p, U32 *t_hist0, U32 *t_hist1, U32 *t_hist2, U32 *t_hist3);
void FL_hog(I32 count, U8 *mag, U8 *dir, U32 *hist1, U32 *hist2, U32 *hist3, U32 *hist4);
void FL_hog2(I32 count, U8 *mag, U8 *dir, U32 *hist);               /* histogram of gradient    */
void FL_polarhisto(U32 count, U32 *hist, U16 *atan, U16 *norm, U32 *hue, U32 *sat);
void FL_polarhisto2(U32 count, U32 minsat, U32 *hist, U16 *atan, U16 *norm, U32 *hue, U32 *sat);
void FL_block_histo_16x16(I32 nblocks, U8 *dir0, I32 pitch, U8 *hist_line0, U8 *hist_line1);
void FL_MAXHISTO16_U8P(I32 count, U8 *src, U8 *mag, U8 *dir);

/**@}@endcond*//*---End histoFlib        -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  houghFlib        Hough Flib        
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* hough        */
#define TMUL4   (16)                                                /* scaling multiplier       */
#define TSHIFT4 (4)                                                 /* scaling shift            */
#define HShift  (15)                                                /* for sin and cos          */
#define HShift2 (1<<(HShift-1))
#define DEG90   (64)
#define DEG180  (128)
#define MASK180 (DEG180-1)
#define MASK360 (0xFF)
#define abscirc(x, mask)    min((x) & mask, (-(x)) & mask)          /* circular distance            */

void FL_hough0(I32 StartX, I32 EndX, I32 Dphi, I16 *Sin, I32 *posx, /* hough transform mode 0       */
    I32 y, I32 Offs, I32 Dst_pitch, U8 *mSrc, U8 *dSrc, I32 *pDst);
void FL_hough1(I32 StartX, I32 EndX, I32 Dphi, I16 *Sin, I32 *posx, /* hough transform mode 1       */
    I32 y, I32 Offs, I32 Dst_pitch, U8 *mSrc, U8 *dSrc, I32 *pDst);
void FL_hough2(I32 StartX, I32 EndX, I32 Dphi, I16 *Sin, I32 *posx, /* hough transform mode 2       */
    I32 y, I32 Offs, I32 Dst_pitch, U8 *mSrc, U8 *dSrc, I32 *pDst);
void FL_hough3(I32 StartX, I32 EndX, I16 *Sin, I32 *posx,           /* hough transform mode 3       */
    I32 y, I32 Offs, I32 Dst_pitch, U8 *mSrc, I32 *pDst);


void FL_houghs0(I32 SX, I32 EX, I32 Dphi, I32 phi, I16 *sn, I32 *px, /* hough subtraction mode 0    */
    I32 y, I32 Offs, I32 Dst_pitch, U8 *mSrc, U8 *dSrc, I32 *pDst);
void FL_houghs1(I32 SX, I32 EX, I32 Dphi, I32 phi, I16 *sn, I32 *px, /* hough subtraction mode 1    */
    I32 y, I32 Offs, I32 Dst_pitch, U8 *mSrc, U8 *dSrc, I32 *pDst);
void FL_houghs2(I32 SX, I32 EX, I32 Dphi, I32 phi, I16 *sn, I32 *px, /* hough subtraction mode 2    */
    I32 y, I32 Offs, I32 Dst_pitch, U8 *mSrc, U8 *dSrc, I32 *pDst);
void FL_houghs3(I32 SX, I32 EX, I16 *sn, I32 *px, I32 y,             /* hough subtraction mode 3    */
    I32 Offs, I32 Dst_pitch, U8 *mSrc, I32 *pDst);


I32 FL_houghlist0(I32 StartX, I32 EndX, I32 DeltaAngle, I32 phi,    /* line list generation mode 0,1*/
                    I32 *posx, I32 y, U8 *mSrc, U8 *dSrc, I32 *xy);
I32 FL_houghlist2(I32 StartX, I32 EndX, I32 DeltaAngle, I32 phi,    /* line list generation mode 2  */
                    I32 *posx, I32 y, U8 *mSrc, U8 *dSrc, I32 *xy);
I32 FL_houghlist3(I32 StartX, I32 EndX, I32 *posx,                  /* line list generation mode 3  */
                            I32 y, U8 *mSrc,  I32 *xy);

/**@}@endcond*//*---End of houghFlib        -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  integralFlib        Integral Flib
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

void  FL_int_U8P_U32P(I32 count, U8 *restrict src, U32 *restrict dst);
void  FL_int_U8P_U16P(I32 count, U8 *restrict src, U16 *restrict dst);

/**@}@endcond*//*---End of integralFlib -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  jpegFlib      Jpeg Flib      
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* jpeglib      */
#define DCTSIZE2 64
#define DCTSIZE   8
I32  FL_arrng_U8P_U16P(I32 pitch, U8 *src, I16 *dst);               /* copy pixels to 8x8 blocks    */
void FL_YCbYCr_arrng(I32 pitch, U8 *src, I16 *y1,                   /* version for YCbYCr format    */
                             I16 *y2, I16 *cb, I16 *cr);
void FL_dct_I16P(I16 *data);                                        /* discrete cosine transform 8x8*/
//void FL_zigzag(I16 *src, I32 *dst);                                 /* JPEG ZIGZAG                  */
void FL_zigzag(I16 *src, I16 *dst);                                 /* JPEG ZIGZAG                  */
void FL_zagzig(int *src, int *dest);                                /* JPEG ZAGZIG                  */
void divide(I32 count, I16 *num, I32 *den, I16 *result);            /* quantisation                 */

/**@}@endcond*//*---End of jpegFlib      -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  minmaxFlib       Minmax Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* minmax       */
U32 FL_MAX_U8P(U32 count, U8 *src, I32 *index);                     /* vector maximum & index       */
U32 FL_MAX_U16P(U32 count, U16 *src, I32 *index);                   /* vector maximum & index       */
U32 FL_MAX_U32P(U32 count, U32 *src, I32 *index);                   /* vector maximum & index       */
I32 FL_MAX_I32P(U32 count, I32 *src, I32 *index);                   /* vector maximum & index       */
U32 FL_MAXABS_I16P(U32 count, I16 *src, I32 *index);                /* absolute maximum & index     */
U32 FL_MAXABS_I32P(U32 count, I32 *src, I32 *index);                /* absolute maximum & index     */
U32 FL_MIN_U16P(U32 count, U16 *src, I32 *index);                   /* vector minimum & index       */
I32 FL_MAX_I16P(U32 count, I16 *src, I32 *index);                   /* vector maximum & index       */
I32 FL_MIN_I16P(U32 count, I16 *src, I32 *index);                   /* vector minimum & index       */

/**@}@endcond*//*---End of minmaxFlib       -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  movingFlib       Moving Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*  "MovingSum" calculates the moving sum of a vector and stores the result in the destination vector,
*  whereas "avgadd" ADDS the (same) result to the destination vector.
*
* @{
*/

/* moving       */
void FL_MovingSum_U8P_U16P (U32 count, U32 kx, U8  *src, U16 *dst); /* moving sum of vector         */
//void FL_MovingSum_U8P_U32P (U32 count, U32 kx, U8  *src, U32 *dst); /* moving sum of vector         */
void FL_MovingSum_U16P_U16P(U32 count, U32 kx, U16 *src, U16 *dst); /* moving sum of vector         */
void FL_MovingSum_I16P_I16P(U32 count, U32 kx, I16 *src, I16 *dst); /* moving sum of vector         */
void FL_MovingSum_I32P_I32P(U32 count, U32 kx, I32 *src, I32 *dst); /* moving sum of vector         */
void FL_MovingSum_U32P_U32P(U32 count, U32 kx, U32 *src, U32 *dst); /* moving sum of vector         */
void FL_MSQ_U32P_U40P(I32 dx, I32 kx, U32 *src, U40 *dst);          /* moving square                */ 
void FL_avgadd_U8P_U16P(U32 count, U32 kx, U8 *src, U16 *accu);     /* add moving sum to accu       */
void FL_avgadd_U8P_U32P(U32 count, U32 kx, U8 *src, U32 *accu);     /* add moving sum to accu       */
void FL_avgsub_U8P_U16P(U32 count, U32 kx, U8 *src, U16 *accu);     /* sub moving sum from accu     */
void FL_avgsub_U8P_U32P(U32 count, U32 kx, U8 *src, U32 *accu);     /* sub moving sum from accu     */
void FL_asqadd_U8P_U32P(U32 count, U32 kx, U8 *src, U32 *accu);     /* add moving square to accu    */
void FL_asqadd_U8P_U40P(U32 count, U32 kx, U8 *src, U40 *accu);     /* add moving square to accu    */
void FL_asqsub_U8P_U32P(U32 count, U32 kx, U8 *src, U32 *accu);     /* sub moving square from accu  */
void FL_asqsub_U8P_U40P(U32 count, U32 kx, U8 *src, U40 *accu);     /* sub moving square from accu  */

/**@}@endcond*//*---End of movingFlib       -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  mulFlib       Mul Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* mullib       */
void FL_mul_U8P_U16P_U8P(I32 count, U8 *src, U16 *mul, U8 *dst);    /* multiply 2 vectors and shift */
void FL_mul_I16P_I16P_I16P(I32 count, I16 *src, I16 *mul, I16 *dst);/* multiply 2 vectors and shift */
                                                                    /* word alignment / even count !*/
void FL_muls_I32P_I32P_I32P(I32 cnt, I32 *p1,                       /* multiplication of 2 vectors  */
                                I32 *p2, I32 *pr, I32 sh);          /* and shift                    */
void FL_mulx_U16P_U16(U32 count, U16 *src, U16 factor, U16 *dst);   /* multiply vector with const   */
void FL_mulx_I16P_I16(U32 count, I16 *src, I16 factor, I16 *dst);   /* multiply vector with const   */
void FL_mulx2_U16P_U16(U32 count, U16 *src, U16 factor, U8 *dst);   /* multiply vector with const   */
void FL_norm_I16P_U8P(U32 cnt, I32 rnd, I32 sh, I16 *src, U8 *dst); /* round and normalize          */
void FL_MUL_U16P_U16P(I32 count, U16 *src1, U16 *src2, U16 *dst);	/* multiply 2 vectors U16 		*/
I32  FL_MULMAX_U16P_U16P(I32 count, U16 *src1, U16 *src2, I32 *ndx);/* multiply and search maximum	*/

// not necessary void FL_vmul_I16P_I16(U32 count, I16 *src1, I16 *src2, I16 *dst);   /* vector multiply             */

/**@}@endcond*//*---End of mulFlib       -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  opFlib        Op Flib        
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* oplib        */
void FL_add2f(U32 count, U8 *p1, U8 *p2, U8 *pr, I32 sh);           /* add vectors and shift        */
void FL_sub2f(U32 count, U8 *p1, U8 *p2, U8 *pr);                   /* absolute difference          */
void FL_sub2x(U32 count, U8 *p1, U8 *p2, U8 *pr, I32 offs);         /* sutract w. offset & clipping */
void FL_sub2y(U32 count, U8 *p1, U8 *p2, U8 *pr);                   /* subtract and binarize        */
void FL_mul2f(U32 count, U8 *p1, U8 *p2, U8 *pr, I32 sh);           /* multiply and shift           */
void FL_max2f(U32 count, U8 *p1, U8 *p2, U8 *pr);                   /* elementwise maximum          */
void FL_min2f(U32 count, U8 *p1, U8 *p2, U8 *pr);                   /* elementwise minimum          */
void FL_and2f(U32 count, U8 *p1, U8 *p2, U8 *pr);                   /* vector AND                   */
void FL_or2f(U32 count, U8 *p1, U8 *p2, U8 *pr);                    /* vector OR                    */
void FL_xor2f(U32 count, U8 *p1, U8 *p2, U8 *pr);                   /* vector XOR                   */

/**@}@endcond*//*---End of opFlib        -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  plistFlib        Plist Flib        
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* plist        */
void FL_adcalc_U8(U32 count, I32 *xy, U8 *ad_list[], U8 *start, I32 pitch);    /* 8bit address calculation     */
void FL_adcalc_U32(U32 count, I32 *xy, U32 *ad_list[], U32 *start, I32 pitch); /* 32bit address calculation    */
void FL_wpadd_I32(U32 count, I32 *ad_list[], I32 value);                       /* pixel list ADD 32 bit        */
void FL_wpaddc_I32(U32 count, I32 *ad_list[], I32 value);                      /* ADD with clip for v < 0      */
void FL_wpset_U8(U32 count, U8 *ad_list[], I32 value);                         /* pixel list SET  8 bit        */
void FL_wpxor_U8(U32 count, U8 *ad_list[], I32 value);                         /* pixel list XOR  8 bit        */
void FL_wplist_U8(U32 count, U8 *ad_list[], U8 *v_list);                       /* pixel list LIST WRITE 8 bit  */
void FL_rplist_U8(U32 count, U8 *ad_list[], U8 *v_list);                       /* pixel list LIST READ  8 bit  */
void FL_rotate_plist(U32 count, I16 *src_xy,  I16 *dst_xy,                     /* rotate pixel list            */
                            I16 SinAng, I16 CosAng, I32 Shift);

/**@}@endcond*//*---End of plistFlib        -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  recFlib       Rec Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* reclib       */
void FL_GAUSS_U8P_I16P(U32 ncols, U8 *x, U8 *y, I16 *A,             /* recursive gauss filter       */
                        I16 a, I16 b1, I16 b2, I16 b3);
void FL_ISEF_U8P_U8P(U32 ncols, U8 *x, U8 *y, U16 *A,               /* exponential filter           */
                        U16 b0, U16 b1, U16 b2, U16 b3, U16 b4);

/**@}@endcond*//*---End of recFlib       -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  setFlib       Set Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* setlib       */
void FL_Set_U8P_U8(U32 count, U8 *dst, U8 color);                               /* U8  fill buffer  */
void FL_Set_U16P_U16(U32 count, U16 *dst, U16 value);                           /* U16 fill buffer  */
void FL_Set_U32P_U32(U32 count, U32 *dst, U32 value);                           /* U32 fill buffer  */
void FL_SetBin_U8P_U8P(U32 count, U8 *src, U8 *dst, U8 thr, U8 black, U8 white);/* binarize         */
void FL_SetIfEQ_U8P_U8_U8(U32 count, U8 *src, U8 thresh, U8 color);
void FL_SetIfNE_U8P_U8_U8(U32 count, U8 *src, U8 thresh, U8 color);
void FL_SetIfLT_U8P_U8P(U32 count, U8 *src, U8 *dst, U8 thresh, U8 color);
void FL_SetIfLE_U8P_U8_U8(U32 count, U8 *src, U8 thresh, U8 color);
void FL_SetIfGT_U8P_U8_U8(U32 count, U8 *src, U8 thresh, U8 color);
void FL_SetIfGE_U8P_U8P(U32 count, U8 *src, U8 *dst, U8 thresh, U8 color);
void FL_SetIfGELE_U8P_U8_U8_U8(U32 count, U8 *src, U8 low, U8 high, U8 color);
void FL_SetIfLTGT_U8P_U8_U8_U8(U32 count, U8 *src, U8 low, U8 high, U8 color);
void FL_OR_I8P_I8(U32 count, I8 *src, I8 *dst, I8 constant);                    /* OR vector with constant  */

/**@}@endcond*//*---End of setFlib       -------------------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  subFlib       Sub Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* sublib       */
void FL_sub_U8P_U8(U32 count, U8 *src, U8 offset, I16 *dst);        /* subtract constant            */
void FL_sub_U8P_U8P(U32 count, U8 *src1, U8 *src2, U8 *dst);        /* subtract 2 vectors           */
void FL_sub_I16P_I16P(U32 count, I16 *src1, I16 *src2, I16 *dst);   /* subtract 2 vectors           */
void FL_sub_U16P_U16P(U32 count, U16 *src1, U16 *src2, U16 *dst);   /* subtract 2 vectors           */
void FL_sub_I32P_I32P(U32 count, I32 *src1, I32 *src2, I32 *dst);   /* subtract 2 vectors           */
void FL_subc_U8P_I8P_U8P(U32 count, U8 *src1, I8 *src2, U8 *dst);   /* subtract and zero clip       */
void FL_subc_U8P_I32_U8P(U32 count, U8 *src, I32 offset, U8 *dst);  /* subtract constant with clip  */
void FL_absdif_I32P_I32P(U32 cnt, I32 *src1, I32 *src2, I32 *dst);  /* absolute difference          */
U32  FL_SAD_U8P_U8P(U32 count, U8 *src1, U8 *src2);                 /* sum of absolute difference   */

/**@}@endcond*//*---End of subFlib       -------------------------------------------*/

/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  utilLib       util Flib       
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* util			*/
U16 *rlcmkfc(I32 dx, U8 thr, U8 *px, U16 *pr);
void FL_load_rlcmk_buffer(I32 dx);
void FL_unload_rlcmk_buffer();

/**@}@endcond*//*---End of subFlib       -------------------------------------------*/

/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( flib )
* @defgroup  variousFlib    Various  Flib    
* 
*  Functions in this group 
*  @todo weitere Doku
*
* @{
*/

/* functions    */

U32 FL_invsqrt_U32(U32 v);                                          /* inverse square root          */

/* macros       */

#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b))

#define MEMCLR(t, n)    FL_Set_U32P_U32(n, t, 0)                    /* careful: 32bit clear !       */

/**@}@endcond*//*---End of variousFlib    -------------------------------------------*/



#ifdef __cplusplus
}
#endif


#endif  // FLIB_H

/* EOF */

