/**********************************************************************//**
***************************************************************************
*** @file   macros.h
***
*** @brief  Macros which made/makes programmer's life easier.
***
***   Some macro definitions are quite ancient and are only there
***   for compatibility reasons.
***
***
*** @author  Copyright (c) 2004-2014 Vision Components
*** @author  All rights reserved.
*** @author  This software embodies materials and concepts which are
***          confidential to Vision Components.
***
***
*** @revisionHistory
***
***    Date          Version    Author  Changes
***
*** @endRevisionHistory
***************************************************************************
***************************************************************************/
#ifndef  _MACROS__H_h
#define  _MACROS__H_h


/* Constants */
#define   BitsPerByte             ( 8)
#define   BitsPerWord             (32)
#define   BytesPerWord            ( 4)
#define   BytesPerPage            ( 1)


/* Conversion macros */
#define   BitsAsBytes(bits)       ((bits)/(BitsPerByte))
#define   BitsAsWords(bits)       ((bits)/(BitsPerWord))


/* Image Pixel Adressing Macros */
#ifdef OLD_IMAGE_VAR /* Old ImageAssign for old color image variable */
  #define ImageAssign(a,newst,newdx,newdy,newpitch)\
                      {(a)->st   =(long)(newst   );\
                       (a)->dx   =(I32 )(newdx   );\
                       (a)->dy   =(I32 )(newdy   );\
                       (a)->pitch=(I32 )(newpitch);}
#else
  #define ImageAssign(a,newst,newdx,newdy,newpitch)\
                      {(a)->st   =(U8 *)(newst   );\
                       (a)->type = 0              ;\
                       (a)->dx   =(I32 )(newdx   );\
                       (a)->dy   =(I32 )(newdy   );\
                       (a)->pitch=(I32 )(newpitch);\
                       (a)->ccmp1=(U8 *) 0        ;\
                       (a)->ccmp2=(U8 *) 0        ;}
  #define ImageAssignC(a,newst,newtype,newdx,newdy,newpitch,newccmp1,newccmp2)\
                      {(a)->st   =(U8 *)(newst   );\
                       (a)->type =      (newtype );\
                       (a)->dx   =(I32 )(newdx   );\
                       (a)->dy   =(I32 )(newdy   );\
                       (a)->pitch=(I32 )(newpitch);\
                       (a)->ccmp1=(U8 *)(newccmp1);\
                       (a)->ccmp2=(U8 *)(newccmp2);}
#endif
#define   ImagePrintMembers(text,a) print(text);print("st=%ld(0x%lx),dx=%d,dy=%d,pitch=%d\n",(a)->st,(a)->st,(a)->dx,(a)->dy,(a)->pitch)
#ifdef OLD_IMAGE_VAR
  #define ImageAddr(a,x,y)        ((long)((a)->st+(x)+(y)*(a)->pitch))
#else
  #define ImageAddr(a,x,y)        ((U8 *)((a)->st+(x)+(y)*(a)->pitch))
#endif
#define   ImageSetPixel(a,x,y,g)  (*((U8 *)(ImageAddr((a),(x),(y)))) = (U8)(g)) 
#define   ImageGetPixel(a,x,y)    (*((U8 *)(ImageAddr((a),(x),(y)))))



/* screen macros */
#define   ScrGetCaptPage          getvar(CAPT_START)
#define   ScrSetCaptPage(addr)    setvar(CAPT_START,(addr))
#define   ScrGetColumns           getvar(HWIDTH)
#define   ScrGetRows              getvar(VWIDTH)
#define   ScrGetPitch             getvar(VPITCH)
#define   SizeOfScreen            (ScrGetPitch*ScrGetRows)

#ifdef OLD_IMAGE_VAR
  #define ScrGetPhysPage          getvar(CAPT_START)
#else
  #define ScrGetPhysPage          (U8 *)getvar(CAPT_START)
#endif
#define   ScrSetPhysPage(addr)    setvar(DISP_START,(addr));  setvar(CAPT_START,(addr));
#define   ScrGetLogPage           getvar(SCRLOGPAGE)
#define   ScrSetLogPage(addr)     setvar(SCRLOGPAGE,(int)(addr))
#ifdef OLD_IMAGE_VAR
  #define ScrByteAddr(x,y)        ((long)(ScrGetLogPage+(x)+(y)*ScrGetPitch))
#else
  #define ScrByteAddr(x,y)        ((U8 *)(ScrGetLogPage+(x)+(y)*ScrGetPitch))
#endif
#define   ScrSetPixel(x,y,value)  (*((U8 *)(ScrByteAddr(x,y))) = (U8)(value))
#define   ScrGetPixel(x,y)        (*((U8 *)(ScrByteAddr(x,y))))
#define   ScrGetX(addr)           (((U32)(addr)-(U32)ScrByteAddr(0,0))%ScrGetPitch)
#define   ScrGetY(addr)           (((U32)(addr)-(U32)ScrByteAddr(0,0))/ScrGetPitch)


/* Display and Overlay Page mainly for Cameras with VGA Display */
#define   ScrGetDispPage          getvar(DISP_START)
#define   ScrSetDispPage(addr)    setvar(DISP_START,(addr))
#define   DispGetColumns          getvar(DHWIDTH)
#define   DispGetRows             getvar(DVWIDTH)
#define   DispGetPitch            getvar(VPITCH)


#define   OvlGetPhysPage          getvar(OVLY_START)
#define   OvlSetPhysPage(addr)    setvar(OVLY_START,(addr))
#define   OvlGetLogPage           getvar(OVLLOGPAGE)
#define   OvlSetLogPage(addr)     setvar(OVLLOGPAGE,(addr))
#define   OvlGetPitch             ScrGetPitch
#define   SizeOfOverlay           SizeOfScreen

#ifdef OLD_IMAGE_VAR
  #define OvlByteAddr(x,y)        ((long)(OvlGetLogPage+(x)+(y)*OvlGetPitch)) 
#else
  #define OvlByteAddr(x,y)        ((U8 *)(OvlGetLogPage+(x)+(y)*OvlGetPitch)) 
#endif
#define   OvlBitAddr(x,y)         OvlByteAddr(x,y)
#define   OvlSetPixel(x,y,value)  wovl(value,OvlBitAddr((x),(y)))
#define   OvlGetPixel(x,y)        rovl(OvlBitAddr((x),(y)))
#define   OvlGetColumns           ScrGetColumns
#define   OvlGetRows              ScrGetRows
#ifdef OLD_IMAGE_VAR
  #define OvlGetX(addr)           ((long)(addr)-OvlBitAddr(0,0))%OvlGetPitch
  #define OvlGetY(addr)           ((long)(addr)-OvlBitAddr(0,0))/OvlGetPitch
#else
  #define OvlGetX(addr)           ((U8 *)(addr)-OvlBitAddr(0,0))%OvlGetPitch
  #define OvlGetY(addr)           ((U8 *)(addr)-OvlBitAddr(0,0))/OvlGetPitch
#endif
#define   OvlClearAll             {image ovl;  ImageAssign(&ovl,OvlByteAddr(0,0),OvlGetColumns,OvlGetRows,OvlGetPitch);  set(&ovl,0   );};
#define   OvlSetAll               {image ovl;  ImageAssign(&ovl,OvlByteAddr(0,0),OvlGetColumns,OvlGetRows,OvlGetPitch);  set(&ovl,0xff);};


/* Aliases for video-modi */
#define   vmLive                  (0)
#define   vmStill                 (1)
#define   vmLiveRefresh           (2)
#define   vmFreeze                (3)
#define   vmOvlLive               (4)
#define   vmOvlStill              (5)
#define   vmOvlLiveRefresh        (6)
#define   vmOvlFreeze             (7)




/* Old Macros */
#define   ByteAddrAsBitAddr(addr) (addr)
#define   BitAddrAsByteAddr(addr) (addr)

#define   DRAMTYPE                MDATA
#define   DRAMBytesAvail()        DRAMAvail()
#define   DRAMPagesAvail()        DRAMBytesAvail() 
#define   DRAMWordsAvail()        (DRAMBytesAvail()/BytesPerWord)
#ifdef OLD_IMAGE_VAR
  #define DRAMByteMalloc(n)       ((long)sysmalloc(((n)+BytesPerWord-1)/BytesPerWord,DRAMTYPE))
  #define DRAMWordMalloc(n)       DRAMByteMalloc((BytesPerWord*(n))/2)
#else
  #define DRAMByteMalloc(n)       ((U8 *)sysmalloc(((n)+BytesPerWord-1)/BytesPerWord,DRAMTYPE))
  #define DRAMWordMalloc(n)       (U32 *)DRAMByteMalloc((BytesPerWord*(n))/2)
#endif
#define   DRAMPgMalloc(n)         DRAMByteMalloc(n)
#define   DRAMPageMalloc(n)       DRAMByteMalloc(n)
#define   DRAMByteFree(n)         sysfree((void *)(n))
#define   DRAMWordFree(n)         DRAMByteFree((n)*2)
#define   DRAMPgFree(n)           DRAMByteFree(n)
#ifdef OLD_IMAGE_VAR
  #define DRAMScreenMalloc()      ((I32 )sysmalloc(((SizeOfScreen)+1024+BytesPerWord-1)/BytesPerWord,MIMAGE))
  #define DRAMDisplayMalloc()     ((I32 )sysmalloc((DispGetPitch*DispGetRows+1024+BytesPerWord-1)/BytesPerWord,MIMAGE))
#else
  #define DRAMScreenMalloc()      ((U8 *)sysmalloc(((SizeOfScreen)+1024+BytesPerWord-1)/BytesPerWord,MIMAGE))
  #define DRAMDisplayMalloc()     ((U8 *)sysmalloc((DispGetPitch*DispGetRows+1024+BytesPerWord-1)/BytesPerWord,MIMAGE))
#endif
#define   DRAMOvlMalloc()         DRAMScreenMalloc()



#define   KEY_INVALID             (-1)
#define   KEY_ENTER               ( 0)
#define   KEY_ESC                 ( 1)
#define   KEY_F1                  ( 2)
#define   KEY_F2                  ( 3)
#define   KEY_UP                  ( 4)
#define   KEY_DOWN                ( 5)
#define   KEY_RIGHT               ( 6)
#define   KEY_LEFT                ( 7)

#ifndef getchar
  #define getchar                 rs232rcv
#endif
#ifndef putchar
  #define putchar                 rs232snd
#endif
#define   kbhit()                 (-1 != rbempty())




#endif //_MACROS__H_h
/* EOF */
