/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2000-2010 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: register.h
***                                                            
*** Comments: 
***		system register header file
***
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 10, 2002  5.00       ME      Initial Version
***	   Jul 19, 2004	 5.17		ME		cosmetic update
***	   Sep 17, 2004	 5.17		ME		additional registers for C64
***    Jul 22, 2005	 5.21		MS		added CACHE_LINE_SIZE
***    Aug 02, 2005  5.22       MS      added SDRAMCLK for VC40XX family
***    Dec 02, 2005  5.23       MS      added new REGS for VC40XX family
***    Sep 27, 2006  5.25       MS      added new REGS for VC44XX family
***    Jan 30, 2009  5.28       MS      added new REGS for VC64XX family
***    Apr 30, 2010  5.29       MS      added new Bits for VC64XX family
***
***************************************************************************
*END***********************************************************************/
/* C6211 hardware registers     */

/* address mapping:

   C6DAC / VIRTEX           CE0     0x80000000
   onboard FLASH 2M         CE1     0x90000000
   onboard SDRAM 16/64M     CE2     0xA0000000
   adapter board LEDs       CE3     0xB0000000
   D0-D7 program byte for VIRTEX

*/
#ifndef register_h
#define register_h

#define     SDRAM16MB       /* 16 MBytes SDRAM  */

#ifdef DM648
#define VIRTEX_BASE                 0xB0000000		/* CE3					*/
#define EMIF_CE3_space_control      0x70000084     /* CE3CFG            */
#elif defined DM6435
#undef  VIRTEX_BASE
#else
#define VIRTEX_BASE                 0x80000000		/* CE0					*/
#endif
#define FLASH_BASE                  0x90000000		/* CE1					*/
#define FLASH_BASE_B                0x64000000		/* CE1 EMIFB			*/
#define SDRAM_BASE                  0xA0000000		/* CE2					*/
#define ADAPT_BASE                  0xB0000000		/* CE3					*/
#define FLASH_BASE_C                0xA0000000		/* CE2 EMIFA DM648  	*/
#define FLASH_BASE_D                0x42000000		/* CE2 EMIFA DM6435 	*/

/*
00 Video Output
04
08
0c
*/

#if !defined DM648 && !defined DM6435
#define EMIF_global_control         0x01800000
#define EMIF_CE1_space_control      0x01800004
#define EMIF_CE0_space_control      0x01800008
#define EMIF_CE2_space_control      0x01800010
#define EMIF_CE3_space_control      0x01800014
#define EMIFB_global_control        0x01A80000
#define EMIFB_CE1_space_control     0x01A80004

#define EMIF_SDRAM_control          0x01800018      /* SDRAM control reg.   */
#define EMIF_SDRAM_timing_register  0x0180001C      /* SDRAM timing register*/
#define EMIF_SDRAM_ext_register     0x01800020      /* SDRAM extension reg. */

#define IMCR_CCFG                   0x01840000      /* cache configuration  */
#define IMCR_L2FBAR                 0x01844000      /* L2 flush base        */
#define IMCR_L2FWC                  0x01844004      /* L2 flush wordcount   */
#define IMCR_L2CBAR                 0x01844010      /* L2 clean base address*/
#define IMCR_L2CWC                  0x01844014      /* L2 clean wordcount   */
#define IMCR_L1PFBAR                0x01844020      /* L1P flush base       */
#define IMCR_L1PFWC                 0x01844024      /* L1P flush wordcount  */
#define IMCR_L1DFBAR                0x01844030      /* L1D flush base       */
#define IMCR_L1DFWC                 0x01844034      /* L1D flush wordcount  */
#define IMCR_L2FLUSH                0x01845000      /* L2 flush register    */
#define IMCR_L2CLEAN                0x01845004      /* L2 clean register    */
#define IMCR_MAR0                   0x01848200      /* CE0 0x8000 0000      */
#define IMCR_MAR1                   0x01848204      /* CE0 0x8100 0000      */
#define IMCR_MAR2                   0x01848208      /* CE0 0x8200 0000      */
#define IMCR_MAR3                   0x0184820C      /* CE0 0x8300 0000      */
#define IMCR_MAR4                   0x01848240      /* CE1 0x9000 0000      */
#define IMCR_MAR5                   0x01848244      /* CE1 0x9100 0000      */
#define IMCR_MAR6                   0x01848248      /* CE1 0x9200 0000      */
#define IMCR_MAR7                   0x0184824C      /* CE1 0x9300 0000      */
#define IMCR_MAR8                   0x01848280      /* CE2 0xA000 0000      */
#define IMCR_MAR9                   0x01848284      /* CE2 0xA100 0000      */
#define IMCR_MAR10                  0x01848288      /* CE2 0xA200 0000      */
#define IMCR_MAR11                  0x0184828C      /* CE2 0xA300 0000      */
#define IMCR_MAR12                  0x018482C0      /* CE3 0xB000 0000      */
#define IMCR_MAR13                  0x018482C4      /* CE3 0xB100 0000      */
#define IMCR_MAR14                  0x018482C8      /* CE3 0xB200 0000      */
#define IMCR_MAR15                  0x018482CC      /* CE3 0xB300 0000      */

/* 11.07.2005 introduced CACHE_LINE_SIZE (L2-cache) */
#define CACHE_LINE_SIZE             (128)

#define MBSP_DRR0                   0x018C0000      /* Data Receive Reg.    */
#define MBSP_DR20                   0x30000000      /* Data Receive Reg.    */
#define MBSP_DXR0                   0x018C0004      /* Data Transmit Reg.   */
#define MBSP_DX20                   0x30000000      /* Data Transmit Reg.   */
#define MBSP_SPCR0                  0x018C0008      /* control reg.         */
#define MBSP_RCR0                   0x018C000C      /* receive control      */
#define MBSP_XCR0                   0x018C0010      /* transmit control     */
#define MBSP_SRGR0                  0x018C0014      /* sample rate generator*/
#define MBSP_MCR0                   0x018C0018      /* multichannel control */
#define MBSP_RCER0                  0x018C001C      /* rcv channel enable   */
#define MBSP_XCER0                  0x018C0020      /* xmt channel enable   */
#define MBSP_PCR0                   0x018C0024      /* pin control register */

#define MBSP_DRR1                   0x01900000      /* Data Receive Reg.    */
#define MBSP_DXR1                   0x01900004      /* Data Transmit Reg.   */
#define MBSP_SPCR1                  0x01900008      /* control reg.         */
#define MBSP_RCR1                   0x0190000C      /* receive control      */
#define MBSP_XCR1                   0x01900010      /* transmit control     */
#define MBSP_SRGR1                  0x01900014      /* sample rate generator*/
#define MBSP_MCR1                   0x01900018      /* multichannel control */
#define MBSP_RCER1                  0x0190001C      /* rcv channel enable   */
#define MBSP_XCER1                  0x01900020      /* xmt channel enable   */
#define MBSP_PCR1                   0x01900024      /* pin control register */

#define TIMR_control1               0x01980000      /* timer 1 control      */
#define TIMR_period1                0x01980004      /* timer 1 period       */
#define TIMR_counter1               0x01980008      /* timer 1 counter      */
#define TIMR_control0               0x01940000      /* timer 0 control      */
#define TIMR_period0                0x01940004      /* timer 0 period       */
#define TIMR_counter0               0x01940008      /* timer 0 counter      */

#else
/* DM648 / DM6435 Cache */
#define L2CFG    0x01840000 // L2 cache configuration register
#define L1PCFG   0x01840020 // L1P size cache configuration register
#define L1PCC    0x01840024 // L1PCC L1P freeze mode cache configuration register
#define L1DCFG   0x01840040 // L1D size cache configuration register
#define L1DCC    0x01840044 // L1D freeze mode cache configuration register
#define L2ALLOC0 0x01842000 // L2 allocation register 0
#define L2ALLOC1 0x01842004 // L2 allocation register 1
#define L2ALLOC2 0x01842008 // L2 allocation register 2
#define L2ALLOC3 0x0184200C // L2 allocation register 3
#define L2WBAR   0x01844000 // L2 writeback base address register
#define L2WWC    0x01844004 // L2 writeback word count register
#define L2WIBAR  0x01844010 // L2 writeback invalidate base address register
#define L2WIWC   0x01844014 // L2 writeback invalidate word count register
#define L2IBAR   0x01844018 // L2 invalidate base address register
#define L2IWC    0x0184401C // L2 invalidate word count register
#define L1PIBAR  0x01844020 // L1P invalidate base address register
#define L1PIWC   0x01844024 // L1P invalidate word count register
#define L1DWIBAR 0x01844030 // L1D writeback invalidate base address register
#define L1DWIWC  0x01844034 // L1D writeback invalidate word count register
#define L1DWBAR  0x01844040 // L1D block writeback
#define L1DWWC   0x01844044 // L1D block writeback
#define L1DIBAR  0x01844048 // L1D invalidate base address register
#define L1DIWC   0x0184404C // L1D invalidate word count register
#define L2WB     0x01845000 // L2 writeback all register
#define L2WBINV  0x01845004 // L2 writeback invalidate all register
#define L2INV    0x01845008 // L2 global invalidate without writeback
#define L1PINV   0x01845028 // L1P global invalidate
#define L1DWB    0x01845040 // L1D global writeback
#define L1DWBINV 0x01845044 // L1D global writeback with invalidate
#define L1DINV   0x01845048 // L1D global invalidate without writeback

// 
// DM6435
//

#define MAR128 0x01848200 // Controls EMIFA CE0 range 8000 0000 - 80FF FFFF
#define MAR129 0x01848204 // Controls EMIFA CE0 range 8100 0000 - 81FF FFFF
#define MAR130 0x01848208 // Controls EMIFA CE0 range 8200 0000 - 82FF FFFF
#define MAR131 0x0184820C // Controls EMIFA CE0 range 8300 0000 - 83FF FFFF
#define MAR132 0x01848210 // Controls EMIFA CE0 range 8400 0000 - 84FF FFFF
#define MAR133 0x01848214 // Controls EMIFA CE0 range 8500 0000 - 85FF FFFF
#define MAR134 0x01848218 // Controls EMIFA CE0 range 8600 0000 - 86FF FFFF
#define MAR135 0x0184821C // Controls EMIFA CE0 range 8700 0000 - 87FF FFFF

// 
// DM648
//

// MAR0-MAR15 read only - MAR0 used as base register for MAR regs
#define MAR0   0x01848000 
// MAR224-MAR239 Memory attribute registers for DDR2 0xE0000000 - 0xEFFFFFFF
#define MAR224 0x01848380 
#define MAR225 0x01848384 
#define MAR226 0x01848388 
#define MAR227 0x0184838C 
#define MAR228 0x01848390 
#define MAR229 0x01848394 
#define MAR230 0x01848398 
#define MAR231 0x0184839C 
#define MAR232 0x018483A0 
#define MAR233 0x018483A4 
#define MAR234 0x018483A8 
#define MAR235 0x018483AC
#define MAR236 0x018483B0 
#define MAR237 0x018483B4 
#define MAR238 0x018483B8 
#define MAR239 0x018483BC 
// MAR240-MAR255 Memory attribute registers for DDR2 0xF0000000 - 0xFFFFFFFF
#define MAR240 0x018483C0 
#define MAR241 0x018483C4 
#define MAR242 0x018483C8 
#define MAR243 0x018483CC 
#define MAR244 0x018483D0 
#define MAR245 0x018483D4 
#define MAR246 0x018483D8 
#define MAR247 0x018483DC 
#define MAR248 0x018483E0 
#define MAR249 0x018483E4 
#define MAR250 0x018483E8 
#define MAR251 0x018483EC 
#define MAR252 0x018483F0 
#define MAR253 0x018483F4 
#define MAR254 0x018483F8 
#define MAR255 0x018483FC 

#define CACHE_LINE_SIZE             (128)

#define IMCR_CCFG    L2CFG    /* L2 cache configuration  */
#define IMCR_L2FBAR  L2WBAR   /* L2 flush base        */
#define IMCR_L2FWC   L2WWC    /* L2 flush wordcount   */
#define IMCR_L2CBAR  L2WIBAR  /* L2 clean base address*/
#define IMCR_L2CWC   L2WIWC   /* L2 clean wordcount   */
#define IMCR_L1PFBAR L1PIBAR  /* only invalidate - L1P flush base       */
#define IMCR_L1PFWC  L1PIWC   /* only invalidate - L1P flush wordcount  */
#define IMCR_L1DFBAR L1DWBAR  /* L1D flush base       */
#define IMCR_L1DFWC  L1DWWC   /* L1D flush wordcount  */
#define IMCR_L2FLUSH L2WB     /* L2 flush register    */
#define IMCR_L2CLEAN L2WBINV  /* L2 clean register    */
#define IMCR_L2IBAR  L2IBAR   /* L2 invalidate base	  */	
#define IMCR_L2IWC   L2IWC    /* L2 inval. wordcount  */
#define IMCR_L1DIBAR L1DIBAR  /* L1D invalidate base  */
#define IMCR_L1DIWC	 L1DIWC   /* L1D inval. wordcount */	

#endif


/* MTYPE

   0000b    8  bit asynchronous
   0001b    16 bit asynchronous
   0010b    32 bit asynchronous
   0011b    32 bit SDRAM

*/

#ifdef DM648

#define VIRTX_STAT        (VIRTEX_BASE+0x0000)/* status register        (r)   */
#define VIRTX_CTRL        (VIRTEX_BASE+0x8000)/* virtex control register(w)   */
#define VIRTX_INT         (VIRTEX_BASE+0x8000)/* interrupt register     (r)   */
#define VIRTX_CTRL2       (VIRTEX_BASE+0x8400)/* virtex control reg 2   (w)   */
#define VIRTX_PDEB        (VIRTEX_BASE+0xC000)/* PLC input debounce     (w)   */
/* channel A */
#define VIRTX_A_ENCSTAT   (VIRTEX_BASE+0x10000)/* encoder status reg    (r)   */
#define VIRTX_A_ENCLB     (VIRTEX_BASE+0x18000)/* encoder lightbarrier  (r)   */
#define VIRTX_A_ENCCOMP1  (VIRTEX_BASE+0x5000)/* encoder comparator 1   (w)   */
#define VIRTX_A_ENCCOMP2  (VIRTEX_BASE+0x5004)/* encoder comparator 2   (w)   */
#define VIRTX_A_ENCCOMP3  (VIRTEX_BASE+0x5008)/* encoder comparator 3   (w)   */
#define VIRTX_A_ENCCTRL   (VIRTEX_BASE+0x4C00)/* virtex control register(w)   */
#define VIRTX_A_ENCCTRL2  (VIRTEX_BASE+0x500C)/* virtex control reg 2   (w)   */
#define VIRTX_A_ILUT      (VIRTEX_BASE+0x0000)/* camera input LUT B/W   (w)   */
#define VIRTX_A_EXPV      (VIRTEX_BASE+0x4000)/* exposure value register(w)   */
#define VIRTX_A_ADCP      (VIRTEX_BASE+0x4400)/* ADC register address   (w)   */
#define VIRTX_A_PIX_START (VIRTEX_BASE+0x4800)/* pix start register     (w)   */
/* channel B */
#define VIRTX_B_ENCSTAT   (VIRTEX_BASE+0x20000)/* encoder status reg    (r)   */
#define VIRTX_B_ENCLB     (VIRTEX_BASE+0x28000)/* encoder lightbarrier  (r)   */
#define VIRTX_B_ENCCOMP1  (VIRTEX_BASE+0x6400)/* encoder comparator 1   (w)   */
#define VIRTX_B_ENCCOMP2  (VIRTEX_BASE+0x6404)/* encoder comparator 2   (w)   */
#define VIRTX_B_ENCCOMP3  (VIRTEX_BASE+0x6408)/* encoder comparator 3   (w)   */
#define VIRTX_B_ENCCTRL   (VIRTEX_BASE+0x6000)/* virtex control register(w)   */
#define VIRTX_B_ENCCTRL2  (VIRTEX_BASE+0x640C)/* virtex control reg 2   (w)   */
#define VIRTX_B_ILUT      (VIRTEX_BASE+0x2000)/* camera input LUT B/W   (w)   */
#define VIRTX_B_EXPV      (VIRTEX_BASE+0x5400)/* exposure value register(w)   */
#define VIRTX_B_ADCP      (VIRTEX_BASE+0x5800)/* ADC register address   (w)   */
#define VIRTX_B_PIX_START (VIRTEX_BASE+0x5C00)/* pix start register     (w)   */

#define TRIG_A_int_ena    0x80000000          /* Trigger input interrupt ena. */
#define TRIG_A_OUT        0x00000800          /* Trigger output signal        */
#define TRIG_A_out_pol    0x00000400          /* Trigger output polarity      */
#define TRIG_A_out_exp    0x00000200          /* Trigger output = exposure    */
#define TRIG_A_inp_sticky 0x00000100          /* Sticky trigger mode          */
#define TRIG_A_inp_pol    0x00800000          /* Trigger input polarity       */

#define TRIG_B_int_ena    0x01000000          /* Trigger input interrupt ena. */
#define TRIG_B_OUT        0x00000080          /* Trigger output signal        */
#define TRIG_B_out_pol    0x00000040          /* Trigger output polarity      */
#define TRIG_B_out_exp    0x00000020          /* Trigger output = exposure    */
#define TRIG_B_inp_sticky 0x00000010          /* Sticky trigger mode          */
#define TRIG_B_inp_pol    0x00400000          /* Trigger input polarity       */

#define PLC_int_ena     0x00000010            /* PLC interrupt enable         */
#define IOPORT_int_ena  0x00020000            /* IOPORT interrupt enable      */

#define TRIG_int_ena    0x81000000            /* Trigger input interrupt ena. */
#define TRIG_OUT        0x00000880            /* Trigger output signal        */
#define TRIG_out_pol    0x00000440            /* Trigger output polarity      */
#define TRIG_out_exp    0x00000220            /* Trigger output = exposure    */
#define TRIG_inp_sticky 0x00000110            /* Sticky trigger mode          */
#define TRIG_inp_pol    0x00C00000            /* Trigger input polarity       */

#else

#ifndef DM6435

#define     GCNTL   0x00003078              /* EMIF global control default  */
#define     ASYNC   0x1091C221              /* VIRTEX ASYNC access          */

#define VIRTX_CLUT      (VIRTEX_BASE+0x0000)/* display LUT            (w)   */
#define VIRTX_OLUT      (VIRTEX_BASE+0x0400)/* overlay LUT            (w)   */
#define VIRTX_ILUT      (VIRTEX_BASE+0x0800)/* camera input LUT B/W   (w)   */
#define VIRTX_ILUT_R    (VIRTEX_BASE+0x0800)/* camera input LUT RED   (w)   */
#define VIRTX_ILUT_GR   (VIRTEX_BASE+0x0A00)/* camera input LUT GREEN_RED  (w)   */
#define VIRTX_ILUT_GB   (VIRTEX_BASE+0x0C00)/* camera input LUT GREEN_BLUE (w)   */
#define VIRTX_ILUT_B    (VIRTEX_BASE+0x0E00)/* camera input LUT BLUE  (w)   */
#define VIRTX_CTRL      (VIRTEX_BASE+0x2000)/* virtex control register(w)   */
#define VIRTX_CTRL2     (VIRTEX_BASE+0x2004)/* virtex control reg 2   (w)   */
#ifdef OLD
#define VIRTX_CMDR      (VIRTEX_BASE+0x2004)/* virtex command register(w)   */
#endif
#define VIRTX_ADDR      (VIRTEX_BASE+0x2800)/* write address          (w)   */
#define VIRTX_DATA      (VIRTEX_BASE+0x2c00)/* write data             (w)   */
#define VIRTX_STAT      (VIRTEX_BASE+0x2000)/* status register        (r)   */
#define VIRTX_STCL      (VIRTEX_BASE+0x2044)/* status reg. / int. clr.(r)   */
#define VIRTX_RINP      (VIRTEX_BASE+0x1000)/* RS232 input register   (r)   */

#ifndef VC4XXX
  #define VIRTX_PINP     (VIRTEX_BASE+0x1004)/* PLC input register     (r) 	*/
#else
  #define VIRTX_PINP    (VIRTEX_BASE+0x1200)/* PLC input register     (r)   */
  #define VIRTX_PDEB    (VIRTEX_BASE+0x1C00)/* PLC input debounce     (w)   */
#endif

#define VIRTX_ENCSTAT   (VIRTEX_BASE+0x1004)/* encoder status reg     (r)   */
#define VIRTX_ENCLB     (VIRTEX_BASE+0x1008)/* encoder lightbarrier   (r)   */
#define VIRTX_ENCCOMP1  (VIRTEX_BASE+0x3400)/* encoder comparator 1   (w)   */
#define VIRTX_ENCCOMP2  (VIRTEX_BASE+0x3404)/* encoder comparator 2   (w)   */
#define VIRTX_ENCCOMP3  (VIRTEX_BASE+0x3408)/* encoder comparator 3   (w)   */
#define VIRTX_ENCCTRL   (VIRTEX_BASE+0x1008)/* virtex control register(w)   */
#define VIRTX_ENCCTRL2  (VIRTEX_BASE+0x340C)/* virtex control reg 2   (w)   */

#define VIRTX_ROUT      (VIRTEX_BASE+0x1000)/* RS232 output register  (w)   */
#define VIRTX_BAUD      (VIRTEX_BASE+0x1004)/* RS232 baud rate reg.r  (w)   */

//#ifndef VC4XXX
  #define VIRTX_POUT    (VIRTEX_BASE+0x1008)/* PLC output register    (w)   */
//#else
  #define VIRTX_TDIV    (VIRTEX_BASE+0x1008)/* 24 bit trigger divider (w)   */
/* MS 12.12.2006 PICLINE for reading which lines are already transfered     */
  #define VIRTX_PICLINE (VIRTEX_BASE+0x100C)/* bit 10:0  line    nr   (r)   */
                                            /* bit 15:11 picture nr   (r)   */
  #define VIRTX_TCNT    (VIRTEX_BASE+0x1204)/* 24 bit trigger counter (r)   */
//#endif

#define VIRTX_EXPV      (VIRTEX_BASE+0x1400)/* exposure value register(w)   */
#define VIRTX_KBIN      (VIRTEX_BASE+0x1044)/* keypad in register     (r)   */

#ifdef OLD
  #define VIRTX_DODD    (VIRTEX_BASE+0x308C)/* camera input odd  data (r)   */
  #define VIRTX_DEVN    (VIRTEX_BASE+0x328C)/* camera input even data (r)   */
#endif

#define VIRTX_DODD      (VIRTEX_BASE+0x3148)/* camera input odd  data (r)   */
#define VIRTX_DEVN      (VIRTEX_BASE+0x3348)/* camera input even data (r)   */
#define VIRTX_CAMB      (VIRTEX_BASE+0x3000)/* camera input dest-addr (w)   */
#define VIRTX_ADCP      (VIRTEX_BASE+0x1800)/* ADC register address   (w)   */

//MS 27.06.2006
#define VIRTX_PIX_START (VIRTEX_BASE+0x1804)/* pix start register     (w)   */

/* VIRTEX DMA register defines												*/

#define VIRTX_DISP		(VIRTEX_BASE+0x3000)/* display  base address  (w)	*/
#define VIRTX_TXD		(VIRTEX_BASE+0x3004)/* transmit base address  (w)	*/
#define VIRTX_RXD		(VIRTEX_BASE+0x3008)/* receive  base address  (w)	*/
#define VIRTX_CAM		(VIRTEX_BASE+0x300C)/* capture  base address  (w)	*/   

/* VIRTEX ETHERNET defines													*/

#define VIRTX_ETST      (VIRTEX_BASE+0x0000)/* status reg. / ethernet (r)	*/
#define VIRTX_ETCL      (VIRTEX_BASE+0x0044)/* status reg. / int. clr.(r)   */
#define VIRTX_ETAD		(VIRTEX_BASE+0x1C04)/* MAC address			  (w)   */
#define VIRTX_TXDD 		(VIRTEX_BASE+0x100C)/* Data output FIFO 			*/
#define VIRTX_TXLN 		(VIRTEX_BASE+0x100C)/* Tx Frame length / DMA version*/
#define VIRTX_TXDL 		(VIRTEX_BASE+0x101C)/* FIFO last access word		*/

/* SDCARD defines MS 20.03.2007 */

#define VIRTX_SD_RCVD   (VIRTEX_BASE+0x0400)/* SD Card read data       (r)  */
#define VIRTX_SD_TRAD   (VIRTEX_BASE+0x2800)/* SD Card write data      (w)  */
#define VIRTX_SD_CMD1   (VIRTEX_BASE+0x2400)/* SD Card cmd1            (w)  */
#define VIRTX_SD_CMD2   (VIRTEX_BASE+0x2404)/* SD Card cmd2            (w)  */
#define VIRTX_SD_STAT   (VIRTEX_BASE+0x0C00)/* SD Card status/response (r)  */
#define VIRTX_SD_RESP   (VIRTEX_BASE+0x0800)/* SD Card response        (r)  */


/* VIRTX_CTRL register bit defines                                          */

#define RTS_OUT         0x00000100          /* RS232 RTS output             */
#define DISP_int_ena    0x00000200          /* Display interrupt enable     */
#define PLC_int_ena     0x00000400          /* PLC interrupt enable         */
#define TRIG_int_ena    0x00000800          /* Trigger input interrupt ena. */
#define TRIG_OUT        0x00001000          /* Trigger output signal        */
#define TRIG_out_pol    0x00002000          /* Trigger output polarity      */
#define TRIG_out_exp    0x00004000          /* Trigger output = exposure    */
#define TRIG_inp_sticky 0x00008000          /* Sticky trigger mode          */
#define TRIG_inp_pol    0x00010000          /* Trigger input polarity       */
#define MEMORY256       0x00020000          /* 256 MByte mode for memory    */
#define KILL_DMA        0x00080000          /* Kill image ackquisition      */
#define DMAenable       0x00080000          /* DMA enable      				*/

#define MDO        		0x00800000          /*       						*/
#define RXreset        	0x00400000          /*       						*/
#define TXreset        	0x00200000          /*       						*/
#define CAMreset        0x00100000          /*       						*/
#define DMAen        	0x00080000          /* see above: KILL_DMA ??		*/
#define LUTnorm         0x00040000          /* LUT normal / translucent		*/
#define MDC         	0x00000100          /* see above: RTS_OUT			*/
#define ENC_use_SPSin   0x02000000          /* use SPS signal 0 for encoder */  

/* VIRTX_CTRL2 register bit defines                                         */

#define SD_CLK_ENA		0x00000010			/* SD clock enable              */
#define SD_RESET		0x00000008			/* reset SD card controller     */
#define SD_CLK_25M		0x00000004			/* SD clock = 25Mhz             */
#define SD_CLK_50M		0x00000002			/* SD clock = 50Mhz             */
#define ENC_LB_INT_ENA	0x00000001			/* enable encoder lb interrupt  */

/*  CAM_enc_ctrl2 register defines for encoder control						*/

#define invert_SPS_LB	0x00000001			/* polarity of SPSin0 / encoder */

/*  CAM_enc_ctrl2 register defines for encoder control						*/



#endif

/* SBC4000 / C6210 CTRL register bit defines								*/

#ifndef DM648 // DM648 platform bits are different!
#define TRIG_int_ena    0x00000800          /* Trigger input interrupt ena. */
#define TRIG_OUT        0x00001000          /* Trigger output signal        */
#define TRIG_out_pol    0x00002000          /* Trigger output polarity      */
#define TRIG_out_exp    0x00004000          /* Trigger output = exposure    */
#define TRIG_inp_sticky 0x00008000          /* Sticky trigger mode          */
#define TRIG_inp_pol    0x00010000          /* Trigger input polarity       */
#endif
#define HIGH_binn       0x80000000          /* high speed binning  VC4008   */
#define ILLU_pol        0x00000008          /* illumination output polarity */
#define ILLU_usr        0x00000004          /* illumination output user 	*/
#define TRIG_inp_usr    0x00000002          /* trigger input user 			*/
#define BOOST			0x00000001			/* boost signal					*/


#ifdef VC44XX
/* VIRTX asynchronous register timing                                       */
#define     VIRTX_wr_setup      6           /* < 16     */
#define     VIRTX_wr_strobe     1           /* < 64     */
#define     VIRTX_wr_hold       0           /* < 4     */
#define     VIRTX_rd_setup      3           /* < 16     */
#define     VIRTX_rd_strobe     1           /* < 63     */
#define     VIRTX_rd_hold       0           /* < 8      */
#define     VIRTX_TA            1           /* < 4      */

#elif defined VC40XX
/* VIRTX asynchronous register timing                                       */
#define     VIRTX_wr_setup      4           /* < 16     */
#define     VIRTX_wr_strobe     1           /* < 64     */
#define     VIRTX_wr_hold       0           /* < 4     */
#define     VIRTX_rd_setup      3           /* < 16     */
#define     VIRTX_rd_strobe     1           /* < 63     */
#define     VIRTX_rd_hold       0           /* < 8      */
#define     VIRTX_TA            1           /* < 4      */

#else 
/* VIRTX asynchronous register timing                                       */
/* parameters for read and write must be identical due to processor error	*/
/* VIRTX_wr_setup may in some cases be 1 although programmed differently	*/

#define     VIRTX_wr_setup      3           /* < 16     */
#define     VIRTX_wr_strobe     1           /* < 64     */
#define     VIRTX_wr_hold       0           /* < 4     */
#define     VIRTX_rd_setup      3           /* < 16     */
#define     VIRTX_rd_strobe     1           /* < 63     */
#define     VIRTX_rd_hold       0           /* < 8      */
#define     VIRTX_TA            1           /* < 4      */

#endif

#define     VIRTX   0x00000020 +                \
                    (VIRTX_wr_setup << 28)  +   \
                    (VIRTX_wr_strobe << 22) +   \
                    (VIRTX_wr_hold << 20)   +   \
                    (VIRTX_rd_setup << 16)  +   \
                    (VIRTX_rd_strobe << 8)  +   \
                    (VIRTX_rd_hold << 0)    +   \
                    (VIRTX_TA << 14)
#endif

/* FLASH asynchronous access timing                                         */
/* parameters for read and write must be identical due to processor error	*/
/* FLASH_wr_setup may in some cases be 1 although programmed differently	*/

#define     FLASH_wr_setup      5           /* < 16     */
#define     FLASH_wr_strobe     12          /* < 64     */
#define     FLASH_wr_hold       2           /* < 4     */
#define     FLASH_rd_setup      5           /* < 16     */
#define     FLASH_rd_strobe     12          /* < 63     */
#define     FLASH_rd_hold       2           /* < 8      */
#define     FLASH_TA            3           /* < 4      */


#define     FLASH   0x00000010 +                \
                    (FLASH_wr_setup << 28)  +   \
                    (FLASH_wr_strobe << 22) +   \
                    (FLASH_wr_hold << 20)   +   \
                    (FLASH_rd_setup << 16)  +   \
                    (FLASH_rd_strobe << 8)  +   \
                    (FLASH_rd_hold << 0)    +   \
                    (FLASH_TA << 14)

#if defined FLASH_32MB
  #define     FLASH_size          0x02000000  /* size = 32 MBytes         */
#elif defined FLASH_16MB
  #define     FLASH_size          0x01000000  /* size = 16 MBytes         */
#elif defined FLASH_8MB
  #define     FLASH_size          0x00800000  /* size = 8 MBytes          */
#elif ( defined (SBC4000) && !defined (VC4018EB) )  || defined (VC4XXX)
  #define     FLASH_size          0x00400000  /* size = 4 MBytes          */
#else 
  #define     FLASH_size          0x00200000  /* size = 2 MBytes          */
#endif

#if defined DM648 || defined DM6435
#define     FLASH_secsize       0x00020000  /* 128k   sector size       */
#else
#define     FLASH_secsize       0x00010000  /* normal sector size       */
#endif
//#define     FLASH_system        2           /* start of system sectors  */
//#define     FLASH_user          6           /* start of user sectors    */
#define     FLASH_system        8           /* start of system sectors  */
#define     FLASH_user          16          /* start of user sectors    */

#define     SDRAM   0x00000030L

#ifdef VC44XX
  /* ECLKOUT frequency: 125.00 MHz -> p =  8.0 nsec               */
  #define     SDRAMCLK            125000000   /* 125.00 MHz       */
#elif defined VC40XX
  /* ECLKOUT frequency: 100.00 MHz -> p = 10.0 nsec               */
  #define     SDRAMCLK            100000000   /* 100.00 MHz       */
#else
  /* ECLKOUT frequency: 93.75 MHz -> p = 10.7 nsec                */
  #define     SDRAMCLK            93750000    /* 93.75 MHz        */
#endif

#define     SDRAM_TRC           8           /* tRC/p  - 1, < 16 */
#define     SDRAM_TRP           2           /* tRP/p  - 1, < 16 */
#define     SDRAM_TRCD          2           /* tRCD/p - 1, < 16 */
#define     SDRAM_INIT          1           /* 0,1              */
#define     SDRAM_RFEN          1           /* 0,1              */
#define     SDRAM_SDBSZ         1           /* 4 banks          */

#ifdef      SDRAM16MB                       /* 16 MBytes SDRAM  */
  #define   SDRAM_SDCSZ         1           /* 256 cols         */
  #define   SDRAM_SDRSZ         1           /* 4096 rows        */
#else                                       /* 64 MBytes SDRAM  */
  #define   SDRAM_SDCSZ         0           /* 512 cols         */
  #define   SDRAM_SDRSZ         2           /* 8192 rows        */
#endif

#define     SDRAM_CNTRL     (SDRAM_SDBSZ << 30)     +   \
                            (SDRAM_SDRSZ << 28)     +   \
                            (SDRAM_SDCSZ << 26)     +   \
                            (SDRAM_RFEN  << 25)     +   \
                            (SDRAM_INIT  << 24)     +   \
                            (SDRAM_TRCD  << 20)     +   \
                            (SDRAM_TRP   << 16)     +   \
                            (SDRAM_TRC   << 12)

#define     SDRAM_PERIOD        1460        /* < 4096           */
#define     SDRAM_XRFR          0           /* < 4              */

#define     SDRAM_TIMING    (SDRAM_XRFR  << 24)     +   \
                            (SDRAM_PERIOD<<  0)

#define     SDRAM_TCL           1           /* 0: 2tck, 1: 3tck */
#define     SDRAM_TRAS          7 /* 5 */   /* tRAS/p - 1, < 8  */
#define     SDRAM_TRRD          1 /* 0 */   /* 0: 2tck, 1: 3tck */
#define     SDRAM_TWR           1 /* 0 */   /* tWR/p - 1, < 4   */
#define     SDRAM_THZP          2 /* 0 */   /* tHZP/p - 1, < 4  */
#define     SDRAM_RD2RD         1           /* 0: 1tck, 1: 2tck */
#define     SDRAM_RD2DEAC       3           /*             < 4  */
#define     SDRAM_RD2WR         5           /*             < 8  */
#define     SDRAM_R2WDQM        3           /*             < 4  */
#define     SDRAM_WR2WR         1           /*             < 2  */
#define     SDRAM_WR2DEAC       1           /*             < 4  */
#define     SDRAM_WR2RD         1           /*             < 2  */

#define     SDRAM_EXTENSION (SDRAM_WR2RD  << 20)    +   \
                            (SDRAM_WR2DEAC<< 18)    +   \
                            (SDRAM_WR2WR  << 17)    +   \
                            (SDRAM_R2WDQM << 15)    +   \
                            (SDRAM_RD2WR  << 12)    +   \
                            (SDRAM_RD2DEAC<< 10)    +   \
                            (SDRAM_RD2RD  <<  9)    +   \
                            (SDRAM_THZP   <<  7)    +   \
                            (SDRAM_TWR    <<  5)    +   \
                            (SDRAM_TRRD   <<  4)    +   \
                            (SDRAM_TRAS   <<  1)    +   \
                            (SDRAM_TCL    <<  0)


#ifdef SDRAM16MB                            /*  SDRAM 16 MBytes             */

#define     SDRAM_size      0x01000000      /* 16 MBytes                    */
#define     SDRAM_bank0     0xA0000000      /* start bank0                  */
#define     SDRAM_bank1     0xA0400000      /* start bank1                  */
#define     SDRAM_bank2     0xA0800000      /* start bank2                  */
#define     SDRAM_bank3     0xA0C00000      /* start bank3                  */

#endif

#ifdef SDRAM64MB                            /*  SDRAM 64 MBytes             */

#define     SDRAM_size      0x04000000      /* 64 MBytes                    */
#define     SDRAM_bank0     0xA0000000      /* start bank0                  */
#define     SDRAM_bank1     0xA1000000      /* start bank1                  */
#define     SDRAM_bank2     0xA2000000      /* start bank2                  */
#define     SDRAM_bank3     0xA3000000      /* start bank3                  */

#endif

#define     MTEXT       0
#define     MSTACK      1
#define     MDATA       2
#define     MIMAGE      3


/* cache cofiguration register bit settings             */

#define     L2MODE              7           /* 64K 4-way cache      */
#define     RPRIORITY           1           /* 0: CPU prioritized   */
#define     ID                  1           /* 1: invalidate L1D    */
#define     IP                  1           /* 1: invalidate L1P    */

#define IMCR_NOCACHE        0


#define IMCR_INVAL          (RPRIORITY    << 31)    +   \
                            (IP           <<  9)    +   \
                            (ID           <<  8)    +   \
                            (L2MODE       <<  0)

#define IMCR_CACHE          (RPRIORITY    << 31)    +   \
                            (0            <<  9)    +   \
                            (0            <<  8)    +   \
                            (L2MODE       <<  0)

#if defined DM648 || defined DM6435

/* EDMA 3 defines																				*/

typedef struct
  {
  int   options;        			/* EDMA options                 							*/
  void *src_addr;       			/* source address               							*/
  int   BCAC;           			/* B count / A count             							*/
  void *dst_addr;       			/* destination address          							*/
  int   DBIXSBIX;         			/* dest B / src B index          							*/
  int   BCRLDLNK;       			/* BC reload / link address     							*/
  int   DCIXSCIX;                   /* dest C / src C index                                     */
  int	RSVDCC;                     /* reserved / C count                                       */
  } EDMA3;

#if defined DM648

#define EDMA3_REGS   0x02A00000     /* EDMA3 register base address      						*/
#define EDMA3_BASE   0x02A04000     /* parameter ram base address    (64 param sets)			*/

#elif defined DM6435

#define EDMA3_REGS   0x01C00000     /* EDMA3 register base address      						*/
#define EDMA3_BASE   0x01C04000     /* parameter ram base address    (64 param sets)			*/

#endif

#define EDMA3_DCMAP (EDMA3_REGS+0x0100)/* DMA channel map register 0 (64 regs)					*/

#define EDMA3_DMAQ0 (EDMA3_REGS+0x0240)/* DMA queue number map register 0          				*/
#define EDMA3_DMAQ1 (EDMA3_REGS+0x0244)/* DMA queue number map register 1          				*/
#define EDMA3_DMAQ2 (EDMA3_REGS+0x0248)/* DMA queue number map register 2          				*/
#define EDMA3_DMAQ3 (EDMA3_REGS+0x024c)/* DMA queue number map register 3          				*/
#define EDMA3_DMAQ4 (EDMA3_REGS+0x0250)/* DMA queue number map register 4          				*/
#define EDMA3_DMAQ5 (EDMA3_REGS+0x0254)/* DMA queue number map register 5          				*/
#define EDMA3_DMAQ6 (EDMA3_REGS+0x0258)/* DMA queue number map register 6          				*/
#define EDMA3_DMAQ7 (EDMA3_REGS+0x025c)/* DMA queue number map register 7          				*/

#define EDMA3_QUEPRI (EDMA3_REGS+0x284)/* queue priority register               				*/

#define EDMA3_EMR   (EDMA3_REGS+0x0300)/* event missed register            						*/
#define EDMA3_EMRH  (EDMA3_REGS+0x0304)/* event missed register high       						*/
#define EDMA3_EMCR  (EDMA3_REGS+0x0308)/* event missed clear register      						*/
#define EDMA3_EMCRH (EDMA3_REGS+0x030c)/* event missed clear register high 						*/

#define EDMA3_CCERR (EDMA3_REGS+0x0318)/* EDMA3CC error register 			                	*/ 
#define EDMA3_CCERRCLR (EDMA3_REGS+0x031c)/* EDMA3CC error clear register        				*/

#define EDMA3_QWMTHRA  (EDMA3_REGS+0x0620)/* queue watermark threshold A register for Q[3:0]	*/

#define EDMA3_ER    (EDMA3_REGS+0x1000)/* event register                   						*/
#define EDMA3_ERH   (EDMA3_REGS+0x1004)/* event register high              						*/
#define EDMA3_ECR   (EDMA3_REGS+0x1008)/* event clear register             						*/
#define EDMA3_ECRH  (EDMA3_REGS+0x100c)/* event clear register high        						*/
#define EDMA3_ESR   (EDMA3_REGS+0x1010)/* event set register               						*/
#define EDMA3_ESRH  (EDMA3_REGS+0x1014)/* event set register high          						*/
#define EDMA3_CER   (EDMA3_REGS+0x1018)/* chained event register   					        	*/
#define EDMA3_CERH  (EDMA3_REGS+0x101c)/* chained event register high				        	*/
#define EDMA3_EER   (EDMA3_REGS+0x1020)/* event enable register            						*/
#define EDMA3_EERH  (EDMA3_REGS+0x1024)/* event enable register high       						*/
#define EDMA3_EECR  (EDMA3_REGS+0x1028)/* event enable clear register         					*/
#define EDMA3_EECRH (EDMA3_REGS+0x102c)/* event enable clear register high    					*/
#define EDMA3_EESR  (EDMA3_REGS+0x1030)/* event enable set register         					*/
#define EDMA3_EESRH (EDMA3_REGS+0x1034)/* event enable set register high    					*/
#define EDMA3_SER   (EDMA3_REGS+0x1038)/* secondary event register            					*/
#define EDMA3_SERH  (EDMA3_REGS+0x103c)/* secondary event register high       					*/
#define EDMA3_SECR  (EDMA3_REGS+0x1040)/* secondary event clear register      					*/
#define EDMA3_SECRH (EDMA3_REGS+0x1044)/* secondary event clear register high 					*/

#define EDMA3_IER   (EDMA3_REGS+0x1050)/* interrupt enable register			         			*/
#define EDMA3_IERH  (EDMA3_REGS+0x1054)/* interrupt enable register high	         			*/
#define EDMA3_IECR  (EDMA3_REGS+0x1058)/* interrupt enable clear register		       			*/
#define EDMA3_IECRH (EDMA3_REGS+0x105c)/* interrupt enable clear register high	       			*/
#define EDMA3_IESR  (EDMA3_REGS+0x1060)/* interrupt enable set register			       			*/
#define EDMA3_IESRH (EDMA3_REGS+0x1064)/* interrupt enable set register high	       			*/
#define EDMA3_IPR   (EDMA3_REGS+0x1068)/* interrupt pending register   	       					*/
#define EDMA3_IPRH  (EDMA3_REGS+0x106c)/* interrupt pending register high      					*/
#define EDMA3_ICR   (EDMA3_REGS+0x1070)/* interrupt clear register   	       					*/
#define EDMA3_ICRH  (EDMA3_REGS+0x1074)/* interrupt clear register high      					*/
#define EDMA3_IEVAL (EDMA3_REGS+0x1078)/* interrupt evaluate register       					*/

#else
/* EDMA defines																					*/

typedef struct
  {
  int   options;        			/* EDMA options                 							*/
  void *src_addr;       			/* source address               							*/
  int   FCEC;           			/* frame count / element count  							*/
  void *dst_addr;       			/* destination address          							*/
  int   FIXEIX;         			/* frame / element index        							*/
  int   ECRLDLNK;       			/* EC reload / link address     							*/
  } EDMA;

#define EDMA_BASE   0x01A00000      /* parameter ram base address       						*/

#define EDMA_PQSR   0x01A0FFE0      /* priority queue status register   						*/
#define EDMA_CIPR   0x01A0FFE4      /* channel interrupt pending reg.   						*/
#define EDMA_CIER   0x01A0FFE8      /* channel interrupt enable register						*/
#define EDMA_CCER   0x01A0FFEC      /* channel chain enable register    						*/
#define EDMA_ER     0x01A0FFF0      /* event register                   						*/
#define EDMA_EER    0x01A0FFF4      /* event enable register            						*/
#define EDMA_ECR    0x01A0FFF8      /* event clear register             						*/
#define EDMA_ESR    0x01A0FFFC      /* event set register               						*/

/* additional registers for C64 EDMA															*/

#define EDMA_EPRH	0x01A0FF9C		/* Event polarity high register								*/
#define EDMA_CIPRH	0x01A0FFA4		/* Channel interrupt pending high register					*/
#define EDMA_CIERH	0x01A0FFA8		/* Channel interrupt enable high register					*/
#define EDMA_CCERH	0x01A0FFAC		/* Channel chain enable high register						*/
#define EDMA_ERH	0x01A0FFB0		/* Event high register										*/
#define EDMA_EERH	0x01A0FFB4		/* Event enable high register								*/
#define EDMA_ECRH	0x01A0FFB8		/* Event clear high register								*/
#define EDMA_ESRH	0x01A0FFBC		/* Event set high register									*/
#define EDMA_PQAR0	0x01A0FFC0		/* Priority queue allocation register 0						*/
#define EDMA_PQAR1	0x01A0FFC4		/* Priority queue allocation register 1						*/
#define EDMA_PQAR2	0x01A0FFC8		/* Priority queue allocation register 2						*/
#define EDMA_PQAR3	0x01A0FFCC		/* Priority queue allocation register 3						*/
#define EDMA_EPRL	0x01A0FFDC		/* Event polarity low register								*/
#define EDMA_CIPRL	0x01A0FFE4		/* Channel interrupt pending low register (CIPR)			*/
#define EDMA_CIERL	0x01A0FFE8		/* Channel interrupt enable low register (CIER)				*/
#define EDMA_CCERL	0x01A0FFEC		/* Channel chain enable low register (CCER)					*/
#define EDMA_ERL	0x01A0FFF0		/* Event low register (ER)									*/
#define EDMA_EERL	0x01A0FFF4		/* Event enable low register								*/
#define EDMA_ECRL	0x01A0FFF8		/* Event clear low register (ECR)							*/
#define EDMA_ESRL	0x01A0FFFC		/* Event set low register (ESR)								*/

/* additional cache registers											*/

#define IMCR_L2IBAR  0x01844018	    /* L2 invalidate base	*/	
#define IMCR_L2IWC   0x0184401C	    /* L2 inval. wordcount	*/
#define IMCR_L1DIBAR 0x01844048	    /* L1D invalidate base	*/
#define IMCR_L1DIWC	 0x0184404C	    /* L1D inval. wordcount	*/	
#define EDMA_WEIGHT	 0x01841000	    /* EDMA vs L2 priority	*/	

/* C64xx defines														*/

#define GPEN					0x01B00000			/* general purpose enable	*/
#define GPDIR					0x01B00004			/* general purpose direction*/
#define GPVAL					0x01B00008			/* general purpose value	*/
#define GPGC					0x01B00020			/* global control			*/
#define GPPOL					0x01B00024			/* polarity					*/
#define GPHM					0x01B00014			/* delta high mask			*/
#define GPLM					0x01B0001C			/* delta low mask			*/
#define GPDH					0x01B00010			/* delta high register		*/
#define GPDL					0x01B00018			/* delta low  register		*/

#endif

/* VC7210 defines                                                         */   

#define C6220_output            0x44000000          /* C6220 PLC output register*/
#define C6220_input             0x44000000          /* C6220 PLC intput register*/
#define C6220_FLASH             0x44000040          /* Flash Eprom address bits */
#define C6220_STATUS            0x44000040          /* C6220 status register    */
#define C6220_CONTROL           0x44000080          /* C6220 control register   */
#define C6220_VERSION           0x44000080          /* C6220 version register   */
#define C6220_exposure          0x440000C0          /* video input exposure ctl */

/* VC6210 defines CPLD                                                    */   

#define C6210_LCON              0x44000000          /* serial shift reg LCON    */
#define C6210_mode              0x44000040          /* serial mode register     */
#define C6210_exposure          0x44000080          /* sensor exposure ctl      */
#define C6210_output            0x440000C0          /* PLC output register      */
#define C6210_LED               0x44000100          /* LED control              */
#define C6210_TTL               0x44000100          /* TTL input register       */
#define C6210_FLASH             0x44000140          /* Flash Eprom address bits */
#define C6210_CONTROL           0x44000180          /* C6210 control register   */
#define C6210_STATUS            0x44000180          /* C6210 status register    */
#define C6210_AD                0x440001C0          /* C6210 a/d converterserial*/
#define C6210_FLASH_COUNTER     0x440001C0          /* C6210 flash counter reg  */

/* VC6210 defines FPGA (compatible with CPLD)                                   */   

#define F6210_mode              0x44000040          /* serial mode register (wr)*/
#define F6210_exposure          0x44000080          /* sensor exposure ctl  (wr)*/
#define F6210_output            0x440000C0          /* PLC output register  (wr)*/
#define F6210_LED               0x44000100          /* LED control          (wr)*/
#define F6210_TTL               0x44000100          /* TTL input register   (rd)*/
#define F6210_FLASH             0x44000140          /* Flash address bits   (wr)*/
#define F6210_CONTROL           0x44000180          /* F6210 control reg1   (wr)*/
#define F6210_STATUS            0x44000180          /* F6210 status register(rd)*/

/* VC6210 defines FPGA (new for VC6210RQ/QU)                                    */   
#define F6210_line_cnt_reg      0x44000000          /* line count debug reg (rd)*/
#define F6210_STATUS2           0x44000040          /* F6210 status reg2    (rd)*/
#define F6210_SEN_ERR_INT       0x44000080          /* F6210 sen_error_int  (rd)*/
#define F6210_SEN_CTRL          0x440001C0          /* VC6210RQ ctrl_reg2   (wr)*/

/* FA40 / SBC4000 defines                                                 */	

#define RS485_receive			0x80000000			/* RS485 receive register	*/
#define RS485_transmit			0x80000000			/* RS485 transmi register	*/
#define RS485_mode				0x80000008			/* RS485 mode register		*/
#define FA40_exposure			0x80000010			/* video input exposure ctl	*/
#define FA40_output				0x80000018			/* PLC output register		*/
#define FA40_LED				0x80000020			/* LED control				*/
#define FA40_TTL				0x80000020			/* TTL input register		*/
#define FA40_FLASH				0x80000028			/* Flash Eprom address bits	*/
#define FA40_CONTROL			0x80000030			/* FA40 control register	*/
#define FA40_STATUS				0x80000030			/* FA40 status register		*/
#define FA40_AD					0x80000038			/* FA40 a/d converter serial*/


#ifdef DM648

/* DM648 defines														*/

#define UART_RBR				0x02047000			/* UART Receiver Buffer Register (Read Only)		 */
#define UART_THR				0x02047000			/* Transmitter Holding Register (Write Only)		 */
#define UART_IER				0x02047004			/* Interrupt Enable Register						 */
#define UART_IIR				0x02047008			/* Interrupt Identification Register (Read Only)	 */
#define UART_FCR				0x02047008			/* FIFO Control Register (Write Only)				 */
#define UART_LCR				0x0204700C			/* Line Control Register							 */
#define UART_MCR				0x02047010			/* Modem Control Register							 */
#define UART_LSR				0x02047014			/* Line Status Register								 */
#define UART_DLL				0x02047020			/* UART Divisor Latch (LSB)							 */
#define UART_DLH				0x02047024			/* UART Divisor Latch (MSB)							 */
#define UART_PID				0x02047028			/* Peripheral Identification Register				 */
#define UART_PWREMU_MGMT		0x02047030			/* Power and Emulation Management Register			 */
#define UART_MDR				0x02047034			/* Mode Definition Register							 */

#endif

#ifdef DM6435

/* DM6435 defines														*/

#define UART_RBR				0x01C20000			/* UART Receiver Buffer Register (Read Only)		 */
#define UART_THR				0x01C20000			/* Transmitter Holding Register (Write Only)		 */
#define UART_IER				0x01C20004			/* Interrupt Enable Register						 */
#define UART_IIR				0x01C20008			/* Interrupt Identification Register (Read Only)	 */
#define UART_FCR				0x01C20008			/* FIFO Control Register (Write Only)				 */
#define UART_LCR				0x01C2000C			/* Line Control Register							 */
#define UART_MCR				0x01C20010			/* Modem Control Register							 */
#define UART_LSR				0x01C20014			/* Line Status Register								 */
#define UART_DLL				0x01C20020			/* UART Divisor Latch (LSB)							 */
#define UART_DLH				0x01C20024			/* UART Divisor Latch (MSB)							 */
#define UART_PID				0x01C20028			/* Peripheral Identification Register				 */
#define UART_PWREMU_MGMT		0x01C20030			/* Power and Emulation Management Register			 */
#define UART_MDR				0x01C20034			/* Mode Definition Register							 */

#endif

/* VC64XX Stereo FPGA control bits for use without extra define         */

#define TRIG_S_int_ena    0x81000000            /* Trigger input interrupt ena. */
#define TRIG_S_OUT        0x00000880            /* Trigger output signal        */
#define TRIG_S_out_pol    0x00000440            /* Trigger output polarity      */
#define TRIG_S_out_exp    0x00000220            /* Trigger output = exposure    */
#define TRIG_S_inp_sticky 0x00000110            /* Sticky trigger mode          */
#define TRIG_S_inp_pol    0x00C00000            /* Trigger input polarity       */

#endif  /* register_h define    */																		 
																										 
/* EOF */

																									 
