/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2000-2011 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: sysvar.h
***
*** Comments:
***     header for handling of VCRT system variables
***     system variables are "globals" which change characteristics
***     of certain functions or contain system specific information
***     system variables may be accessed with getvar() or setvar()
***     functions (user context) or directly - e.g. *sec (kernel)
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 10, 2002  5.00       ME      Initial Version
***    Jul 19, 2004  5.17       ME      cosmetic update
***    Dec 08, 2009  5.28.8     MS      reformatted with spaces 
***
***************************************************************************
*END***********************************************************************/
#ifndef register_h
#include <register.h>
#endif

#define     DISP_PERIOD     0   /* refresh rate for display & overlay   */
#define     DISP_CNT        1   /* counter for refresh rate             */
#define     DISP_START      2   /* start address for display            */
#define     OVLY_START      3   /* start address for overlay            */
#define     DISP_ACTIVE     4   /* 0: no refresh / 1: refresh (display) */
#define     OVLY_ACTIVE     5   /* 0: no refresh / 1: refresh (overlay) */
#define     CAPT_START      6   /* start address for image capture      */
#define     HWIDTH          7   /* active horizontal pixels             */
#define     VWIDTH          8   /* number of active vertical lines      */
#define     VPITCH          9   /* video pitch                          */
#define     EXPCNT          10  /* number of exposure cycles (lines)    */
#define     GAIN            11  /* video gain value                     */
#define     IMODE           12  /* video mode                           */
#define     VSTAT           13  /* video status 0=idle 1=capture busy   */
#define     INTFL           14  /* interrupt flag                       */
#define     CPUCLK          15  /* master cpu clock frequency           */

#define     MSEC            16  /* real-time clock: millisecond         */
#define     SEC             17  /* real-time clock: seconds since 1900  */
/* 18: second part of SEC long value    */
#define     EXUNIT          19  /* time unit for exposure control [usec]*/
#define     TIMESTAMP       20  /* timestamp for last captured image[ms]*/
#define     DAYLIGHT        21  /* daylight savings time flag           */
#define     TIMEZONE        22  /* real-time clock: timezone            */
#define     LOWBAT          23  /* low battery voltage: time invalid    */
#define     TEMP            24  /* cpu board temperature                */
#define     VERSION         25  /* VCRT software version                */
#define     DRAMSIZE        26  /* size of main SDRAM                   */

#define     HEAP            27  /* start address of heap                */
#define     FREEP           28  /* heap control pointer, top of heap    */
#define     TOPSTK          29  /* top of stack                         */
#define     STSIZE          30  /* stack size - linker info only        */

#define     PLCOUT          31  /* PLC output value                     */
#define     PLCIN           32  /* PLC input value                      */
#define     EXPOSING        33  /* image number of image being exposed  */
#define     STORING         34  /* image number of image being stored   */
#define     IMGREADY        35  /* image number of image ready for use  */
#define     POWFAIL         36  /* 1: PLC power failure / 0: power ok   */
#define     LATENCY         37  /* maximum interrupt latency            */
#define     MMC             38  /* missing multi-media card: -1         */
#define     IPADDR          39  /* IP address (ethernet version)        */
#define     IPMASK          40  /* IP mask    (ethernet version)        */
#define     IPGATE          41  /* IP gateway (ethernet version)        */
#define     DHCP            42  /* dhcp 1=on 0=off -1=failure           */
#define     TPRIORITY       43  /* exec1 task priority default=9        */
#define     FPGAVERSION     44  /* fpga version date                    */
#define     OVC_STAT        45  /* overcurrent status (DM640 only)      */
#define     CAPT_ERR        46  /* capture error counter (DM640 only)   */
#define     SCRLOGPAGE      47  /* needed for malloc.h                  */
#define     OVLLOGPAGE      48  /* needed for malloc.h                  */
#define     MODEL           49  /* camera model                         */
#define     DHWIDTH         50  /* display horizontal width             */
#define     DVWIDTH         51  /* display vertical width               */
#define     OVL_MASK        52  /* overlay mask default value           */
#define     PRIVATE         53  /* index for private sysvars            */
#define     TELNET          54  /* telnet active                        */
#define     ETHANDLE        55  /* ethernet handle                      */
#define     TOTAL           56  /* total number of clocks in line       */
#define     XSG             57  /* clock cycles between XSUB & XSG      */
#define     USR_EVENT       58  /* first user event number              */
#define     USR_EVT_LAST    59  /* last user event number               */
#define     RED             60  /* whitebalance RED value               */
#define     GREEN           61  /* whitebalance GREEN value             */
#define     BLUE            62  /* whitebalance BLUE value              */
#define     GAMMA           63  /* gamma for output LUT                 */
#define     RGBO_START      64  /* start of RGBO buffer / color camera  */
#define     COLOR_MODE      65  /* mode for color display               */
#define     UPTIME          66  /* system uptime in seconds             */
#define     SYSMEM          67  /* system mem struct                    */
#define     TESTVERSION     68  /* testversion 0 = releas               */
#define     ETHLINK         69  /* ethernet link info                   */
#define     LTEST           70  /* interrupt latency test               */
#define     EMAC_COUNT      71  /* emac_count                           */
#define     TIME_SLICE      72  /* time_slice                           */
#define     SENSORID        73  /* sensor id of camera head             */
#define     DEFHANDLE       74  /* initial io default handle            */
#define     DEBUG_SWITCH    75  /* switch to enable debugging info      */
#define     DNSSERVER1      76  /* dns server 1 ip address              */
#define     DNSSERVER2      77  /* dns server 2 ip address              */
#define     EXUNIT_F        78  /* time for exposure control [us] float */
#define     GAINDIFF        79  /* offset for second ADC1 values +/-32  */
#define     PPW             80  /* password pointer                     */
#define     SDCARD          81  /* sdcard options                       */
#define     CAPT_START2     82  /* start address second image capture   */
#define     MONI_TICKS      83  /* polling ticks of monitor task in ms  */

/* VC64XXE TEST private vars - will be removed later !!!                */
#define     IOPORTIN        84  /* IO PORT INPUT value                  */
#define     IOPORTOUT       85  /* IO PORT OUTPUT value                 */
#define     IOPORTDIR       86  /* IO PORT DIR value                    */

#define     SENSORID2       87  /* sensor id of camera head 2           */
#define     DHCPSERVER      88  /* dhcpserver                           */
#define     VC_HW_TYPE      89  /* VC Hardware platform type            */
#define     STEREO_MODE     90  /* VC64XX stereo mode 0=compat 1=offset */
#define     PRE_ALLOC       91  /* preallocated memory address          */
#define     PRE_MSIZE       92  /* preallocated memory size             */
#define     SEN_ERR_RQ      93  /* sensor error flag VCX210RQ           */
#define     GAINDIFF2       94  /* offset for sensor2 ADC2 values +/-32 */
#define     GAINDIFF3       95  /* offset for sensor2 ADC3 values +/-32 */
#define     PRIVATESYS      96  /* storage for 50 private sysvars       */
                                /* must be last item in list !          */

#ifdef KERNEL

#ifndef SYS_DECLARE
extern unsigned int sysvar[];
#endif

#define     disp_period     ((int *)&sysvar[DISP_PERIOD])
#define     disp_cnt        ((int *)&sysvar[DISP_CNT])
#define     disp_start      ((unsigned int **)&sysvar[DISP_START])
#define     ovly_start      ((unsigned int **)&sysvar[OVLY_START])
#define     disp_active     ((int *)&sysvar[DISP_ACTIVE])
#define     ovly_active     ((int *)&sysvar[OVLY_ACTIVE])
#define     capt_start      ((unsigned int **)&sysvar[CAPT_START])
#define     hwidth          ((int *)&sysvar[HWIDTH])
#define     vwidth          ((int *)&sysvar[VWIDTH])
#define     vpitch          ((int *)&sysvar[VPITCH])
#define     imode           ((volatile int *)&sysvar[IMODE])
#define     vstat           ((volatile int *)&sysvar[VSTAT])
#define     expcnt          ((int *)&sysvar[EXPCNT])
#define     gain            ((int *)&sysvar[GAIN])
#define     msec            ((int *)&sysvar[MSEC])
#define     sec             ((long *)&sysvar[SEC])
#define     timestamp       ((unsigned int *)&sysvar[TIMESTAMP])
#define     daylight        ((int *)&sysvar[DAYLIGHT])
#define     timezone        ((int *)&sysvar[TIMEZONE])
#define     lowbat          ((int *)&sysvar[LOWBAT])
#define     temp            ((int *)&sysvar[TEMP])
#define     version         ((int *)&sysvar[VERSION])
#define     dramsize        ((int *)&sysvar[DRAMSIZE])
#define     plcout          ((int *)&sysvar[PLCOUT])
#define     plcin           ((int *)&sysvar[PLCIN])
#define     exposing        ((int *)&sysvar[EXPOSING])
#define     storing         ((int *)&sysvar[STORING])
#define     imgready        ((int *)&sysvar[IMGREADY])
#define     powfail         ((int *)&sysvar[POWFAIL])
#define     latency         ((int *)&sysvar[LATENCY])
#define     mmc             ((int *)&sysvar[MMC])
#define     ipaddr          ((int *)&sysvar[IPADDR])
#define     ipmask          ((int *)&sysvar[IPMASK])
#define     ipgate          ((int *)&sysvar[IPGATE])
#define     dhcp            ((int *)&sysvar[DHCP])
#define     tpriority       ((unsigned int *)&sysvar[TPRIORITY])
#define     fpgaversion     ((unsigned int *)&sysvar[FPGAVERSION])
#define     ovc_stat        ((int *)&sysvar[OVC_STAT])
#define     capt_err        ((int *)&sysvar[CAPT_ERR])
#define     model           ((int *)&sysvar[MODEL])
#define     ovl_mask        ((int *)&sysvar[OVL_MASK])
#define     telnet          ((int *)&sysvar[TELNET])
#define     ethandle                ((int *)&sysvar[ETHANDLE])
#define     dhwidth         ((int *)&sysvar[DHWIDTH])
#define     dvwidth         ((int *)&sysvar[DVWIDTH])
#define     rgbo_start      ((unsigned int **)&sysvar[RGBO_START])
#define     color_mode      ((int *)&sysvar[COLOR_MODE])
#define     uptime          ((int *)&sysvar[UPTIME])
#define     sysmem          ((unsigned int *)&sysvar[SYSMEM])
#define     testversion     ((int *)&sysvar[TESTVERSION])
#define     ethlink         ((unsigned int *)&sysvar[ETHLINK])
#define     ltest           ((unsigned int *)&sysvar[LTEST])
#define     emac_count      ((unsigned int *)&sysvar[EMAC_COUNT])
#define     time_slice      ((unsigned int *)&sysvar[TIME_SLICE])
#define     sensorid        ((unsigned int *)&sysvar[SENSORID])
#define     defhandle       ((unsigned int *)&sysvar[DEFHANDLE])
#define     debug_switch    ((unsigned int *)&sysvar[DEBUG_SWITCH])
#define     dnsserver1      ((unsigned int *)&sysvar[DNSSERVER1])
#define     dnsserver2      ((unsigned int *)&sysvar[DNSSERVER2])
#define     gaindiff        ((int *)&sysvar[GAINDIFF])
#define     ppw             ((unsigned int *)&sysvar[PPW])
#define     sdcard          ((unsigned int *)&sysvar[SDCARD])
#define     capt_start2     ((unsigned int **)&sysvar[CAPT_START2])
#define     moni_ticks      ((unsigned int *)&sysvar[MONI_TICKS])

/* VC64XXE TEST private vars - will be removed later !!!                */
#define     ioportin        ((unsigned int *)&sysvar[IOPORTIN])
#define     ioportout       ((unsigned int *)&sysvar[IOPORTOUT])
#define     ioportdir       ((unsigned int *)&sysvar[IOPORTDIR])

#define     sensorid2       ((unsigned int *)&sysvar[SENSORID2])
#define     dhcpserver      ((unsigned int *)&sysvar[DHCPSERVER])
#define     vc_hw_type      ((unsigned int *)&sysvar[VC_HW_TYPE])
#define     stereo_mode     ((unsigned int *)&sysvar[STEREO_MODE])
#define     sen_err_rq      ((unsigned int *)&sysvar[SEN_ERR_RQ])
#define     gaindiff2       ((int *)&sysvar[GAINDIFF2])
#define     gaindiff3       ((int *)&sysvar[GAINDIFF3])

#endif

#ifdef SYS_DECLARE

unsigned int sysvar[] = {
    4,          /* 0    disp_period                 */
    1,          /* 1    disp_cnt                    */
    0,          /* 2    disp_start                  */
    0,          /* 3    ovl_start                   */
    1,          /* 4    disp_active                 */
    0,          /* 5    ovly_active                 */
    0,          /* 6    capt_start                  */
    752,        /* 7    hwidth                      */
    582,        /* 8    vwidth                      */
    768,        /* 9    vpitch                      */
    200,        /* 10   expcnt                      */
    0x60,       /* 11   gain                        */
    0,          /* 12   imode                       */
    0,          /* 13   vstat                       */
    0,          /* 14   intfl                       */
    0,          /* 15   cpuclk                      */
    0,          /* 16   msec                        */
    0,          /* 17   sec                         */
    0,          /* 18   sec2 (long)                 */
    39,         /* 19   exunit 39 us deprecated     */
    0,          /* 20   timestamp                   */
    1,          /* 21   daylight                    */
    1,          /* 22   timezone                    */
    1,          /* 23   lowbat                      */
    0,          /* 24   temp                        */
    531,        /* 25   version                     */
    SDRAM_size, /* 26   DRAM size                   */
    0,          /* 27                               */
    0,          /* 28                               */
    0,          /* 29                               */
    0,          /* 30                               */
    0,          /* 31   PLC output                  */
    0,          /* 32   PLC input                   */
    0,          /* 33   exposing                    */
    0,          /* 34   storing                     */
    0,          /* 35   imgready                    */
    1,          /* 36   powfail                     */
    0,          /* 37   latency                     */
    -1,         /* 38   mmc                         */
    0xC0A80041, /* 39   ipaddr = 192.168.0.65       */
    0xFFFFFF00, /* 40   ipmask = 255.255.255.0      */
    0,          /* 41   ipgate = none               */
    0,          /* 42   dhcp = 0 (off)              */
    9,          /* 43   tpriority = 9 for exec1     */
    0,          /* 44   fpga version date           */
    0,          /* 45   overcurrent status DM640    */
    0,          /* 46   capture error DM640         */
    0,          /* 47   scrlogpage                  */
    0,          /* 48   ovllogpage                  */
    0,          /* 49   model                       */
    752,        /* 50   dhwidth                     */
    582,        /* 51   dvwidth                     */
    0,          /* 52   ovl_mask                    */
    PRIVATESYS, /* 53   private                     */
    0,          /* 54   telnet                      */
    0,          /* 55   ethandle                    */
    3676,       /* 56   total                       */
    1863,       /* 57   xsg                         */
    18,         /* 58   usr_event                   */
    63,         /* 59   usr_evt_last                */
    1240,       /* 60   red                         */
    1024,       /* 61   green                       */
    1128,       /* 62   blue                        */
    60,         /* 63   gamma value * 100           */
    0,          /* 64   rgbo_start                  */
    -1,         /* 65   color_mode  (b/w = -1)      */
    0,          /* 66   system uptime in seconds    */
    0,          /* 67   system mem struct           */
    0,          /* 68   testversion                 */
    0,          /* 69   ethlink                     */
    0,          /* 70   ltest                       */
    0,          /* 71   emac_count                  */
    10,         /* 72   time_slice                  */
    0,          /* 73   sensorid                    */
    0,          /* 74   defhandle                   */
    2,          /* 75   debug_switch                */
    0,          /* 76   dnsserver1                  */
    0,          /* 77   dnsserver2                  */
    0x421CCCCD, /* 78   exunit_f 39.211 usec        */
    0,          /* 79   gaindiff                    */
    0,          /* 80   ppw                         */
    0,          /* 81   sdcard                      */
    0,          /* 82   capt_start2                 */
    10,         /* 83   moni_ticks                  */
    0, 0, 0,    /* TEST VC64XXE maybe removed later */
    0,          /* 87   sensorid2                   */
    0,          /* 88   dhcpserver                  */
    VC_HWPL,    /* 89   VC HW platform              */
    0,          /* 90   VC64XX stereo mode          */
    0,          /* 91   pre_alloc memory address    */
    0,          /* 92   pre_msize memory size       */
	0,          /* 93   sen_err_rq VCX210RQ         */
    0,          /* 94   gaindiff2                   */
    0,          /* 95   gaindiff3                   */
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,/* 96  storage for */
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,/*     50 private  */
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,/*     sysvars     */
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

#endif

/* EOF */
