/**********************************************************************//**
***************************************************************************
*** @file   vclib.h
***
*** @brief  Main VCLIB Header File for the VC Image Processing Library
***
*** @author  Copyright (c) 2004-2014 Vision Components
*** @author  All rights reserved.
*** @author  This software embodies materials and concepts which are
***          confidential to Vision Components.
***
***
*** @revisionHistory
***
***    Date          Version    Author  Changes
***    Nov 04, 2004  2.10       ME      initial version
***    Jan 11, 2008  3.02       ME      sobelf, laplacef, maxf, minf replaced
***                                     by FLIB functions
***    Jul 01, 2008  3.05       MS      added c++ support
***    Feb 04, 2009  3.06       ME
***    Oct 26, 2010  3.10       KS      added mom_calc_area()
***    Apr 14, 2011  3.11       KS      added lib version defines
***    Nov 24, 2011  3.12       MS      added HoughList() prototype again
***    Jun 28, 2012  3.12       ME      some changes for color conversion
***                                     routines, color JPEG added
***    Aug 01, 2012  3.13       ME      median 1xN and Nx1 added
***                                     affine transf. / quadratic interpol.
***    Feb 14, 2013  3.13       ME      return value of rlc_delete() changed
***    Mar 29, 2014  3.13       ME      NEW_IMAGE_VAR no longer necessary
***
***
*** @endRevisionHistory
***************************************************************************
***************************************************************************/
#ifndef _VCLIB_h
#define _VCLIB_h


#define  FAST    /* This is the Fast Version of the Library */

//Now globally for all.
#include "vctypes.h"
/* 
#ifndef _VCRT_h
  typedef void *         pointer;
  typedef unsigned char  U8;
  typedef unsigned short U16;
  typedef unsigned int   U32;
  typedef char           I8;
  typedef short          I16;
  typedef int            I32;
  typedef float          F32;
  typedef double         F64;
  
  #if defined __GNUC__
    #define VC_BASICTYPES
    typedef signed   long long I40;
    typedef unsigned long long U40;
    typedef unsigned long long U64;
    typedef signed   long long I64;
    #define   VCRT_NO_COMPAT
    #define  _nassert
    #define  _abs            abs
  #endif
  
  #if defined __TI_COMPILER_VERSION__
    #define VC_BASICTYPES
    typedef long           I40;
    typedef unsigned long  U40;
    #if __TI_COMPILER_VERSION__ >= 5000000
      typedef unsigned long long U64;
      typedef   signed long long I64;
    #endif
  #endif
  
  #ifndef VC_BASICTYPES
    // Microsoft Visual C++ 6.0 definitions
    #define VC_BASICTYPES
    typedef unsigned __int64 U40;
    typedef          __int64 I40;
    typedef unsigned __int64 U64;
    typedef          __int64 I64;
    #define   VCRT_NO_COMPAT
    #define  _nassert
    #define  _abs            abs
  #endif
    

  #ifdef  NULL
     #undef  NULL
  #endif
  #ifdef __cplusplus
     #define NULL (0)
  #else
     #define NULL ((pointer)0)
  #endif

  #ifndef MDATA
    #define     MDATA       2
  #endif
  void *sysmalloc(unsigned nwords, int type);
  int   sysfree(void *ap);
  //#define vcsetup()
  #define vcmalloc(x)     sysmalloc((x),MDATA)
  #define vcfree(x)       sysfree((x))
  //#define prtfree()       sysprtfree(MDATA)
  #define byte_malloc(x)  sysmalloc(((x)+3)/4,MDATA)
  #define byte_free(x)    sysfree((x))
  
  #if defined __TI_COMPILER_VERSION__
    void  pstr(char *);
    void _pstr(char *);
    void  print(char *, ...);
    void _print(char *, ...);
    void  sprint(char *s, char *pfmt,...);
    void  io_sprintf(char *s, char *pfmt,...);
    #define  getchar    rs232rcv
    #define  putchar    rs232snd
    #define  printf     print
    #define  sprintf    io_sprintf
    #define  sprint     io_sprintf
    #define  fprintf    io_fprintf
    #define  fscanf     io_fscanf
  #else
    #define  print      printf
    #define  sprint     sprintf
    #define  pstr       printf
    #define  rs232rcv   getchar
    #define  rs232snd   putchar
  #endif
#endif
*/


#ifndef __LIST_H
 #include <list.h>
#endif


#ifdef __cplusplus
 extern "C" {
#endif

#define  VCLIB_VERSION        (314)  /**<  Main Version of the vclib/extlib   */
#define  VCLIB_SUBVERSION     (  6)  /**<    Subversion of the vclib/extlib   */

/*  NEW_IMAGE_VAR is deprecated, for old code now use 'OLD_IMAGE_VAR'  */

#ifndef OLD_IMAGE_VAR
   #define NEW_IMAGE_VAR
#endif

#define  addlf(n, p)          FL_sum_U8P(n, p)


#if !defined(__TI_COMPILER_VERSION__)
 //Replacements for functions only available at TI.
 float acotf(float v);
#endif


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup elementaryStructsMacrosAndTypes  Elementary Structs, Macros and Types
*
*   More structs, macros and types are listed beneath their associated
*   functions, like @ref HoughControl and @ref HLine.
*
* @{
*/

 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup imageStructuresAndTypes   Images
 *
 * @{
 */

  /*--*GROUP*-----------------------------------------------------------------*/
  /**
  *@cond ( vclib || extlib )
  *
  * @defgroup pixelwiseDatedImages   Pixel based Image Data
  *
  *    \image html  image_x_y.png
  *    \image latex image_x_y.pdf
  *
  *    Images are usually represented by a two-dimensional set of pixels.
  *    Since computer memories always have a linear structure, image
  *    data is saved in sequence, the pixels are stored in the same order as characters in a (western) book:
  *    The image origin is located at position (0,0). This pixel corresponds to the
  *    image start address @p st at the upper left corner.
  *    Increasing the \a x value by one will lead to the next pixel column to the right,
  *    increasing the \a y value by one will lead to the next pixel row @e downwards.
  *    An image consists of @p dx pixel in the
  *    horizontal direction and @p dy pixel in the vertical direction.
  *    Note that the 'last' viewable pixel lies at (@p dx-1, @p dy-1).
  *
  *    @p pitch defines the @e pixel count going from one pixel to
  *    its lower neighbouring pixel. The advantage of such a parameter lies in the ability
  *    to define sub-images of a larger image: The @p pitch of the larger image is also
  *    used as pitch for the sub-image, but
  *    the dimensions of the sub-image are set to a smaller value, and the
  *    start address pointer of the sub-image is set to an address within the
  *    larger image. It is clear that @p pitch must always be equal or greater
  *    than @p dx.
  *
  *    \image html  image_dx_dy_pitch.png
  *    \image latex image_dx_dy_pitch.pdf
  *
  *    Therefore the address of a pixel should always be calculated using the
  *    following formula:
  *    @f[
  *      \textsl{pPx}_{(x,y)}\/ = \texttt{st}\/ + [(y) * \texttt{pitch}\/] + (x).
  *    @f]
  *    Here, @p st is the start address of the video memory area
  *    (where also the data of the pixel at position (0,0) lies),
  *    @a x and @a y are the coordinates of the pixel to be addressed by @p pPx.
  *
  * @{
  */

   #ifndef NEW_IMAGE_VAR
      /**  Obsolete, for Compatibility Reasons only.*/
    typedef struct {I40 st; I32 dx; I32 dy; I32 pitch;} image;
    #define  NULL_IMAGE  {0, 0, 0, 0}
   #else
    /*--*STRUCT*--------------------------------------------------------------*/
    /**
    *
    *  @brief  Main Structure for Describing Pixel Images.
    *
    *    This structure describes the type of the image, its dimensions and
    *    its location in memory. It is the most important structure of the
    *    \VCLib.
    *
    */
    typedef struct  /* new image variable                                          */
    {               /* includes color images                                       */
     U8  *st;       /*!<  Image Data Start Address (Channel 0 Pixel Data).         */
     U32  type;     /*!<  Type of Image Data, Listed in '@ref imageVariableTypes'. */
     I32  dx;       /*!<  Horizontal Visible Pixel Count.                          */
     I32  dy;       /*!<  Vertical Visible Pixel Count.                            */
     I32  pitch;    /*!<  Address Offset from a Pixel to its Lower Neighbour Pixel.*/
     U8  *ccmp1;    /*!<  Color Component 1  (Channel 1 Pixel Data).               */
     U8  *ccmp2;    /*!<  Color Component 2  (Channel 2 Pixel Data).               */
    } image;
    /*------------------------------------------------------------------------*/
    #define  NULL_IMAGE  {NULL, IMAGE_GREY, 0, 0, 0, NULL, NULL}
   #endif



   /*--*GROUP*----------------------------------------------------------------*/
   /**
   *@cond ( vclib || extlib )
   *
   * @defgroup imageVariableTypes   Types of Image Data
   *
   *  @brief  Used by the struct @ref image (@ref image.type).
   *
   *   The type of the image defines the location and availability of
   *   data accessible over the three address pointers, @p st, @p ccmp1
   *   and @p ccmp2.
   *   @p ccmp1 and @p ccmp2 are the start addresses of the additional
   *   color components. The following table gives an overview of the various
   *   image types and color components.
   *
   *
   *   @p type | Definition         | Image Type              | Memory Requirement     | st    | ccmp1 | ccmp2
   *   -------:|:-------------------|:------------------------|-----------------------:|:-----:|:-----:|:------:
   *   0       | @ref IMAGE_GREY    | Grey-Scale Image (U8)   | 1   * @p dy * @p pitch | grey  | ---   | ---
   *   1       | @ref IMAGE_BAYER   | Bayer Pattern           | 1   * @p dy * @p pitch | bayer | ---   | ---
   *   2       | @ref IMAGE_RGB     | Color Image RGB         | 3   * @p dy * @p pitch | red   | green | blue
   *   3       | @ref IMAGE_CBCR444 | Color Image YCbCr 4:4:4 | 3   * @p dy * @p pitch | y     | u     | v
   *   4       | @ref IMAGE_CBCR411 | Color Image YCbCr 4:1:1 | 3/2 * @p dy * @p pitch | y     | u     | v
   *   5       | @ref IMAGE_YUVNORM | Normalized YcbCr 4:4:4  | 3   * @p dy * @p pitch | y     | u*    | v*
   *   6       | @ref IMAGE_IHS     | Color Image IHS (HSI)   | 3   * @p dy * @p pitch | i     | h     | s
   *   7       | @ref IMAGE_RGBO    | Reserved                | ---                    | ---   | ---   | ---
   *   8       | @ref IMAGE_VECTOR  | Vector Image            | 2   * @p dy * @p pitch | mag   | dir   | ---
   *   9       | @ref IMAGE_GREY16  | Grey-Scale Image (U16)  | 2   * @p dy * @p pitch | grey  | ---   | ---
   *   10      | @ref IMAGE_GREY32  | Grey-Scale Image (U32)  | 4   * @p dy * @p pitch | grey  | ---   | ---
   *   11      | @ref IMAGE_CMPLX16 | Complex Valued Image    | 4   * @p dy * @p pitch | cmplx | ---   | ---
   *   13      | @ref IMAGE_CBCR422 | Color Image YCbCr 4:2:2 | 2   * @p dy * @p pitch | y     | u     | v
   *   14      | @ref IMAGE_YCBYCR  | Reserved                | ---                    | ---   | ---   | ---
   *
   *   @image  html  image_st_ccmp1_ccmp2.png
   *   @image  latex image_st_ccmp1_ccmp2.pdf
   *
   *   For a grey scale image, the upper left pixel is stored at address @p st.
   *   Going 1 to the right (x-direction), 1 must be added to this address.
   *   Going down (y-direction), pitch must be added. For a color image, two additional memory
   *   blocks are needed to store the color information. It makes sense to store these 2 blocks
   *   tightly behind the first block and behind each other, but this is not mandatory.
   *
   * @{
   */

    ///
    /// @brief  @unitNr{8,Bit} Values per Pixel (0-255), One Channel (@p st).
    ///
    #define  IMAGE_GREY      (0)
    ///
    /// @brief  @unitNr{8,Bit} Values per Pixel (0-255), One Channel (@p st), Patterned (RGRG..GBGB..RGRG...).
    ///
    #define  IMAGE_BAYER     (1)
    ///
    /// @brief  @unitNr{8,Bit} Values per Pixel (0-255), Three Channels (@p st:  R, @p ccmp1:  G, @p ccmp2:  B).
    ///
    #define  IMAGE_RGB       (2)
    ///
    /// @brief  @unitNr{8,Bit} Values per Pixel (0-255), Three Channels (@p st:  Y, @p ccmp1:  U, @p ccmp2:  V).
    ///
    #define  IMAGE_CBCR444   (3)
    ///
    /// @brief  @unitNr{8,Bit} Values per Pixel (0-255), Three Channels (@p st:  Y, @p ccmp1:  U, @p ccmp2:  V);
    ///         U and V Channels have @p dx, @p dy and @p pitch divided by 2, which means that each @times{2,2} Y pixel
    ///         block share the same U and V pixel.
    ///
    #define  IMAGE_CBCR411   (4)
    ///
    /// @brief  @unitNr{8,Bit} Values per Pixel (0-255), Three Channels (@p st:  Y, @p ccmp1: U*, @p ccmp2: V*).
    ///
    #define  IMAGE_YUVNORM   (5)
    ///
    /// @brief  @unitNr{8,Bit} Values per Pixel (0-255), Three Channels (@p st:  I, @p ccmp1:  H, @p ccmp2:  S).
    ///
    #define  IMAGE_IHS       (6)
    ///
    /// @brief Only used Internally.
    ///
    #define  IMAGE_RGBO      (7)
    ///
    /// @brief  @unitNr{8,Bit} Values per Pixel (0-255), Two Channels (@p st: Magnitude, @p ccmp1: Direction (f. ex. @ref gradient_2x2())).
    ///
    #define  IMAGE_VECTOR    (8)
    ///
    /// @brief  @unitNr{16,Bit} Values per Pixel (0-65535), One Channel (@p st: Always cast to (U16*)).
    ///
    #define  IMAGE_GREY16    (9)
    ///
    /// @brief  @unitNr{32,Bit} Values per Pixel (0-2<sup>32</sup>), One Channel (@p st: Always cast to (U32*)).
    ///
    #define  IMAGE_GREY32    (10)
    ///
    /// @brief @unitNr{16,Bit} Values per Pixel (0-65535), Two Interleaving Channels (@p st: [Real1, Imaginary1, Real2, ...]; Always cast to (I16*)), See @ref vc_fft().
    ///
    #define  IMAGE_CMPLX16   (11)
    ///
    /// @brief  @unitNr{8,Bit} Values per Pixel (0-255), Three Channels (@p st:  Y, @p ccmp1: U, @p ccmp2: V);
    ///         U and V Channels have @p dx and @p pitch (not @p dy) divided by 2, which means that each two Y pixel
    ///         in a row share the same U and V pixel.
    ///
    #define  IMAGE_CBCR422   (13)
    ///
    /// @brief Only used Internally.
    ///
    #define  IMAGE_YCBYCR    (14)

   /**@}@endcond*//*---End of Types of Image Data-----------------------------*/
  /**@}@endcond*//*---End of Pixel based Image Data---------------------------*/

  /*--*GROUP*-----------------------------------------------------------------*/
  // Documentation Text in 'run_length_encoded_images_text.dox'
  /**
  *@cond ( vclib || extlib )
  *
  * @defgroup runLengthEncodedImages   Run Length encoded Images (RLC)
  * @{
  */
   /*--*GROUP*----------------------------------------------------------------*/
   // Documentation Text in 'segment_label_code_text.dox'
   /**
   *@cond ( vclib || extlib )
   *
   * @defgroup segmentLabelCode   Segment Label Code (SLC)
   * @{
   */
   /**@}@endcond*//*---End of Segment Label Code (SLC)------------------------*/
  /**@}@endcond*//*---End of Run Length encoded Images (RLC)------------------*/

 /**@}@endcond*//*---End of Images--------------------------------------------*/

 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup featureDataHolder   Features
 *
 * @image  html   rlc_ftr.png
 * @image  latex  rlc_ftr.pdf
 *
 *   Determination of object attributes is based on run length encoded
 *   images and will be done as follows:
 *
 *   1. The shape of the desired object is seperated from its background
 *      by an implicit binarization done at the transformation process of the
 *      (preprocessed) pixel image to its run length encoded representation (RLC).
 *   2. Pixel data in RLC will be collated to objects.
 *   3. Object features will be calculated and provided to the program.
 *
 *   Determinable features of a pixel object are its
 *
 *   - Area Size.
 *   - Position of the Mass Center.
 *   - Bounding Box.
 *   - Contour Pixel Position as Contour Inspection Starting Point (@ref contour8()).
 *   - Binary Color to locate several Objects.
 *@cond ( extlib )
 *   - Eccentricity.
 *   - Number of Holes.
 *   - Inclusion Properties.
 *   - Subpixel accurate Mass Center Position.
 *   - Centralized Moments.
 *   - Hu Moments.
 *   - Angle.
 *@endcond
 *
 * @{
 */

 #ifdef LARGER_IMAGES
  /**  @brief  Features of Unlabelled RLC.                     */
  typedef struct
  {
    long  area;            /*!<  Object Pixel Count.                                        */
    long  x_center;        /*!<  Center of Gravity, X Coordinate.                           */
    long  y_center;        /*!<  Center of Gravity, Y Coordinate.                           */
    int   x_min;           /*!<  Smallest X Coordinate at Object (Bounding Box).            */
    int   x_max;           /*!<  Largest  X Coordinate at Object (Bounding Box).            */
    int   y_min;           /*!<  Smallest Y Coordinate at Object (Bounding Box).            */
    int   y_max;           /*!<  Largest  Y Coordinate at Object (Bounding Box).            */
    int   x_lst;           /*!<  Rightmost X Coordinate at Object of Lowermost Object Line. */
  } feature;

  /**  @brief  Features of Labelled RLC Objects.               */
  typedef struct
  {
    long  area;            /*!<  Object Pixel Count.                                        */
    long  x_center;        /*!<  Center of Gravity, X Coordinate.                           */
    long  y_center;        /*!<  Center of Gravity, Y Coordinate.                           */
    int   x_min;           /*!<  Smallest X Coordinate at Object (Bounding Box).            */
    int   x_max;           /*!<  Largest  X Coordinate at Object (Bounding Box).            */
    int   y_min;           /*!<  Smallest Y Coordinate at Object (Bounding Box).            */
    int   y_max;           /*!<  Largest  Y Coordinate at Object (Bounding Box).            */
    int   x_lst;           /*!<  Rightmost X Coordinate at Object of Lowermost Object Line. */
    int   color;           /*!<  Object Color.                                              */
    float x_cf;            /*!<  Center of Gravity, X Coordinate, float.                    */
    float y_cf;            /*!<  Center of Gravity, Y Coordinate, float.                    */
  } ftr;
 #else
  /**  @brief  Features of Unlabelled RLC.                     */
  typedef struct
  {
    U32   area;             /*!<  Object Pixel Count.                                        */
    U32   x_center;         /*!<  Center of Gravity, X Coordinate.                           */
    U32   y_center;         /*!<  Center of Gravity, Y Coordinate.                           */
    I32   x_min;            /*!<  Smallest X Coordinate at Object (Bounding Box).            */
    I32   x_max;            /*!<  Largest  X Coordinate at Object (Bounding Box).            */
    I32   y_min;            /*!<  Smallest Y Coordinate at Object (Bounding Box).            */
    I32   y_max;            /*!<  Largest  Y Coordinate at Object (Bounding Box).            */
    I32   x_lst;            /*!<  Rightmost X Coordinate at Object of Lowermost Object Line. */
  } feature;

  /**  @brief  Features of Labelled RLC Objects.               */
  typedef struct
  {
    U32   area;             /*!<  Object Pixel Count.                                        */
    U32   x_center;         /*!<  Center of Gravity, X Coordinate.                           */
    U32   y_center;         /*!<  Center of Gravity, Y Coordinate.                           */
    I32   x_min;            /*!<  Smallest X Coordinate at Object (Bounding Box).            */
    I32   x_max;            /*!<  Largest  X Coordinate at Object (Bounding Box).            */
    I32   y_min;            /*!<  Smallest Y Coordinate at Object (Bounding Box).            */
    I32   y_max;            /*!<  Largest  Y Coordinate at Object (Bounding Box).            */
    I32   x_lst;            /*!<  Rightmost X Coordinate at Object of Lowermost Object Line. */
    I32   color;            /*!<  Object Color.                                              */
    F32   x_cf;             /*!<  Center of Gravity, X Coordinate, float.                    */
    F32   y_cf;             /*!<  Center of Gravity, Y Coordinate, float.                    */
  } ftr;
 #endif



  /*--*GROUP*-----------------------------------------------------------------*/
  /**
  *@cond  ( extlib )
  *
  * @defgroup momentDataHolder   Moments
  *
  * @image  html   moments_anders.png
  * @image  latex  moments_anders.pdf
  *
  *   For binary images it is possible to describe the shape and orientation
  *   of objects by a concept called "moments". Some of the moments have a
  *   physical interpretation, like the center of gravity, the moment and the
  *   principal axis of inertia.
  *
  * @{
  */

   #ifndef _FLIB_h
    #define MAXPREC 5    /*!< Maximum Number of Words for Multi-Precision Arithmetic Number. */
    /**  @brief (FLib: Multi-Precision Arithmetic Number)
    *
    *  Further Documentation in Flib.
    */
    typedef struct      /* original definition in        */
    {                   /* flib.h                        */
    I32  precision;     /*!< Precision of the Number.    */
    U32  v[MAXPREC];    /*!< Number Data.                */
    }  BITN;
   #endif

   /**
   * @brief  Centralized Moments.
   *
   *  Not necessary to be used directly.
   */
   /*
   *   @f[
   *      \mu_{i,j} = \textrm{\bf{E}}\left[\qquad (X - \textrm{\bf{E}}[X])^i \qquad (Y - \textrm{\bf{E}}[Y])^j \qquad \right]
   *   @f]
   */
   typedef struct
   {
    BITN  mu00;             /*!<  order 0  */
    BITN  mu10;             /*!<  order 1  */
    BITN  mu01;             /*!<  order 1  */
    BITN  mu20;             /*!<  order 2  */
    BITN  mu11;             /*!<  order 2  */
    BITN  mu02;             /*!<  order 2  */
    BITN  mu30;             /*!<  order 3  */
    BITN  mu21;             /*!<  order 3  */
    BITN  mu12;             /*!<  order 3  */
    BITN  mu03;             /*!<  order 3  */
   } moment;

  /**@}@endcond*//*---End of Moments------------------------------------------*/

  /*--*GROUP*-----------------------------------------------------------------*/
  /**
  *@cond  ( extlib )
  *
  * @defgroup contourCode   Contour Code
  *
  *   The contour code (CC) is a method for storing one-dimensional contour
  *   data of (closed) object contours or edge data (not closed). Instead of
  *   storing the @a x and @a y coordinates of all contour pixels, the contour
  *   code stores a differential 3 bit information, indicating the direction
  *   of movement from one pixel to the next in the contour list. With this
  *   data structure only the @a x and @a y coordinates of the starting point
  *   must be given in order to reconstruct the contour.
  *
  *    \image html  contour_code.png
  *    \image latex contour_code.pdf
  *
  *    CC Address (U32*)  |   CC Meaning
  *   --------------------|-----------------------------------------------------
  *   @p pCC  + 0         |   Count of Contour Pixels.
  *   @p pCC  + 1         |   CC Status (See Text).
  *   @p pCC  + 2         |   X Coordinate of Start Pixel.
  *   @p pCC  + 3         |   Y Coordinate of Start Pixel.
  *   @p pCC  + 4, Byte 0 |   1. Movement Direction.
  *   @p pCC  + 4, Byte 1 |   2. Movement Direction.
  *   @p pCC  + 4, Byte 2 |   3. Movement Direction.
  *   @p pCC  + 4, Byte 3 |   4. Movement Direction.
  *   @p pCC  + 5, Byte 0 |   5. Movement Direction.
  *   ...                 |   ...
  *
  *   The movement directions are encoded by the following Values:
  *
  *    Movement Direction |  Encoding Value
  *    ------------------:|:-------------------
  *    Up                 | 0
  *    Up Right           | 1
  *    Right              | 2
  *    Down Right         | 3
  *    Down               | 4
  *    Down Left          | 5
  *    Left               | 6
  *    Up Left            | 7
  *
  *   @section ccStatus  CC status
  *
  *   The generation of contour code may terminate due to different stop
  *   conditions. If an object contour is followed, the code generation
  *   will usually stop when the starting pixel is reached in the same
  *   direction (Pixels may be in the contour list more than once, but
  *   only once for each direction). The code generation will also stop,
  *   if a corner of the image variable is reached or if the space for
  *   the contour list is exhausted.
  *
  *   The stop condition is stored in the CC status word in the header
  *   of the contour code according to the following table:
  *
  *   Bit Nr. Set  | Meaning
  *   ------------:|:-------------------------------------------------------
  *         0 ( 1) |  Closed Contour (End Pixel = Starting Pixel / Same Direction).
  *         1 ( 2) |  Contour stops at Left  Corner of Image Variable.
  *         2 ( 4) |  Contour stops at Right Corner of Image Variable.
  *         3 ( 8) |  Contour stops at Upper Corner of Image Variable.
  *         4 (16) |  Contour stops at Lower Corner of Image Variable.
  *         5 (32) |  Space Exhausted (CC Lenght > @p lng).
  *
  *   Some of the conditions may be true at the same time,
  *   for example, contour stops at the left upper corner of the image
  *   variable. In this case the individual codes will be added
  *   (in example: 2+8 = 10).
  *
  *
  * @{
  */

  /**@}@endcond*//*---End of Contour Code-------------------------------------*/

  /*--*GROUP*-----------------------------------------------------------------*/
  /**
  *@cond  ( vclib )
  *
  * @defgroup pixelAddressLists   Pixel/Address Lists
  *
  *   Pixel lists store a sequence of pixel posititons in memory.
  *   Since the count of stored pixels is not provided, one must always pass the
  *   pixel count to processing functions. For example, contour code can be
  *   converted into such a pixel list by the function @ref ccxy() allowing to
  *   do some statistics on that data. The structure is as follows:
  *
  *   PL Address (I32*)  |  Content
  *   ------------------:|:-----------------------------
  *   pXY + 0            |  @a X Coordinate of First Pixel.
  *   pXY + 1            |  @a Y Coordinate of First Pixel.
  *   pXY + 2            |  @a X Coordinate of Second Pixel.
  *   pXY + 3            |  @a Y Coordinate of Second Pixel.
  *   ...                |  ...
  *
  *   Address Lists are much like pixel lists, they also do not provide the address
  *   count, hence it must be passed as parameter. For example, they are used
  *   between a shape defining function, like @ref frame() and the pixel drawing
  *   function, like @ref FL_wpset_U8(). The structure is as follows:
  *
  *   AL Address (Various*)  |  Content
  *   ----------------------:|:-----------------------------
  *   pAL + 0                |  First Address Pointer.
  *   pAL + 1                |  Second Address Pointer.
  *   pAL + 2                |  Third Address Pointer.
  *   ...                    |  ...
  *
  *
  * @{
  */

  /**@}@endcond*//*---End of Pixel/Address Lists------------------------------*/

 /**@}@endcond*//*---End of Features------------------------------------------*/



 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup geometricalStructures   Geometrical Concepts
 *
 *  @image  html   vcline.png
 *  @image  latex  vcline.pdf
 *
 *   The basic data structures for geometric processing are @ref point
 *   and @ref vcline. The @ref vcline uses the so-called standard form
 *   \f$(\texttt{cx} * x) + (\texttt{cy} * y) - \texttt{b}    = 0\f$.
 *   For lines the normal vector (@p cx, @p cy) should be normalized to 1,
 *   although the vcline-struct still defines a line if this is not the case:
 *   @p b cannot be used as the distance from the origin to the line and some
 *   trigonometric functions could have some problems. Please be aware, that
 *   even in the case of a normalized vector, the representation is not unique,
 *   since all values could be replaced by their negative, describing the very
 *   same line. All functions in this chapter use floating-point values and
 *   floating-point calculations.
 *
 *   Angles
 *
 *   There are different definitions for angles in VCLIB:
 *
 *   (1) Angles in radiants (rad). These are mostly used for the mathematical functions
 *   using sin() and cos(). Please note that an increasing value for a radiant
 *   angle corresponds to a CLOCKWISE rotation. The reason for this deviation
 *   from mathematical standards is the fact that the y-vector of the coordinate
 *   system points downward.
 *
 *   (2) Angles in degrees (deg)
 *
 *   (3) Angles in U8 notation (ang8) ranging from [0..255] with a resolution
 *       of approx. 1.4 degrees.
 *
 *   (4) Angles in U16 notation (ang16) ranging from [0..65535]	with a resolution
 *       of 0.0055 degrees. This is a variant of (3) and is mainly used for
 *       angles in Hough Transform related functions to increase the accuracy.
 *
 *
 *   \image html  gradient_values.png
 *   \image latex gradient_values.pdf
 *
 * @{
 */

  /**
  * @brief  2D  Coordinate Point or Vector.
  */
  typedef struct
  {
    F32  x;  /*!<     The float Coordinates/Vector Entries. */
    F32  y;  /*!<     The float Coordinates/Vector Entries. */
  } point;

  /**
  * @brief  2D Line (Normal Form).
  */
  typedef struct
  {
    F32  cx;  /*!< float Parameters for the Line in Normalized Vector Form. */
    F32  cy;  /*!< float Parameters for the Line in Normalized Vector Form. */
    F32  b;   /*!< float Parameters for the Line in Normalized Vector Form \f$(\texttt{cx} * x) + (\texttt{cy} * y) - \texttt{b}    = 0\f$. */
  } vcline;

 /**@}@endcond*//*---End of Geometrical Concepts------------------------------*/



 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup errorTypes   Error Types
 *
 * @note  Additionally to documented return values any library function may
 *        return the Error Code @ref ERR_LICENCE. For more information see
 *        @ref init_licence().
 *
 *
 * @{
 */

  #define  ERR_NONE        ( 0  )    /*!<@brief  No Error                     */ 
  #define  ERR_FORMAT      (-1  )    /*!<@brief  Image Format Error           */ 
  #define  ERR_TYPE        (-2  )    /*!<@brief  Image Type Error             */ 
  #define  ERR_MEMORY      (-3  )    /*!<@brief  Out of Memory                */ 
  #define  ERR_PARAM       (-4  )    /*!<@brief  Parameter Error              */ 
  #define  ERR_LICENCE     (-5  )    /*!<@brief  Licence required             */ 
  #define  ERR_SINGULAR    (-6  )    /*!<@brief  Singular Value detected      */ 
  #define  ERR_OVERRUN     (-7  )    /*!<@brief  Memory Overrun               */ 
  #define  ERR_FIO         (-8  )    /*!<@brief  File I/O Error               */ 
  #define  ERR_BOUNDS      (-9  )    /*!<@brief  Out of Bounds                */ 
  #define  ERR_INIT        (-10 )    /*!<@brief  Missing Initialisation       */ 
  #define  ERR_LOWPREC     (-11 )    /*!<@brief  Max. Precision not available */ 
  #define  ERR_OPEN        (-19 )    /*!<@brief  Open Error                   */ 
  #define  ERR_MODEL       (-51 )    /*!<@brief  Model does not fit Licence   */ 
  #define  ERR_JPG         (-71 )    /*!<@brief  Unrecognized JPEG Token      */ 
  #define  ERR_CORR        (-100)    /*!<@brief  Object not found / vc_corr   */ 
  #define  ERR_FSIGN       (-201)    /*!<@brief  File Signature Error         */ 
  #define  ERR_INCONS      (-202)    /*!<@brief  File inconsistent            */ 
  #define  ERR_FFMT        (-203)    /*!<@brief  Unsupported File Format      */ 
  #define  ERR_READ        (-204)    /*!<@brief  Read Error                   */ 
  #define  ERR_WRITE       (-205)    /*!<@brief  Write Error                  */ 
  #define  ERR_HOUGH0      (-500)    /*!<@brief  Hough Line Overrun           */ 
  #define  ERR_HOUGH1      (-501)    /*!<@brief  Hough Maximum Iterations     */ 

  /* RLC error types                                                          */ 

  #define  ERR_SLC         (-200)    /*!<@brief  RLC is missing SLC           */ 
  #define  ERR_RLC_ONOVR   (-201)    /*!<@brief  RLC Object Number Overrun    */ 
  #define  ERR_SLC_INCON   (-202)    /*!<@brief  SLC is inconsistent          */ 
  #define  ERR_RLCFMT      (-203)    /*!<@brief  RLC Format Error             */ 
  #define  ERR_RLC_MEM     (-204)    /*!<@brief  Memory Allocation Error      */ 
  #define  ERR_RLC_SIZE    (-205)    /*!<@brief  RLC Size Overrun             */ 

 /**@}@endcond*//*---End of Error Types---------------------------------------*/



 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib || collib )
 * @defgroup libraryAndLicenceValidation   Library and Licence Validation
 *
 *   Functions for initializing and identifying the used lib versions.
 *
 * @{
 */

  I32  init_licence(char *code);

  void  GetLibVer_VCLIB  (int *Version, int *SubVersion);
  void  GetLibVer_EXTLIB (int *Version, int *SubVersion);
  void  GetLibVer_COLLIB (int *Version, int *SubVersion);
  void  GetLibVer_DLL    (int *Version, int *SubVersion);

  char *GetLibVerString_VCLIB  (void);
  char *GetLibVerString_EXTLIB (void);
  char *GetLibVerString_COLLIB (void);
  char *GetLibVerString_DLL    (void);

 /**@}@endcond*//*---End of Library and Licence Validation--------------------*/
/**@}@endcond*//*---End of Elementary Structs, Macros and Types---------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  pixelToPixelOperators   Pixel to Pixel Operators
*
*  Functions in this group use only the input data at the same pixel position as the
*  output, for example functions which add two image's grey values, or do an image
*  binarisation.
*
*  Functions which output geometrc transformations of the input image can be
*  found at @ref pixelImageTransformation
*
* @{
*/

 I32   copy(image *a, image *b);

 void  bin0(image *src, image *dest, I32 thr, I32 bl, I32 wt, void (*fc)());
 #define  binarize(s, d, t, b, w)   bin0(s, d, t, b, w, (void (*)())FL_SetBin_U8P_U8P)
 #define  PaintWhite(s, d, t, w)    bin0(s, d, t, w, 0, (void (*)())FL_SetIfGE_U8P_U8P)
 #define  PaintBlack(s, d, t, b)    bin0(s, d, t, b, 0, (void (*)())FL_SetIfLT_U8P_U8P)
 #define  OR_image(s, d, c)         bin0(s, d, c, 0, 0, (void (*)())FL_OR_I8P_I8)

 void  img2(image *a, image *b, image *c, void (*func)(), I32 q);
 #define  add2(a, b, c, sh)         img2(a, b, c, (void (*)())FL_add2f, sh)
 #define  sub2(a, b, c)             img2(a, b, c, (void (*)())FL_sub2f, 0)
 #define  subx2(a, b, c, offs)      img2(a, b, c, (void (*)())FL_sub2x, offs)
 #define  suby2(a, b, c)            img2(a, b, c, (void (*)())FL_sub2y, 0)
 #define  mul2(a, b, c, sh)         img2(a, b, c, (void (*)())FL_mul2f, sh)
 #define  max2(a, b, c)             img2(a, b, c, (void (*)())FL_max2f, 0)
 #define  min2(a, b, c)             img2(a, b, c, (void (*)())FL_min2f, 0)
 #define  and2(a, b, c)             img2(a, b, c, (void (*)())FL_and2f, 0)
 #define  or2(a, b, c)              img2(a, b, c, (void (*)())FL_or2f,  0)
 #define  xor2(a, b, c)             img2(a, b, c, (void (*)())FL_xor2f, 0)

 I32   equalize(image *src, image *dst);
 I32   look(image *a, image *b, U32 table[]);
 I32   lookfalse(image *src, image *dst, U8 red[], U8 grn[], U8 blu[]);
 void  SetFalseColorLUT(U8 *red, U8 *grn, U8 *blu);

/**@}@endcond*//*---End of Processing Pixel to Pixel--------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  neighbourhoodOperators   Neighbourhood Operators
*
*  Functions in this group use more than the input data at same pixel position to
*  calculate an output pixel value, for example functions which do some filtering,
*  such as sobel or pyramid, or convolution.
*
* @{
*/

 void  imgf(image *a, image *b, void (*func)());
 

 
 /*--*GROUP*-------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup  rankFilter   Rank Filter
 *
 *
 * @{
 */
  I32   maxMxN(image *src, image *dst, I32 mx, I32 my);
  I32   minMxN(image *src, image *dst, I32 mx, I32 my);

  I32   median3(image *src, image *dst);
  I32   pmed5x5(image *src, image *dst);
  I32   pmed7x7(image *src, image *dst);

  I32   median1xN(image *src, image *dst, I32 my);
  I32   medianNx1(image *src, image *dst, I32 mx);

 /**@}@endcond*//*---End of Rank Filter ---------------------------------------*/
 
 /*--*GROUP*-------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup  highPassFilter   High Pass Filter
 *
 *
 * @{
 */

    void  robert(image *a, image *b);
    
    #define  sobel(a, b)         imgf(a, b, (void (*)())FL_3x3_SOB_U8P)
    #define  sobelh(a, b)        imgf(a, b, (void (*)())FL_3x3_SOBH_U8P)
    #define  sobelv(a, b)        imgf(a, b, (void (*)())FL_3x3_SOBV_U8P)
    #define  sobelf(count, p1, p2, p3, pr)    FL_3x3_SOB_U8P(count, p1, p2, p3, pr)    /* for old style programming */
    #define  laplace(a, b)       imgf(a, b, (void (*)())FL_3x3_LAP_U8P)
    
    I32  box_DoG_filter(image *src, image *dst, I32 nx, I32 ny, I32 kx, I32 ky);
    I32  box_DoG_filter2(image *src, image *dst, I32 addC, I32 force, I32 nx, I32 ny, I32 kx, I32 ky);

    //Wrong Location: mx() and mn() are Rank Filter (Doxygen Problem).
    #define  mx(a, b)            imgf(a, b, (void (*)())FL_3x3_MAX_U8P)
    #define  mn(a, b)            imgf(a, b, (void (*)())FL_3x3_MIN_U8P)
 /**@}@endcond*//*---End of High Pass Filter ----------------------------------*/

 /*--*GROUP*-------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup  lowPassFilter   Low Pass Filter
 *
 *
 * @{
 */

  I32   gauss(image *src, image *dst, F32 sigma);
  I32   gauss_hor(image *src, image *dst, F32 sigma);
  I32   gauss_ver(image *src, image *dst, F32 sigma);
  I32   gauss_fir(image *src, image *dst, F32 sigma);

  I32   boxavg(image *src, image *dst, I32 kx, I32 ky);
  void  ii_boxavg(image *integral, image *dst, I32 kx, I32 ky);
  
  #define  avg_3x3(a, b)           imgf(a, b, (void (*)())FL_3x3_AVG_U8P)

  I32   avg(image *a, image *b, I32 kx, I32 ky,void (*func)(), I32 v);
  #define  avgm(a, b, kx, ky)      avg(a, b, kx, ky, (void (*)())0, 0)
  #define  maskx(a, b, kx, ky, v)  avg(a, b, kx, ky, (void (*)())FL_sub2x, v)
  #define  masky(a, b, kx, ky)     avg(a, b, kx, ky, (void (*)())FL_sub2y, 0)

  I32   avg2(image *a, image *b, I32 kx, I32 ky, void (*func)(), I32 v); /* image not centered    */
  #define  avgm2(a, b, kx, ky)     avg2(a, b, kx, ky, (void (*)())0, 0)
  #define  maskx2(a, b, kx, ky, v) avg2(a, b, kx, ky, (void (*)())FL_sub2x, v)
  #define  masky2(a, b, kx, ky)    avg2(a, b, kx, ky, (void (*)())FL_sub2y, 0)

  I32   avg_1xN(image *a, image *b, I32 ky, void (*func)(), I32 v);
  #define  avgm_1xN(a, b, ky)      avg_1xN(a, b, ky, (void (*)())0, 0)
  #define  maskx_1xN(a, b, ky, v)  avg_1xN(a, b, ky, (void (*)())FL_sub2x, v)
  #define  masky_1xN(a, b, ky)     avg_1xN(a, b, ky, (void (*)())FL_sub2y, 0)

  I32   avg2_1xN(image *a, image *b, I32 ky, void (*func)(), I32 v);     /* image not centered    */
  #define  avgm2_1xN(a, b, ky)     avg2_1xN(a, b, ky, (void (*)())0, 0)
  #define  maskx2_1xN(a, b, ky, v) avg2_1xN(a, b, ky, (void (*)())FL_sub2x, v)
  #define  masky2_1xN(a, b, ky)    avg2_1xN(a, b, ky, (void (*)())FL_sub2y, 0)

  I32   isef(image *src, image *dst, F32 b);
  I32   isef_hor(image *src, image *dst, F32 b);
  I32   isef_ver(image *src, image *dst, F32 b);

  //internal, undocumented functions
  void  med_3x3_cn(I32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);
  void  med_3x3_ee(I32 count, U8 *p1, U8 *p2, U8 *p3, U8 *pr);
 /**@}@endcond*//*---End of Low Pass Filter-----------------------------------*/




 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup  convolutions   Convolutions
 *
 *
 * @{
 */
  I32   ff3(image *a, image *b, I32 c[3][3], I32 sh);
  I32   ff5(image *a, image *b, I32 c[5][5], I32 sh);
  I32   ff5y(image *a, image *b, I32 *h, I32 *v, I32 sh);
  I32   ff7y(image *a, image *b, I32 *h, I32 *v, I32 sh);

 /**@}@endcond*//*---End of Convolutions--------------------------------------*/



/**@}@endcond*//*---End of Neighbourhood Operators ---------------------------*/



  

/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  correlationAndMatching   Correlation and Matching
*
*  Functions in this group allow the localisation of objects in images.
*
* @{
*/
 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup  correlations   Correlations
 *
 *
 * @{
 */


  I32  vc_corr0(image *a, image *b, I32 mcn, I32 mcr, I32 *x0, I32 *y0);
  #define  vc_corr1(a, b, c, d, e, f)    vc_corr0(a, b, c, d, e, f)              /* function replaced        */
  I32  vc_corr2(image *a, image *b, I32 mcn, I32 mcr, I32 *x0, I32 *y0);         /* extended kernel version  */
  I32  vc_corr3(image *src, image *smp, image *dst32, I32 mcn);                  /* IMAGE_GREY32 output      */
  I32  contr3(image *smp);                                                       /* inverse contrast function*/
  F32  corrcheck(image *a, image *b);                                            /* calculate corr value     */

  I32  NormalizedCorrelation(image *SearchX, image *Pattern, I32 mode,            /* subpixel accurate NC     */
                              F32 MinContr, F32 MinCorr, F32 *Contr, F32 *Corr,   /* mode = 0: pixel accurate */
                              F32 *PosX, F32 *PosY);                              /* mode = 1: subpixel       */

  I32  NormalizedCorrelationI32(image *src, image *ref, F32 MinContr,             /* large kernels "ref"      */
                                I32 *Corr, F32 *Contr, I32 *PosX, I32 *PosY);     /* with minimum contrast    */
  #define  NC(src, ref, PosX, PosY, Corr)    NormalizedCorrelationI32(src, ref, 0, Corr, NULL, PosX, PosY)

  I32  BarCorrV(image *SearchX, image *Pattern, F32 MinContr,                     /* correlation with         */
                I32 *Corr, F32 *Contr, I32 *PosX);                                /*    vertical projection   */
  #define  BCV(src, ref, PosX, Corr)         BarCorrV(src, ref, 0.0, Corr, NULL, PosX)

  I32  BarCorrV2(image *SearchX, image *Pattern, F32 MinContr,                    /* correlation with         */
                 I32 *Corr, F32 *Contr, I32 *PosX, I32 *PosY);                    /* vertical projection      */
  #define  BCV2(src, ref, PosX, PosY, Corr)  BarCorrV2(src, ref, 0.0, Corr, NULL, PosX, PosY)


  //internal, undocumented functions
  U32  corr(I32 dx, I32 kx, I32 ky, U8 *img, U8 *sample, I32 img_pitch, I32 sample_pitch);
  I32  corr_subpixel(float *corr3x3, float *x0, float *y0, float *corr);


 /**@}@endcond*//*---End of Correlations--------------------------------------*/

/**@}@endcond*//*---End of Correlation and Matching --------------------------*/

/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  processingFromMultiplePixelDataToValues   Processing from Multiple Pixel Data to Values
*
*  Functions in this group use a larger number of pixels of the source image in
*  order to calculate a value or a number of values. Examples are projections
*  and average calculations.
*
* @{
*/

 U32   arx(image *a, I32 thr);
 U32   arx2(image *a, I32 th1, I32 th2);
 I32   mean(image *a);
 U32   variance(image *a);
 I32   contrast(image *a);
 I32   contrastF(image *src, F32 *avg, F32 *contrast);
 U32   focus(image *a, I32 sh);
 U32   SumImage (image *Src);
 U40   ProdImage (image *Src1, image *Src2);
 I32   histo(image *a, U32 hist[]);
 void  projh(image *a, U32 result[]);
 void  projv(image *a, U32 result[]);

 //internal, undocumented functions
 U32   arxf(I32 dx, U8 *p, I32 thr);

/**@}@endcond*//*---End of Processing from Multiple Pixel Data to Values------*/

/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  ColorImageProcessing   Color Image Processing
*
*  Functions in this group perform the processing and evaluation of color
*  images in the various color image formats.
*
* @{
*/
 I32  init_color_table(U32 red, U32 green, U32 blue, U8 table[]);
 I32  WhiteBalanceValues (image *bayer, I32 *red, I32 *green, I32 *blue);
 void StoreRGB_Centred  (U8 *ResultRed, U8 *ResultGreen, U8 *ResultBlue, I32 Line, I32 dx, image *RGB);
 void StoreGrey_Centred (U8 *ResultRed, U8 *ResultGreen, U8 *ResultBlue, I32 Line, I32 dx, image *RGB);
 void StoreRGB          (U8 *Red, U8 *Green, U8 *Blue, I32 Line, I32 dx, image *RGB);
 void StoreGrey         (U8 *Red, U8 *Green, U8 *Blue, I32 Line, I32 dx, image *RGB);
 void StoreYCBCR444     (U8 *Red, U8 *Green, U8 *Blue, I32 Line, I32 dx, image *YCBCR);
 void StoreYCBCR422     (U8 *Red, U8 *Green, U8 *Blue, I32 Line, I32 dx, image *YCBCR);
 void StoreYCBCR411     (U8 *Red, U8 *Green, U8 *Blue, I32 Line, I32 dx, image *YCBCR);
 void StoreRGBO         (U8 *Red, U8 *Green, U8 *Blue, I32 Line, I32 dx, image *RGBO, image *OVL);
 void StoreGreyRGBO     (U8 *Red, U8 *Green, U8 *Blue, I32 Line, I32 dx, image *RGBO, image *OVL);

 I32  RGB_YCbCr(image *rgb, image *ycbcr);
 I32  RGB_YCbCr411(image *rgb, image *ycbcr);        /* YCbCr 4:1:1 */
 I32  RGB_YCbCr422(image *rgb, image *ycbcr);        /* YCbCr 4:2:2 */
 I32  RGB_IHS(image *rgb, image *ihs);

 I32  YCbCr_RGB(image *ycbcr, image *rgb);
 I32  YCbCr_RGB411(image *ycbcr, image *rgb);        /* YCbCr 4:1:1 */
 I32  YCbCr_RGB422(image *ycbcr, image *rgb);        /* YCbCr 4:2:2 */
 I32  YCbCr_NORM(image *ycbcr, image *yuvnrm);

 I32  NORM_YCbCr(image *yuvnrm, image *ycbcr);

 I32  clut_bayer(image *bayer1, image *bayer2, U8 table[]);
 I32  BayerToRGB_5x5(image *bayer, image *rgb, image *ovl, void *(*function)());
 #define  BayerToRGB(bayer, rgb)              BayerToRGB_5x5(bayer, rgb, NULL, (void *(*)())StoreRGB)
 #define  BayerToRGB_Centred(bayer, rgb)      BayerToRGB_5x5(bayer, rgb, NULL, (void *(*)())StoreRGB_Centred)
 #define  BayerToGrey(bayer, grey)            BayerToRGB_5x5(bayer, grey, NULL, (void *(*)())StoreGrey)
 #define  BayerToGrey_Centred(bayer, grey)    BayerToRGB_5x5(bayer, grey, NULL, (void *(*)())StoreGrey_Centred)
 #define  BayerToYCbCr(bayer, ycbcr)          BayerToRGB_5x5(bayer, ycbcr, NULL, (void *(*)())StoreYCBCR444)
 #define  BayerToYCbCr411(bayer, ycbcr)       BayerToRGB_5x5(bayer, ycbcr, NULL, (void *(*)())StoreYCBCR411)
 #define  BayerToYCbCr422(bayer, ycbcr)       BayerToRGB_5x5(bayer, ycbcr, NULL, (void *(*)())StoreYCBCR422)
 #define  BAYER_RGBO(bayer, ovl, rgbo)        BayerToRGB_5x5(bayer, rgbo, ovl, (void *(*)())StoreRGBO)
 #define  BAYER_GREY_RGBO(bayer, ovl, rgbo)   BayerToRGB_5x5(bayer, rgbo, ovl, (void *(*)())StoreGreyRGBO)

 I32  hue_histo(U32 hist[], U32 hue[], U32 sat[]);
 I32  hue_histo_init(I32 u0, I32 v0);
 void hue_histo_deinit();
 I32  color_histo(image *img, U32 hist[]);
 I32  color_classify(image *src, image *dst, U8 table[]);
 I32  color_sector(U8 *table, I32 hue, I32 tol, I32 minsat, I32 maxsat, I32 value);
 I32  display_chisto(image *map, U32 hist[]);

/**@}@endcond*//*---End of Processing Color Image Processing -----------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  processingFromValuesToPixelData   Processing from Values to Multiple Pixels
*
*  Functions in this group use a few values to calculate image pixel data,
*  for example functions which set all pixel to the same value, or generate
*  test patterns.
*
*  Functions drawing geometrical shapes can be found at @ref drawingPixelData.
*
* @{
*/

 I32  set(image *a, I32 x);
 I32  vset(image *a, I32 mag, I32 dir);
 I32  cset(image *rgb, I32 cc0, I32 cc1, I32 cc2);

 I32  mask_frame(image *src, I32 sx0, I32 sx1, I32 sy0, I32 sy1, I32 color);     /* draw a frame with color and size inside an image */
 I32  mask_polygon(image *psImgIn, image *psImgOut, U8 color, U32 u32CornerCount, I32 *pi32PxLst, U8 u8OutputSizeToSelectionIff1);

/**@}@endcond*//*---End of Processing from Values to Multiple Pixels----------*/
 // Not yet released functions:



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( extlib )
*
* @defgroup  geometricImageTransformation   Geometric Image Transformation
*
*  Functions in this group output geometric transformations of the input image,
*  for example functions that rotate, skew, translate images.
*
* @{
*/


 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( extlib )
 *
 * @defgroup  affineImageTransformations   Affine Image Transformations And Scaling
 *
 *   Affine transformations map pixels to different pixels using a
 *   linear mapping. All affine transformations can be specified by the following
 *   formula:
 *
 *   \f[
 *        \left( \begin{array}{c} x_\textrm{dst}     \\ y_\textrm{dst}    \end{array} \right)
 *      = \left( \begin{array}{cc} a_{00} & a_{01}   \\ a_{10} & a_{11}   \end{array} \right)
 *        \left( \begin{array}{c} x_\textrm{src}     \\ y_\textrm{src}    \end{array} \right)
 *      + \left( \begin{array}{c} t_0                \\ t_1               \end{array} \right),
 *   \f]
 *
 *   where
 *
 *   - \f$ (x_\textrm{src}, y_\textrm{src})^T \f$ is the coordinate of the source pixel,
 *   - \f$ (x_\textrm{dst}, y_\textrm{dst})^T \f$ is the coordinate of the destination pixel,
 *   - \f$  a_{00}, a_{01}, a_{10}, a_{11}\f$ are coefficients of the transformation matrix, and
 *   - \f$ (t_0,t_1)^T \f$ is the displacement vector.
 *
 *   With the above formula any combination of the following transformation may be performed:
 *
 *   - displacement (movement) parallel to x and y axis
 *   - rotation with an arbitrary rotation center point
 *   - shear
 *   - size scaling (zoom up / down) in x and y separately
 *   - mirroring
 *
 *   For the function @ref affine_image_transform() the inverse transformation
 *   matrix is used.
 *   The functions @ref calc_rotation_matrix(), @ref calc_rotation() and
 *   @ref threepoint_calculate() return the inverse matrix and
 *   vector which can than be used by the @ref affine_image_transform() routine:
 *
 *   \f{eqnarray*}{
 *        \left( \begin{array}{c} x_\textrm{src} \\ y_\textrm{src} \end{array} \right)
 *         &=  \left( \begin{array}{cc} a_{00} & a_{01}   \\ a_{10} & a_{11}   \end{array} \right)^{-1}       \left( \begin{array}{c} x_\textrm{dst}     \\ y_\textrm{dst}    \end{array} \right)
 *         &-  \left[\left( \begin{array}{cc} a_{00} & a_{01}   \\ a_{10} & a_{11}   \end{array} \right)^{-1} \left( \begin{array}{c} t_0                \\ t_1               \end{array} \right) \right]\\
 *         &=: \left( \begin{array}{cc} b_{00} & b_{01}   \\ b_{10} & b_{11}   \end{array} \right)            \left( \begin{array}{c} x_\textrm{dst}     \\ y_\textrm{dst}    \end{array} \right)
 *         &+  \left( \begin{array}{c} u_0                \\ u_1               \end{array} \right),
 *   \f}
 *
 *   where
 *   - \f$  b_{00}, b_{01}, b_{10}, b_{11}\f$ are coefficients of the inverse transformation matrix, and
 *   - \f$ (u_0,u_1)^T \f$ is the 'inverse' displacement vector.
 *
 *
 *
 * @{
 */


 I32  mirror_hor(image *src, image *dst);
 I32  mirror_ver(image *src, image *dst);
 I32  rotate90l(image *src, image *dst);        /* rotate 90 deg. counter-clockwise  */
 I32  rotate90r(image *src, image *dst);        /* rotate 90 deg. clockwise          */
 I32  rotate180(image *src, image *dst);        /* rotate 180 degrees                */
 void xshear(image *src, F32 shear, image *dst, F32 offset, U8 bgnd);
 I32  move_image_alpha(image *src, image *dst, F32 mx, F32 my, U8 bgnd);

 I32  pyramidx(image *a, image *b, void (*func)());
 I32  pyramidc(image *a, image *b, void (*func)());    /* color version for IMAGE_BAYER */
 #define  pyramid(a, b)       pyramidx(a, b, (void (*)())FL_2x2_PAV_U8P_U8P)
 #define  pyr_max(a, b)       pyramidx(a, b, (void (*)())FL_2x2_PMX_U8P_U8P)
 #define  pyr_min(a, b)       pyramidx(a, b, (void (*)())FL_2x2_PMN_U8P_U8P)
 #define  pyr_col(a, b)       pyramidc(a, b, (void (*)())FL_2x2_PAC_U8P_U8P)

 void subsample(image *a, image *b, I32 rh, I32 rv);
 void zoom_up(image *a, image *b, I32 factor);

 I32  threepoint_calculate(point *p0, point *p1, point *p2, point *q0, point *q1, point *q2, F32 **a, F32 *t);
 void calc_rotation_matrix(F32 angle, F32 cx, F32 cy, F32 a[2][2], F32 t[2]);
 void calc_rotation(F32 angle, point *p, point *target_p, F32 a[2][2], F32 t[2]);
 I32  affine_image_transform(image *src, image *dst, F32 a[2][2], F32 t[2], U8 bgnd);
 I32  affine_image_transform2(image *src, image *dst, F32 a[2][2], F32 t[2], U8 bgnd);
 I32  affine_image_transform3(image *src, image *dst, F32 a[2][2], F32 t[2], U8 bgnd);
 I32  affine_pixellist_transform(I32 *src, I32 *dst, U32 pxCount, F32 a[2][2], F32 t[2]);
 I32  AffineInitTable(I32 mode);
 void AffineDeinitTable();

 I32   resample_image(image *src, image *dst, I32 (*func)());
 #define  resample_bilinear(src, dst)     resample_image(src, dst, (I32 (*)())affine_image_transform)
 #define  resample_quadratic(src, dst)    resample_image(src, dst, (I32 (*)())affine_image_transform3)

 /**@}@endcond*//*---End of Affine Image Transformations----------------------*/

 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( extlib )
 *
 * @defgroup  homographicImageTransformations   Homographic Image Transformations
 *
 *
 * @{
 */
  I32  fourpoint_calculate(point *p0, point *p1, point *p2, point *p3, point *q0, point *q1, point *q2, point *q3, F32 **a);
  I32  homography_image_transform(image *src, image *dst, F32 a[3][3], U8 bgnd);
 /**@}@endcond*//*---End of Homographic Image Transformations-----------------*/

 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( extlib )
 *
 * @defgroup  polarImageTransformations   Polar Image Transformations
 *
 *
 * @{
 */
  I32  polar_image_transform(image *src, image *dst, F32 t[2], U32 r0, U8 bgnd);
  I32  polar_image_transform2(image *src, image *dst, F32 t[2], F32 r0, U8 bgnd);
 /**@}@endcond*//*---End of Polar Image Transformations-----------------------*/

 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( extlib )
 *
 * @defgroup  lensTransformations   Lens Transformations (Undistortion)
 *
 *
 * @{
 */
  I32  lens_transform(image *src, image *dst, point *center, F32 k3, U8 bgnd);
  I32  lens_transform2(image *src, image *dst, point *center, F32 f, F32 magnification, U8 bgnd);
  I32  init_lens_transform2(F32 f, F32 magnification, I32 nr, I32 nc);
  void deinit_lens_transform2();
 /**@}@endcond*//*---End of Lens Transformations------------------------------*/

 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( extlib )
 *
 * @defgroup  tableTransformation   Table Transformation (Undistortion)
 *
 *
 * @{
 */
 I32  table_image_transform(image *src, image *tbl, image *dst, U8 bgnd);
 I32  check_transform_table(image *tbl, I32 dx, I32 dy);
 I32  fread_transform_table(char *file_name, image *table);
 /**@}@endcond*//*---End of Lens Transformations------------------------------*/

/**@}@endcond*//*---End of Pixel Image Transformation-------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  drawingPixelData   Drawing Pixel Data
*
*  Functions in this group output geometrical shapes as pixel data, for example
*  functions drawing circles, markers or lines.
*
*
* @{
*/
 I32  draw_line(image *a, F32 cx, F32 cy, F32 b, I32 col, void (*func)());
 #define  draw_lined(a, cx, cy, b, c)     draw_line(a, cx, cy, b,                         c, (void (*)())FL_wpset_U8)
 #define  draw_linex(a, cx, cy, b, c)     draw_line(a, cx, cy, b,                         c, (void (*)())FL_wpxor_U8)
 #define  draw_vclined(a, line, c)        draw_line(a, (line)->cx, (line)->cy, (line)->b, c, (void (*)())FL_wpset_U8)
 #define  draw_vclinex(a, line, c)        draw_line(a, (line)->cx, (line)->cy, (line)->b, c, (void (*)())FL_wpxor_U8)

 I32  line(image *a, I32 x1, I32 y1, I32 x2, I32 y2, I32 col, void (*func)());
 #define  lined(a, x1, y1, x2, y2, c) \
           line(a, x1, y1, x2, y2, c, (void (*)())FL_wpset_U8)
 #define  linex(a, x1, y1, x2, y2, c) \
           line(a, x1, y1, x2, y2, c, (void (*)())FL_wpxor_U8)
 #define  lineo(a, x1, y1, x2, y2) \
           line(a, x1, y1, x2, y2, 1, (void (*)())FL_wpset_U8)
 #define  linez(a, x1, y1, x2, y2) \
           line(a, x1, y1, x2, y2, 1, (void (*)())FL_wpxor_U8)
 void line0(image *a, int x1, int y1, int x2, int y2, int col, void (*func)());

 void chprint(char *str, image *a, I32 cx, I32 cy);
 void chprint1(char *str, image *a, I32 cx, I32 cy, I32 val);

 void  MarkBoxN(image *img, I32 x, I32 y, I32 dx, I32 dy, I32 col, void (*func)());
 #define  MarkBox( img, x, y, dx, dy, col)    MarkBoxN(img, x, y, dx, dy, col, (void (*)())FL_wpset_U8)
 #define  MarkBoxd(img, x, y, dx, dy, col)    MarkBoxN(img, x, y, dx, dy, col, (void (*)())FL_wpset_U8)
 #define  MarkBoxx(img, x, y, dx, dy, col)    MarkBoxN(img, x, y, dx, dy, col, (void (*)())FL_wpxor_U8)

 void MarkSolidBoxN(image *img, I32 x, I32 y, I32 dx, I32 dy, I32 col, void (*func)());
 #define  MarkSolidBox( img, x, y, dx, dy, col)    MarkSolidBoxN(img, x, y, dx, dy, col, (void (*)())FL_wpset_U8)
 #define  MarkSolidBoxd(img, x, y, dx, dy, col)    MarkSolidBoxN(img, x, y, dx, dy, col, (void (*)())FL_wpset_U8)
 #define  MarkSolidBoxx(img, x, y, dx, dy, col)    MarkSolidBoxN(img, x, y, dx, dy, col, (void (*)())FL_wpxor_U8)
 
 void  MarkCrossN(image *img, I32 x, I32 y, I32 col, I32 size, void (*func)());
 #define  MarkCross( img, x, y, col, size)    MarkCrossN(img, x, y, col, size, (void (*)())FL_wpset_U8)
 #define  MarkCrossd(img, x, y, col, size)    MarkCrossN(img, x, y, col, size, (void (*)())FL_wpset_U8)
 #define  MarkCrossx(img, x, y, col, size)    MarkCrossN(img, x, y, col, size, (void (*)())FL_wpxor_U8)

 void  draw_polygon(image *psImgOut, U32 u32CornerCount, I32 *pi32PxLst,  int col, void (*func)());
 #define  draw_polygond(a, cornercount, pxLst, col)    draw_polygon(a, cornercount, pxLst, col, (void (*)())FL_wpset_U8)
 #define  draw_polygonx(a, cornercount, pxLst, col)    draw_polygon(a, cornercount, pxLst, col, (void (*)())FL_wpxor_U8)

 void  draw_polyline(image *psImgOut, U32 u32CornerCount, I32 *pi32PxLst,  int col, void (*func)());
 #define  draw_polylined(a, cornercount, pxLst, col)    draw_polyline(a, cornercount, pxLst, col, (void (*)())FL_wpset_U8)
 #define  draw_polylinex(a, cornercount, pxLst, col)    draw_polyline(a, cornercount, pxLst, col, (void (*)())FL_wpxor_U8)

 I32  draw_circle(image *a, I32 px, I32 py, I32 rad, I32 col, void (*func)());
 #define  draw_circled(a, px, py, r, c)   draw_circle(a, px, py, r, c, (void (*)())FL_wpset_U8)
 #define  draw_circlex(a, px, py, r, c)   draw_circle(a, px, py, r, c, (void (*)())FL_wpxor_U8)

 void  ellipse(image *a, I32 col, void (*func)());
 #define  ellipsed(a, c)     ellipse(a, c,    (void (*)())FL_wpset_U8)
 #define  ellipsex(a, c)     ellipse(a, c,    (void (*)())FL_wpxor_U8)
 #define  ellipseo(a)        ellipse(a, 1,    (void (*)())FL_wpset_U8)
 #define  ellipsez(a)        ellipse(a, 1,    (void (*)())FL_wpxor_U8)

 I32   draw_histo(image *Graph, U32 *Histo, I32 size, I32 BackgndColor, I32 Color, I32 i32Mode, void (*func)());
 #define  draw_histod(Graph, Histo, size, BackgndColor, Color, i32Mode)     draw_histo(Graph, Histo, size, BackgndColor, Color, i32Mode, (void (*)())FL_wpset_U8)
 #define  draw_histox(Graph, Histo, size, BackgndColor, Color, i32Mode)     draw_histo(Graph, Histo, size, BackgndColor, Color, i32Mode, (void (*)())FL_wpxor_U8)

 I32  draw_histo_quantile(image *pOut, U32 *pu32Histo, U8 u8Percentage, U8 u8Color);
 I32  draw_histo_line(image *pOut, U8 u8Column, U8 u8Color);
 
 void  frame(image *a, I32 col, void (*func)());
 #define  framed(a, col)      frame(a, col, (void (*)())FL_wpset_U8)
 #define  framex(a, col)      frame(a, col, (void (*)())FL_wpxor_U8)
 #define  frameo(a)           frame(a,   1, (void (*)())FL_wpset_U8)
 #define  framez(a)           frame(a,   1, (void (*)())FL_wpxor_U8)

 void  dframe(image *a, I32 col, void (*func)());
 #define  dframed(a, col)     dframe(a, col, (void (*)())FL_wpset_U8)
 #define  dframex(a, col)     dframe(a, col, (void (*)())FL_wpxor_U8)
 #define  dframeo(a)          dframe(a,   1, (void (*)())FL_wpset_U8)
 #define  dframez(a)          dframe(a,   1, (void (*)())FL_wpxor_U8)

 void  marker(image *a, I32 col, void (*func)());
 #define  markerd(a, col)     marker(a, col, (void (*)())FL_wpset_U8)
 #define  markerx(a, col)     marker(a, col, (void (*)())FL_wpxor_U8)
 #define  markero(a)          marker(a,   1, (void (*)())FL_wpset_U8)
 #define  markerz(a)          marker(a,   1, (void (*)())FL_wpxor_U8)

 void  dmarker(image *a, I32 col, void (*func)());
 #define  dmarkerd(a, col)    dmarker(a, col, (void (*)())FL_wpset_U8)
 #define  dmarkerx(a, col)    dmarker(a, col, (void (*)())FL_wpxor_U8)
 #define  dmarkero(a)         dmarker(a,   1, (void (*)())FL_wpset_U8)
 #define  dmarkerz(a)         dmarker(a,   1, (void (*)())FL_wpxor_U8)


 void  dashed_U8(U32 count, U8 *ad_list[], I32 value);
 void  dotted_U8(U32 count, U8 *ad_list[], I32 value);
 void  needled_U8(U32 count, U8 *ad_list[], I32 value);

 I32  linexy(I32 dx, I32 dy, I32 *xy);
 I32  EllipseXY(I32 a,I32 b,I32 *xyc);

 //internal, undocumented functions
 void  chgf(U8 *q, I32 dx,I32 dy,I32 pitch,I32 ch_line,U8 bgnd,U8 fgnd);
/**@}@endcond*//*---End of Drawing Pixel Data---------------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
* @defgroup  imageDataPixelConversionAndInputOutput   Image Data Pixel Conversion and Input/Output
*
*  Functions in this group convert image data to another representation, for
*  example functions converting RGB valued pixel information to HSV, or encoding
*  images to JPEG.
*
* @{
*/

 I32  integral_image(image *src, image *dst);

 I32  fread_image(char *path, image *img);
 I32  fwrite_image(char *path, image *img);

 I32  ImgConvert_I16_U8(image *Src16, image *Dst);
 I32  ImgConvert_I32_U8(image *Src32, image *Dst);
 I32  get_component(image *src, image *dst, I32 component);


 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 * @defgroup  jpegRoutines   JPEG Routines
 *
 *   JPEG is a standard for still image compression. The images may be stored
 *   at an arbitrary compression rate. There is some loss of information: the
 *   higher the compression rate the higher the image degradation due to loss
 *   of information. We recommend using quality factors of 50% - 80% for high
 *   quality images at reasonable compression rates.
 *   Since JPEG is a standard it may be used for exchanging image data with
 *   e.g. a PC. Standard PC programs comply with the format used in this
 *   library. Please be sure to store images as grey-level images since this
 *   color compression / decompression is not supported.
 *   Image variables used in JPEG compression must have a format which is a
 *   multiple of 8 for both, @p dx and @p dy. When decompressing images image
 *   variables must have a size of at least the size of the JPEG image -
 *   otherwise no decompression will be preformed.
 *
 * @{
 */
  /* jpeg output buffer size */
  #define  JPGBUF    (0x4000)    /**<  JPEG encode output buffer size, must be > 0x400*/

  I32  fread_jpeg(image *a, char *name);
  I32  fwrite_jpeg(image *a,char *name,I32 quality,U32 maxlng);                 /* monochrome            */
  I32  fwrite_jpegc(image *a,char *name,I32 quality,U32 maxlng);                /* color and monochrome  */

  /* jpeg structs */
  typedef struct
  {
    I32   put_bits;         /**<  number of bits already sent            */
    U32   cde;              /**<  actual bit code                        */
    U8   *buf;              /**<  pointer to start of buffer             */
    U8   *ptr;              /**<  current pointer to buffer              */
    U8   *last;             /**<  last available buffer address for JPEG */
    U32   counter;          /**<  counter for file I/O                   */
    I32   err;              /**<  error flag                             */
    I32 (*fc)();            /**<  write byte function pointer            */
    void *fp;               /**<  file pointer for file I/O              */
    I32   maxlng;           /**<  maximum file size                      */
  } emit_ctrl;

  typedef struct
  {
    U8   *ptr;              /**<  DRAM address                           */
    int   bits_left;        /**<  # of unused bits in it                 */
    long  buffer;           /**<  bit buffer                             */
    int (*fc)();
  } fill_ctrl;

  /* color jpeg */
  typedef struct JPEG_ENCODER_STRUCTURE
  {
    U32  mcu_width;                    /**<  8 (BW), 16 (RGB), 16 (PLANES), 16 (YCbCr422)  */
    U32  mcu_height;                   /**<  always 8                               */
    U32  horizontal_mcus;
    U32  vertical_mcus;
    U32  cols_in_right_mcus;           /**<  remaining columns in right MCU         */
    U32  rows_in_bottom_mcus;          /**<  remaining rows in bottom MCU           */

    U32  rows;
    U32  cols;

    U32  length_minus_mcu_width;
    U32  length_minus_width;
    U32  incr;
    U32  mcu_width_size;
    U32  offset;

    I32  ldc1;                         /**<  last DC value for component 1          */
    I32  ldc2;                         /**<  last DC value for component 2          */
    I32  ldc3;                         /**<  last DC value for component 3          */

    U32  cde;                          /**<  huffman bit-code for output stream     */
    U32  put_bits;                     /**<  number of bits already in cde          */
    U8  *addr;                         /**<  address for output buffer              */

    U8  *buffer;                       /**<  pointer to start of buffer             */
    U8  *ptr;                          /**<  current pointer to buffer              */
    U8  *last;                         /**<  last available buffer address for JPEG */
    U32  counter;                      /**<  counter for file I/O                   */
    I32 (*fc)();                       /**<  write byte function pointer            */
    void *fp;                          /**<  file pointer for file I/O              */
    I32  maxlng;                       /**<  maximum file size                      */
    I32  error;                        /**<  error status                           */

  } JPEG_ENCODER_STRUCTURE;

  /* jpeg i/o functions */
  //void  wr_dram   (emit_ctrl *emc, I32 val); not necessary since VCLIB 2.03 due to internal buffering
  void  wr_flash  (emit_ctrl *emc);
  void  wr_flashc (JPEG_ENCODER_STRUCTURE *jpeg);
  void  wr_ascii  (emit_ctrl *emc);
  void  wr_binary (emit_ctrl *emc);
  I32   rd_dram   (fill_ctrl *fct);
  I32   rd_flash  (fill_ctrl *fct);
  I32   rd_ascii  (fill_ctrl *fct);
  I32   rd_binary (fill_ctrl *fct);

  /* jpeg functions */
  U8  *cjpeg(image *a,I32 quality,U8 *addr,U32 maxlng,I32 (*func)());            /* monochrome            */
  #define  cjpeg_d(img, qual, addr, maxlng)  cjpeg(img, qual, addr, maxlng, (I32 (*)())NULL)
  #define  cjpeg_f(img, qual, addr, maxlng)  cjpeg(img, qual, addr, maxlng, (I32 (*)())wr_flash)

  U8  *encode_jpeg(image *a, I32 quality, U8 *addr, U32 maxlng, I32 (*func)() ); /* color and monochrome  */
  #define  xjpeg_d(img, qual, addr, maxlng)  encode_jpeg(img, qual, addr, maxlng, (I32 (*)())NULL)
  #define  xjpeg_f(img, qual, addr, maxlng)  encode_jpeg(img, qual, addr, maxlng, (I32 (*)())wr_flashc)

  U8  *djpeg(image *a,U8 *addr,I32 (*func)());
  #define  djpeg_d(img, addr)                djpeg(img, addr, (I32 (*)())rd_dram)
  #define  djpeg_f(img, addr)                djpeg(img, addr, (I32 (*)())rd_flash)

 /**@}@endcond*//*---End of JPEG Routines-------------------------------------*/

/**@}@endcond*//*---End of Image Data Conversion and Input/Output-------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  runLengthCodeProcessing   Run Length Code (RLC) Processing
*
*  Functions in this group operate on run length encoded image data, for
*  example functions doing erosion/dilation, blob generation,
*  feature extraction and more.
*
* @{
*/
 #define  MAXOBJECTS 32000        /* max number of objects for sgmt    */

 U16 *rlcmk(image *a, I32 thr, U16 *rlc, I32 size);                      /* horizontal RLC          */
 U16 *rlcmk2(image *src, I32 lowthr, I32 hithr, U16 *rlc, I32 size);     /* 2 thresholds            */
 U16 *rlcmkv(image *src, I32 thr, U16 *rlc, I32 size);                   /* vertical RLC            */
 U16 *rlcmkbit(image *a, I32 bit, U16 *rlc, I32 size);                   /* bitplane RLC            */
 U16 *hysteresis_RLC(image *src, U16 *rlc, I32 size, I32 thr, I32 seed); /* hysteresis threshold    */

 I32  rlcout(image *a, U16 *rlc, U32 dark, U32 bright);                  /* RLC to image            */
 I32  rlcout_transparent(image *a, U16 *rlc, U8 plotBrightIff1, U8 color);
 I32  dispobj(image *a, U16 *rlc);
 I32  rlc_mask_image(image *src, image *dst, U16 *pRlc, I32 *pi32Objs, U8 color, I32 offsetX, I32 offsetY);

 U16 *rlc2(U16 *rlca, U16 *rlcb, U16 *dest, U16 * (*func)() );
 #define  rlcand(a, b, c)     rlc2(a, b, c, (U16 * (*)())rlc_andf   )
 #define  rlcor(a, b, c)      rlc2(a, b, c, (U16 * (*)())rlc_orf    )
 #define  rlcxor(a, b, c)     rlc2(a, b, c, (U16 * (*)())rlc_xorf   )
 #define  rlcnand(a, b, c)    rlc2(a, b, c, (U16 * (*)())rlc_nandf  )
 #define  rlcnor(a, b, c)     rlc2(a, b, c, (U16 * (*)())rlc_norf   )
 #define  rlcequiv(a, b, c)   rlc2(a, b, c, (U16 * (*)())rlc_equivf )
 #define  rlcandnot(a, b, c)  rlc2(a, b, c, (U16 * (*)())rlc_andnotf)
 #define  rlcornot(a, b, c)   rlc2(a, b, c, (U16 * (*)())rlc_ornotf)
 #define  rlcnandnot(a, b, c) rlc2(a, b, c, (U16 * (*)())rlc_nandnotf)
 #define  rlcnornot(a, b, c)  rlc2(a, b, c, (U16 * (*)())rlc_nornotf)

 U16 *rlc_inv(U16 *rlc);                                                 /* RLC color inversion    */
 U16 *rlc_move(U16 *src, U16 *dst, I32 mx, I32 my);
 U16 *rlc90(U16 *src, U16 *dst, I32 size);                               /* rotate RLC 90 deg    */

 U16 *erxdi(U16 *src, U16 *dst, U16 *(*fc1)(), U16 *(*fc2)() );
 #define  erode(a, b)   erxdi(a, b, (U16 *(*)())rlc_xero, (U16 *(*)())rlc_orf)
 #define  dilate(a, b)  erxdi(a, b, (U16 *(*)())rlc_xdil, (U16 *(*)())rlc_andf)

 U16 *erxdi2(U16 *src, U16 *dst, U16 *(*fc1)(), U16 *(*fc2)() );
 #define  erode2(a, b)  erxdi2(a, b, (U16 *(*)())rlc_xero, (U16 *(*)())rlc_orf)
 #define  dilate2(a, b) erxdi2(a, b, (U16 *(*)())rlc_xdil, (U16 *(*)())rlc_andf)

 U16 *rlc_mf(U16 *src, U16 *dst, I32 col, I32 lng);
 
 U32  r_area(U16 **p, I32 dx, I32 color);
 U32  rlc_area(U16 *rlc, I32 color);

 U16 *fread_rlc(char *path, U16 *rlc);
 I32  fwrite_rlc(char *path, U16 *rlc);

 U16 *rlc_label(U16 *rlc, U16 *slc, I32 mode);
 #define  sgmt(rlc, slc)      rlc_label(rlc, slc, 0)                     /* white: 4-connected, black: 4-connected    */
 #define  label44(rlc, slc)   rlc_label(rlc, slc, 0)                     /* white: 4-connected, black: 4-connected    */
 #define  label88(rlc, slc)   rlc_label(rlc, slc, 1)                     /* white: 8-connected, black: 8-connected    */
 #define  label84(rlc, slc)   rlc_label(rlc, slc, 2)                     /* white: 8-connected, black: 4-connected    */
 #define  label48(rlc, slc)   rlc_label(rlc, slc, 3)                     /* white: 4-connected, black: 8-connected    */

 U16 *rlc_cut(U16 *src, U16 *dst, I32 objnum);
 I32  rl_area2(U16 *rlc, U32 *area, U32 n);


 I32  rlc_qin(U16 *rlc, I32 QIN[], U32 n);
 I32  rlc_nhls(U16 *rlc, U32 holes[], U32 n);

 I32  rlc_arf(U16 *src, U16 *dst, U32 min_area);
 I32  rlc_arf2(U16 *src, U16 *dst, I32 col, U32 min_area, U32 max_area);
 I32  rlc_select(U16 *rlc, U16 *rlc2, I32 selector[], U32 n, I32 mode);
 U16 *rlc_delete(U16 *src, U16 *dst, I32 keep[]);
 I32  rlc_getobject(U16 *rlc, I32 x, I32 y);
 I32  rlc_keep_object(U16 *src, U16 *dst, I32 x, I32 y);

 U16 *rlc_zoom(U16 *src, U16 *dst, I32 magX, I32 magY);
 #define rlc_copy(src, dst)	rlc_zoom(src, dst, 1, 1)

 U16 *rlc_fill(U16 *rlc, I32 dx, I32 dy, I32 color);
 U16 *testrlc(U16 *rlc, I32 dx, I32 dy, I32 size);
 I32  chkrlc(U16 *rlc);
 U16 *parse_rlc(U16 *rlc);

 U16 *rlc_print_line(U16 *pu16, U16 dx);
 void rlc_print(U16 *pRLC);

 

 /*--*GROUP*------------------------------------------------------------------*/
 /**@internal
 *
 *@cond ( vclib || extlib )
 *
 * @defgroup  runLengthCodeBasicFunctions   RLC Basic Functions
 *
 *  Functions in this group operate on one line of run length encoded image data.
 *  They are used by functions like @ref rlc2() or @ref erxdi().
 *
 * @{
 */
  /* additional RLC basic functions */
  #if defined __TI_COMPILER_VERSION__
   U16 *rlcmkfa(I32 dx, U8 thr, U8 *px, U16 *pr);
  #endif
  U16 *rlcmkfc(I32 dx, U8 thr, U8 *px, U16 *pr);
  U16 *rlcmkbc(I32 dx, U8 bit, U8 *px, U16 *pr);
  I32  rlcmkvf(U32 dx, U32 y, U16 **ra, U16 **rb, U16 **rc, U16 **vrlc);
  U16 *rlcoutf(U16 *pr, U8 *px, I32 dx, U8 dark, U8 bright);
  U16 *rlcout_transparentf(U16 *pr, U8 *px, I32 dx, U8 plotBrightIff1, U8 color);
  U16 *rlc_andf(   U16 **rlca, U16 **rlcb, U16 **dst, I32 eol);
  U16 *rlc_orf(    U16 **rlca, U16 **rlcb, U16 **dst, I32 eol);
  U16 *rlc_xorf(   U16 **rlca, U16 **rlcb, U16 **dst, I32 eol);
  U16 *rlc_nandf(  U16 **rlca, U16 **rlcb, U16 **dst, I32 eol);
  U16 *rlc_norf(   U16 **rlca, U16 **rlcb, U16 **dst, I32 eol);
  U16 *rlc_equivf( U16 **rlca, U16 **rlcb, U16 **dst, I32 eol);
  U16 *rlc_andnotf(U16 **rlca, U16 **rlcb, U16 **dst, I32 dx );
  U16 *rlc_ornotf(U16 **rlca, U16 **rlcb, U16 **dst, I32 dx);
  U16 *rlc_nandnotf(U16 **rlca, U16 **rlcb, U16 **dst, I32 dx);
  U16 *rlc_nornotf(U16 **rlca, U16 **rlcb, U16 **dst, I32 dx);
  U16 *rlc_xero(U16 **src, U16 **dst, I32 dx);
  U16 *rlc_xdil(U16 **src, U16 **dst, I32 dx);
  U16 *rlc_medf(U16 **px, U16 **pr, I32 lng, I32 col, I32 dx);
  U16 *rlc_movef(U16 **src, U16 **dst, I32 mx, I32 dx);
  void rlc_qinf(U16 **rlc, U16 **slc, U32 *PROJ, I32 *QIN, I32 dx);
  void rlc_areaf(U16 **rlc, U16 **slc, U32 *area, I32 dx);
  //void  rlc_fbounds(U16 **rlc, U16 **slc, BOUNDS *f, I32 y, I32 dx);
  //void  rlc_momf(U16 **rlc, U16 **slc, raw_moments *rm, I32 y, I32 dx);
  //U16 *rlc_selectf(U16 **rlca, U16 **slca, U16 **rlcb, I32 selector[], I32 dx);
  //U16 *rlc_selectf1(U16 **rlca, U16 **slca, U16 **rlcb, I32 selector[], I32 dx);
  //U16 *rlc_selectf2(U16 **rlca, U16 **slca, U16 **rlcb, I32 selector[], I32 dx);
  //U16 *rlc_selectf3(U16 **rlca, U16 **slca, U16 **rlcb, I32 selector[], I32 dx);
  //void  rlc_deletef(U16 **rlc, U16 **slc, U16 **dst, I32 *selector, I32 dx);

 /**@}@endcond*//*---End of RLC Basic Functions-------------------------------*/


 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib || extlib )
 *
 * @defgroup  runLengthCodeFeatureCalculations   RLC Features
 *
 *  For a description see the section @ref featureDataHolder.
 *
 * @{
 */
  void rlc_feature(feature *f, U16 *rlc, I32 color);
  I32  rl_ftr2(U16 *rlc, ftr *f, U32 n);
  I32  rl_ftr3(U16 *rlc, ftr *f, U32 n);

  U16 *x_center(U16 *p, I32 dx, I32 color, feature *f);
  U16 *y_center(U16 *p, I32 dx, I32 color, feature *f, I32 line);
  U16 *r_max(U16 *p, I32 dx, I32 color, feature *f, I32 line);

  I32  rlc_feret(U16 *src, F32 alpha, F32 *fer);
  I32  rlc_minmax_feret(U16 *pRlc, I32 nsteps, F32 fMinAngleRad, F32 fMaxAngleRad, F32 *minferets, F32 *minangles, F32 *maxferets, F32 *maxangles);


   /*--*GROUP*------------------------------------------------------------------*/
   /**
   *@cond ( extlib )
   *
   * @defgroup  runLengthCodeMoments   RLC Moments
   *
   * @image  html   moments_anders.png
   * @image  latex  moments_anders.pdf
   *
   *  For a description see the section @ref momentDataHolder.
   *
   * @{
   */
    I32  rlc_moments(U16 *r0, moment *mx, U32 n);
    F32  mom_calc_area(moment *mx);       /* calculate area                    */
    F32  mom_calc_cgx(moment *mx);        /* center of gravity x coordinate    */
    F32  mom_calc_cgy(moment *mx);        /* center of gravity y coordinate    */
    F32  mom_calc_angle(moment *mx);      /* calculate angle of inertial axis  */
    F32  mom_calc_rad(moment *mx);        /* calculate angle in radiants       */
    F32  mom_calc_ecc(moment *mx);        /* calculate eccentricity            */
    F32  mom_calc_ellipse_a(moment *mx);  /* ellipse half parameter            */
    F32  mom_calc_ellipse_b(moment *mx);  /* ellipse half parameter            */
    F32  mom_calc_phi1(moment *mx);       /* calculate phi1                    */
    F32  mom_calc_phi2(moment *mx);       /* calculate phi2                    */
   /**@}@endcond*//*---End of RLC Moments---------------------------------------*/
 /**@}@endcond*//*---End of RLC Feature Calculations--------------------------*/

/**@}@endcond*//*---End of Run Length Code (RLC) Processing-------------------*/

/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( extlib )
*
* @defgroup  contourCodeProcessing   Contour Code Processing
*
*
* @{
*/
 /* functions for contour following */
 I32  contour8(image *a, I32 x0, I32 y0, I32 dir, I32 thr, U32 lng, U32 **dst);

 void  cdisp(image *a, U32 *src, I32 col, void (*func)());
 #define  cdisp_d(a, src, col)    cdisp(a, src, col, (void (*)())wpix)
 #define  cdisp_x(a, src, col)    cdisp(a, src, col, (void (*)())xorpix)
 #define  cdisp_o(a, src, col)    cdisp(a, src, col, (void (*)())wpix)
 #define  cdisp_z(a, src, col)    cdisp(a, src, col, (void (*)())xorpix)

 I32  ccxy(I32 *src, I32 *xy, I32 *tbl, U32 maxcount);

 I32  edgecontours_from_edgeimage(image *src, I32 thr, I32 minlength, I32 maxcount, I32 *xylist, U8 *maglist, U8 *dirlist, I32 *nr);
 I32  subpixel_edgecontour(image *src, image *edg,  I32 thr, I32 minlength, I32 maxcount, I32 *xylist, U8 *maglist, U8 *dirlist, I32 *nr);

 void wpix(U8 x, U8 *adr);
 void xorpix(U8 x, U8 *adr);
/**@}@endcond*//*---End of Contour Code Processing----------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup shading_correction  Shading Correction
*
*@{
*/

 I32  shading_correct_2D_mul(image *src, image *dst, image *mul);
 I32  shading_correct_2D_mul_calib(image *src, image *mul, U32 u32AvgSizePx);

 I32  shading_correct_1D(image *src, image *dst, I8 *offs, U16 *mul);
 I32  shading_correct_1D_calib(image *src1, image *src2, F32 t1, F32 t2, I8 *offs, U16 *shade);

 I32  line_IIR(image *src, image *dst, I32 tol, I32 int_const, I32 p_const, I32 target);

/**@}@endcond*//*---End of Shading Correction--------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  memoryManagement   Memory Management
*
*  Functions in this group provide methods for memory management.
*
*
* @{
*/

 U8  *ImageAllocate(image *img, U32 type, U32 dx, U32 dy);
 I32  ImageAllocateEdge(image *src, image *dst, I32 edgeX, I32 edgeY);
 void ImageFree(image *img);

 #define  rlcmalloc(x)    (U16 *)sysmalloc((x+1)/2, MDATA)
 #define  rlcfree(x)        sysfree((void *)x)
 void ImageAssign_Bayer(image *img);



 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( extlib )
 *
 * @defgroup  fastBuffer   Fast Buffer
 *
 *  Functions in this group provide non-standard methods for memory management.
 *
 * @{
 */
  typedef  struct   _heade_r                                    /* fast local buffer heap structure        */
  {
   struct _heade_r  *ptr;
   U32             size;
  } hdr;

  typedef hdr  header;

  extern header  *freep;                                        /* pointer for fast local buffer           */
  void  fastbuf_prtfree(void);                                  /* function for local fast buffer          */
  void  fastbuf_setup(header **freex, int *base, int size);     /* call fastbuf_setup() to enable function */
  void  fastbuf_free(void *ap);                                 /* without setup or with space exhausted   */
  void *fastbuf_malloc(unsigned nwords);                        /* function calls sysmalloc()              */

 /**@}@endcond*//*---End of Fast Buffer---------------------------------------*/
/**@}@endcond*//*---End of Memory Management----------------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
* @defgroup  periphery   Periphery and Hardware Related Functions
*
*  Functions in this group provide access to special camera hardware.
*
*
* @{
*/

 void init_color_lut(I32 red, I32 green, I32 blue);

 I32  set_ovl_false_color      (I32 table);
 I32  set_translucent_to_value (I32 table, I32 r, I32 g, I32 b);
 I32  display_directions       (image *src, I32 thresh, I32 startx, I32 starty);

 /* functions dependent on special hardware */
 I32  BayerToYCbYCr_HW(image *src, image *dst);             /* preview engine hardware version  */
 I32  setup_HW_BayerToYCbCr422(image *src, image *dst);     /* setup preview engine             */
 void set_HW_whitebalance(I32 r, I32 gr, I32 gb, I32 b);    /* set preview engine whitebalance  */
 void start_HW_BayerToYCbCr422();
 I32  is_busy_HW_BayerToYCbCr422();                         /* check if preview engine busy     */


 int  ReadKeyboard (int wait);
 void EmptyKeyboardBuffer (void);

/**@}@endcond*//*---End of Periphery------------------------------------------*/





/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( extlib )
*
* @defgroup best_approximation  Best Approximation
*
*  Functions in this group fit forms to a given pixel list.
*
*@{
*/
 I32  bestline( I32 *xy, int N, F32 *cx, F32 *cy, F32 *b);
 I32  bestline0(I32 *xy, int N, F32 *cx, F32 *cy, F32 *b);
 I32  bestline2(I32 *xy, U32 N, F32 *cx, F32 *cy, F32 *b);

 I32  bestline_successive_fit(I32 *pi32LstPxIn, U32 u32PxCount, vcline *psBestline, U32 u32RemovePxCount, U8 u8RemovePercentageIff1, U32 u32UseMinCountPx);
 I32  linear_regression(I32 *xy, int N, F32 *mx, F32 *b);

 I32  bestcircle(I32 *pxy, int N, F32 *px, F32 *py, F32 *rad);

/**@}@endcond*//*---End of Best Approximation---------------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  mathematics    Mathematics
*
*  Functions in this group provide concepts like matrix, vector, ...
*
* @{
*/
 #ifndef  NOMINMAX
   #ifndef max
    /** @def max(a,b)
     *       Outputs the maximum of @p a and @p b. */
    #define  max(a,b)    (((a) > (b)) ? (a) : (b))
   #endif

   #ifndef min
    /** @def min(a,b)
     *       Outputs the minimum of @p a and @p b. */
    #define  min(a,b)     (((a) < (b)) ? (a) : (b))
   #endif

 #endif  /* NOMINMAX */

 /** @def sign(x)
  *       Outputs the sign of @p x, either 1, 0 or -1. */
 #define  sign(x) ((x) > 0 ? 1:  ((x) == 0 ? 0: (-1)))

 U32   sq(U32 x);

 /* linear algebra algorithms								*/
 typedef struct  /* matrix of arbitrary type       			*/
   {               
   I32  type;     /*!<  The type of the matrix.         	*/
   I32  nr;       /*!<  Number of rows.                     */
   I32  nc;       /*!<  Number of columns.                  */
   void **ptr;    /*!<  Pointer to the Iliffe matrix.       */
   } vcmat;

 typedef struct  /* vector of arbitrary type       			*/
   {               
   I32  type;     /*!<  The type of the vector.         	*/
   I32  nr;       /*!<  Number of rows.                     */
   void *ptr;     /*!<  Pointer to the vector.       		*/
   } vcvec;

  #define  NULL_MAT  {0, 0, 0, NULL}
  #define  NULL_VEC  {0, 0, 0, NULL}

  #define MAT_U8		(0)		// matrix with U8  elements
  #define MAT_I16		(1)		// matrix with I16 elements
  #define MAT_I32		(2)		// matrix with I32 elements
  #define MAT_I64		(3)		// matrix with I64 elements
  #define MAT_F32		(4)		// matrix with F32 elements
  #define MAT_F64		(5)		// matrix with F64 elements
  #define MAT_LST		(5)		// last type, number must be same as the last type in the list

  void **matrixALL(I32 nr, I32 nc, I32 type);				// allocates Iliffe matrix of all types
  void free_matrixALL(void **m);							// deallocates Iliffe matrix of all types
  void *vectorALL(I32 nh, I32 type);                		// allocates vector of all types
  void free_vectorALL(void *v);           					// deallocates vector of all types

  I32 vcmat_allocate(vcmat *matp, I32 nr, I32 nc, I32 type);// allocate VC matrix
  I32 vcmat_free(vcmat *matp);								// deallocate VC matrix
  I32 vcmat_print(vcmat *matp);								// print VC matrix
  I32 vcmat_mult(vcmat *a, vcmat *b, vcmat *c);				// multiply  VC matrices
  I32 vcmat_copy(vcmat *src, vcmat *dst);					// copy VC matrices
  I32 vcmat_transp(vcmat *src, vcmat *dst);					// transpose VC matrix

  I32 lu_inverse(vcmat *a, vcmat *y, F64 *det);				// LU inverse of n x n - VC matrix, arbitrary precision
  I32 lu_det(vcmat *a, F64 *det);							// LU determinant of n x n - VC matrix, arbitrary precision

  // internal functions
  I32 lu_inverse_F32(F32 **a, F32 **y, I32 n, F32 *det);	// LU inverse of n x n - matrix, single precision
  F32 lu_det_F32(F32 **a, I32 n);							// LU determinant of n x n - matrix, single precision
  I32 lu_inverse_F64(F64 **a, F64 **y, I32 n, F64 *det);	// LU inverse of n x n - matrix, double precision
  F64 lu_det_F64(F64 **a, I32 n);							// LU determinant of n x n - matrix, double precision


 F32  *vector(int nh);
 void  free_vector(F32 *v);
 F32 **matrix(I32 nr, I32 nc);
 void  free_matrix(F32 **m);

 void  vect_print(I32 n, F32 x[]);
 void  matrix_print(I32 n, I32 m, F32 **a);

 F32   vect_norm(I32 n, F32 x[]);
 void  matrix_vect_mult(I32 n, I32 m, F32 **a, F32 x[], F32 y[]);
 void  matrix_mult(I32 n, I32 m, F32 **a, F32 **x, F32 **y);
 void  matrix_copy(I32 n, I32 m, F32 **src, F32 **dst);


 I32 **imatrix(I32 nr, I32 nc); // function should be replaced by MatrixI32()
 void  free_imatrix(I32 **m);
 I32 **MatrixI32(I32 nr, I32 nc);
 void  FreeMatrixI32(I32 **m);
 U8  **MatrixU8(I32 nr, I32 nc);
 void  FreeMatrixU8(U8 **m);


 /* 3D algorithms and calibration							*/

 /* "mapping" describes a general 3D transformation that	*/
 /* maps one 3D coordinate system into another				*/

 typedef struct
 {
  F64  r[3][3];		/* 3x3 orthonormal rotation matrix		*/
  F64  t[3];		/* 3x1 translation vector				*/
  F64* R[3];		/* the matrix row pointers for r		*/
  F64* T;			/* the vector pointer for t				*/
 } mapping;


 /* Tsai internal calibration parameter struct				*/

 typedef struct
 {
  F64    Nx;      		/* [pix]     horizontal number of pixels                		*/
  F64    Ny;      		/* [pix]     vertical number of pixels                      	*/
  F64    Szx;       	/* [mm]      horizontal pixel size                          	*/
  F64    Szy;       	/* [mm]      vertical pixel size                            	*/
  F64    Sx;       		/* [1]       pixel scale factor horizontal/vertical   	        */
  F64    Gx;       		/* [pix]     geometric center x           						*/
  F64    Gy;       		/* [pix]     geometric center y            						*/
  F64    f;        		/* [mm]      distance from projection center to image plane    	*/
  F64    Cx;       		/* [pix]     distortion center x           						*/
  F64    Cy;       		/* [pix]     distortion center y            					*/
  F64    K1;   			/* [1/mm^2]  radial distortion coefficient                     	*/
  F64    K2;   			/* [1/mm^4]  radial distortion coefficient                     	*/
  F64    P1;   			/* distortion coefficient                     					*/
  F64    P2;   			/* distortion coefficient                     					*/
  F64    P3;   			/* distortion coefficient                     					*/
 } TsaiInternal;

#define	TSI_VC6210  { 752.0,  480.0, 0.0060, 0.0060, 1.0, 376.0, 240.0, 1.0, 376.0, 240.0, 0.0, 0.0, 0.0, 0.0, 0.0}
#define	TSI_VC6211  {1280.0, 1024.0, 0.0053, 0.0053, 1.0, 640.0, 512.0, 1.0, 640.0, 512.0, 0.0, 0.0, 0.0, 0.0, 0.0}
#define	TSI_VC6215  {1600.0, 1200.0, 0.0045, 0.0045, 1.0, 800.0, 600.0, 1.0, 800.0, 600.0, 0.0, 0.0, 0.0, 0.0, 0.0}
#define	TSI_VC6212  {2592.0, 1944.0, 0.0022, 0.0022, 1.0, 640.0, 512.0, 1.0, 640.0, 512.0, 0.0, 0.0, 0.0, 0.0, 0.0}

/**@}@endcond*//*---End of Mathematics----------------------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( extlib )
*
* @defgroup edge_detection  Gradients and Edge Detection
*
*@{
*/

 /* gradient routines */
 I32  gradient_2x2(image *src, image *dst);
 I32  init_gradient_2x2();
 void deinit_gradient_2x2();
 I32  gradient_3x3(image *src, image *dst);
 I32  init_gradient_3x3();
 void deinit_gradient_3x3();
 I32  gradient_3x3_thresh(image *src, image *dst, I32 thresh);
 I32  init_gradient_3x3T();
 void deinit_gradient_3x3T();

 /* edge detection	*/
 I32  edge(image *src, image *dst, I32 type, F32 sigma, I32 BinMode, I32 MinContrast, F32 fthresh, I32 binar_value);
 #define  edge_canny(src, dst, binar_value)    edge(src, dst, 4, 0.625, -2, 5, 0.10, binar_value)
 #define  edge_fast( src, dst, binar_value)    edge(src, dst, 4, 0.625,  2, 5, 0.15, binar_value)
 #define  edge_sobel(src, dst, binar_value)    edge(src, dst, 1, 0.0,    2, 5, 0.10, binar_value)

 /* misc routines working on gradient images */
 I32  nonmax_suppress(image *src, image *dst);
 F32  GradientAngle(image *imgGrad, I32 kx);

 I32  block_orientation_histogram_16x16(image *src, image *dst);

/**@}@endcond*//*---End of Gradients and Edge Detection-------------------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( extlib )
*
* @defgroup hough_transform  Hough Transform
*
*@{
*/

 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( extlib )
 *
 * @defgroup hough_line  Hough Line Transform
 *
 *@{
 */
    /*--*STRUCT*--------------------------------------------------------------*/
    /**
    * @brief  Holds Settings for doing the Hough Line Transform.
    */
    typedef struct
    {
        I32  HoughMode;              /**<  Hough operating mode = 0-3     */
        I32  LineNumber;             /**<  number of lines for output     */
        I32  delta_phi;              /**<  phi tolerance                  */
        I32  MinPix;                 /**<  threshold for number of pixels */
        I32  LineWidth;              /**<  width of lines to be searched  */
        I32  bestfit;                /**<  bestfit line flag 1=bestfit    */
        struct hlstruct  *maxptr;    /**<  pointer to active lines list   */
        struct hlstruct  *empty;     /**<  pointer to empty line list     */
        I32  sorting;                /**<  sorting method, 0=none         */
    }
    HoughControl;

    #define  NULL_HOUGHCONTROL  {0, 0, 0, 0, 0, 0, NULL, NULL, 0}

    /*--*STRUCT*--------------------------------------------------------------*/
    /**
    * @brief  Line Information Returned by the Hough Line Transform With
    *         Additional Information.
    */
    typedef struct hlstruct
    {
        struct hlstruct  *next;      /**<  pointer to next element in list */
        I32   state;                 /**<  internal state                  */
        I32   value;                 /**<  line detection value            */
        I32   phi;                   /**<  line angle                      */
        I32   rho;                   /**<  distance from center of image   */
        I32   strength;              /**<  line detection strength         */
        F32   cx;                    /**< float cx, cy, b-parameters for   */
        F32   cy;                    /**< float line in normalized         */
        F32   b;                     /**< float vector form                */
    }
    HLine;

    #define  HL_VALUE      (2)           /* these offsets must be changed    */
    #define  HL_PHI        (3)           /* according to the sequence of     */
    #define  HL_RHO        (4)           /* values in hlstruct               */
    #define  HL_STRENGTH   (5)

    void  HoughDefaults        (HoughControl *Control);
    I32   HoughCalcDx          (image *src);
    I32   HoughInit            ();
    I32   HoughInit2           (HoughControl *Control);
    void  HoughDeinit          ();
    void  HoughDeinit2         (HoughControl *Control);
    I32   HoughTransform       (image *src, image *Hough32, HoughControl *Control);
    I32   FindHoughLine_PtrList(image *Hough32, image *src, HoughControl *Control, SPtrList *lines);					/* new and preferred		*/
    I32   FindHoughLine        (image *Hough32, image *src, HoughControl *Control);										/* old-style wrapper		*/
    I32   FindHoughLine0       (image *Hough32, image *src, HoughControl *Control);										/* old function				*/
    I32   FindLines            (image *ImgVec, HoughControl *Control, I32 *NrLines);
    I32   FindSingleLine       (image *Hough32, image *src, HoughControl *Control, I32 phi, I32 phi_range, HLine *line);/* function for single line	*/
    I32   HoughRank            (HLine *maxptr, I32 offset, I32 value);
    void  HoughSortLine        (HLine **listptr, I32 offset);
    I32   GetHoughPixels       (image *Src, HLine *line, HoughControl *Control, I32 *xy);
    I32   HoughList            (image *Src, HLine *Line, I32 DeltaAngle, U8 HoughMode, I32 width, I32 *buffer, I32 *xy);

 /**@}@endcond*//*---End of Hough Line Transform------------------------------*/



 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( extlib )
 *
 * @defgroup hough_rectangle  Hough Rectangle Transform
 *
 * @{
 */

    /*--*STRUCT*--------------------------------------------------------------*/
    /**
    * @brief  Features for HoughRectangle() rectangle-finder.
    *
    */
    typedef struct
    {
        I32  search_color;               /**< the color of the rectangle that is searched, 0: black, 1: white                                                                    */
        I32  check_bounds;               /**< 0: no check, the intersection points may lie outside the image, 1: check bounds for corners                                        */
        I32  tolerance_for_90deg;        /**< former "DeltaAng" parameter: the corner angle may be more or less than 90degrees, the maximum tolerance is specified by this value */
        I32  small_sides_min_length;     /**< the small sides are the ones neighboring the large sides no matter their length                                                    */
        I32  small_sides_max_length;     /**< -- " --                                                                                                                            */
        I32  large_sides_min_length;     /**< the large sides are the side with the maximum length and its opposite no matter its length                                         */
        I32  large_sides_max_length;     /**< -- " --                                                                                                                            */
        F32  aspect_ratio_min;           /**< small_side_length / large_side_length                                                                                              */
        F32  aspect_ratio_max;           /**< -- " --                                                                                                                            */
        I32  small_sides_angle_tolerance;/**< with this parameter it is possible to search for rectangles with more or less parallel sides                                       */
        I32  large_sides_angle_tolerance;/**<  -- " --                                                                                                                           */
        I32  major_axis_min_angle;       /**< the angle of the major axis is the average of the angles of the side with the maximum length and its opposite                      */
        I32  major_axis_max_angle;       /**< -- " --; Set @ref major_axis_min_angle = @ref major_axis_max_angle = 0 to switch off this feature                                  */
        F32  deviation_perc_opposite;    /**< former "PropOpp" parameter                                                                                                         */
        I32  cache_control;              /**< 2^cache_control = number of XY-buffers, set it to 6 for 64 buffers                                                                 */
    } HoughRectangleControl;
    /*------------------------------------------------------------------------*/

    typedef struct HoughQuad
    {
        I32    strength;
        I32    average;
        I32    color;
        I32    x[4];
        I32    y[4];
        HLine *Line[4];
    } HoughQuad;

    I32  HoughRectangle(image *Grad, HoughControl *Control, I32 NrLines, HoughRectangleControl *hrc, SPtrList *rectangles);

 /**@}@endcond*//*---End of Hough Rectangle Transform-------------------------*/



 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( extlib )
 *
 * @defgroup hough_circle  Hough Circle Transform
 *
 * @{
 */
    /* basic struct for Hough Circle Transform */
    typedef struct hcstruct
    {
        I32  x0;           /**<  centerpoint x-value         */
        I32  y0;           /**<  centerpoint y-value         */
        I32  r;            /**<  circle radius               */
        F32  x0f;          /**<  centerpoint x-value (float) */
        F32  y0f;          /**<  centerpoint y-value (float) */
        F32  rf;           /**<  circle radius (float)       */
        I32  strength;
    }
    HCircle;

    /* control struct for Hough Circle Transform */
    typedef struct
    {
        /**********************************************************************
         * open parameters                                                    *
         **********************************************************************/

        I32   Thresh;        /**<  threshold for binarization of source image */
        I32   MinRad;        /**<  minimum radius for circle detection        */
        I32   MaxRad;        /**<  maximum radius for circle detection        */
        I32   Error;         /**<  error code                                 */

        /* hough parameters (with relative hough position to the image)       */
        I32   HoughX;        /**<  relative starting point x-coordinate       */
        I32   HoughY;        /**<  relative starting point y-coordinate       */

        I32   bestfit;       /**<  0: no bestcircle, 1: bestcircle approximation */

        /**********************************************************************
         * the following parameters are reserved for internal use             *
         **********************************************************************/

        I32   MaxRLC;
        I32   MaxObj;
        I32   MinObjSize;
        U16  *rlc;
        I32   NrObj;
        ftr  *f;
        I32   ContourColor;
        I32   MaxContour;
        U32  *MemContour;
        I32   MaxLine;
        I32   LenLine;
        I32  *LineXY;
        U8   *Magn;
        U8   *Grad;
        I32   LineOffs;
        I32   LenSegment;
        I32   MinAng;
        I32   MaxAng;
        I32   MinDis;
        I32   MaxDis;
        I32   DeltaRadius;
        I32  *RadiusSquare;
        I32   CentreSize;
        I16  *P1XY;
        I16  *P2XY;
        I32   AvgDx;
        I32   AvgDy;
        I32   TolRadPercent;
        I32   TolRadOffset;
        I32   DeltaRohMin;
        I32   DeltaRohMax;
        I32   RohMinRad;
    }
    HCirclePar;

    I32  HoughCDefaults       (HCirclePar *HCP);
    I32  InitHoughCircle      (HCirclePar *HCP);
    I32  HoughCircleTransform (image *ImgVec, image *Hough32, HCirclePar *HCP);
    I32  FindHoughCircle      (image *Hough32, HCirclePar *HCP, HCircle *Circle);
    I32  DeinitHoughCircle    (HCirclePar *HCP);
    I32  FindMaxImage8        (image *src8,  I32 *ix, I32 *iy);
    I32  FindMaxImage16       (image *Src16, I32 *ix, I32 *iy);
    I32  FindMaxImage32       (image *Src32, I32 *ix, I32 *iy);
    
    /**@}@endcond*//*---End of Hough Circle Transform----------------------------*/
/**@}@endcond*//*---End of Hough Transform------------------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( extlib )
*
* @defgroup fft   Fast Fourier Transform (FFT)
*
* The Fast Fourier Transform Algorithm (FFT) is an efficient algorithm to
* compute the discrete Fourier transform and its inverse. In the following
* we have FFT functions for 2D image data as well as 1D vectors.
*
* For the image containing the fourier transform the data type  @ref IMAGE_CMPLX16
* is used, which uses real and imaginary Values per Pixel.
* The real and imaginary value of one pixel follow each other directly in memory.
*
* @image  html   image_cmplx.png
* @image  latex  image_cmplx.pdf
*
* Since each value, real and imaginary, are of type @ref I16, it is best to
* use an @ref I16 pointer. Image @p pitch is measured in pixels, so to get on
* line down at the image add two times the pitch to the @ref I16 pointer since
* each pixel consists of two values.
*
*
* @{
*/

   I32  vc_fft               (image *src, image *dst, I32 *mean, I32 *scale);
   I32  vc_ifft              (image *src, image *dst, I32 *mean, I32 *scale);

   F32  fft_window_hanning(F32 dist, I32 endInterval);
   I32  draw_fft_window(image *psImgOut,  F32 (*func)(F32,I32));
   #define  draw_fft_window_hanning(psImgOut)   draw_fft_window(psImgOut,&(fft_window_hanning))

   #define  apply_fft_window(psImgToMask, psImgWindow, psImgOut)   mul2(psImgToMask, psImgWindow, psImgOut, -8)

   I32  FFTInitTwiddle       (I32 size);
   void FFTDeinitTwiddle     ();
   I32  GenCplxImg           (image *src, image *dst, I32 *mean);
   I32  DisplayFFT           (image *src, image *dst, I32 log);
   I32  DisplayInvFFT        (image *src, image *dst, I32 mean, I32 scale);
   I32  FindMaxCplx          (image *src, I32 *ix, I32 *iy);
   I32  FindCplxScale        (image *SrcCplx);

   I32  CalcPolarCoordinates (image *dst, I32 deg);
   I32  DeleteFreq           (image *src, image *pol, I32 minrad, I32 maxrad);
   I32  CalcAngleHisto       (image *src, image *pol, I32 mode, I32 minfreq, I32 maxfreq,  U32 *AngleHisto, I32 nr);

/**@}@endcond*//*---End of Fast Fourier Transform (FFT)-----------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup geometric_functions   Geometric Functions
*
* @{
*/

   F32  Norm                 (point *a);                                              /*norm of a point                           */
   F32  AngleP               (point *v);                                              /*angle of a point (from origin)            */
   F32  PointDistance        (point *a, point *b);                                    /*distance between 2 points                 */
   void MidPoint             (point *a, point *b, point *c);                          /*midpoint of 2 points                      */
   void VectorDifference     (point *a, point *b, point *c);                          /*difference vector of 2 vectors/points     */
   void VectorSum            (point *a, point *b, point *c);                          /*sum of 2 vectors/points                   */
   I32  LineIntersection     (vcline *a, vcline *b, point *r);                        /*intersection of 2 lines                   */
   F32  LineAngle            (vcline *a, vcline *b);                                  /*angle between 2 lines                     */
   F32  LineAngle2           (vcline *a, vcline *b);                                  /*as before, using dot and vector product   */
   F32  Angle                (vcline *a);                                             /*angle of a line                           */
   void LineParallelD        (F32 dist, vcline *l, vcline *r);                        /*parallel line at given distance           */
   F32  PointLineDistance    (point *p, vcline *l);                                   /*distance between a point and a line       */
   void LinePerpendicular    (point *p, vcline *l, vcline *r);                        /*perpendicular line through a point        */
   void LineParallel         (point *p, vcline *l, vcline *r);                        /*parallel line through a point             */
   void LineParameters       (point *p1, point *p2, vcline *line);                    /*line through 2 points                     */
   void LineParametersA      (point *p, F32 angle, vcline *line);                     /*line through point at given angle         */
   void LineFromSlopeInterceptForm(F32 mx, F32 b, vcline *psLine);                    /*line from y = mx * x + b                  */
   void LineNormal           (point *p, F32 angle, vcline *line);                     /*line through point perpendicular to angle */
   void affine_point_map     (point *p, point *target_p, F32 a[2][2], F32 t[2]);      /*image of a point under affine transform   */
   I32  IntersectionPointsN  (image *img, vcline *ln, point *p1, point *p2);          /*intersection of a line with bounding box  */
   I32  IntersectionPoints   (image *a, F32 cx, F32 cy, F32 b, I32 *X0, I32 *Y0, I32 *X1, I32 *Y1);

/**@}@endcond*//*---End of Geometric Functions--------------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( extlib )
*
* @defgroup pixel_list_functions   Pixel List Functions
*
* @{
*/

 I32  clip(I32 N, I32 *xy_src, I32 *xy_dst, I32 x_max, I32 x_min, I32 y_max, I32 y_min);
 void translate(I32 N, I32 *xy_src, I32 *xy_dst, I32 mx, I32 my);

 I32  PL_line_stats    (I32 *xy, I32 nr, vcline *line, F32 *dmin, F32 *dmax, F32 *sigma);
 I32  PL_line_statsF   (F32 *xy, I32 nr, vcline *line, F32 *dmin, F32 *dmax, F32 *sigma);
 I32  PL_line_ending   (I32 *xy, I32 nr, vcline *line, I32 *imin, I32 *imax);
 I32  PL_line_endingF  (F32 *xy, I32 nr, vcline *line, I32 *imin, I32 *imax);
 F32  PL_line_bendingF (F32 *xy, I32 nr, vcline *line);
 I32  subpix_line_fast (image *SearchArea, vcline *line, I32 dx, I32 dy, I32 EdgeV, I32 nr, I32 *XY_src, F32 *XYF_dst);
 
 void  PL_reverse(I32 *xylist, I32 n);
 
 //undocumented internal functions follow
 void  xyscf1(U32 n, I32 *xy, I32 xm, I32 ym);

/**@}@endcond*//*---End of Pixel List Functions-------------------------------*/



/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( unavailable )
*
* @defgroup rectangleMeasurement   Rectangle Measurement
*
* @{
*/

//   typedef struct
//   {
//   	//////////////////////////////////////////////////
//   	// input parameters
//   	//////////////////////////////////////////////////
//
//   	I32   FindZoom;         /**<  number of demagnification stages        */
//
//   	//////////////////////////////////////////////////
//   	// edge parameters
//   	//////////////////////////////////////////////////
//
//   	I32   Type;             /**<  see function edge for description       */
//   	I32   Sigma;            /**<  10 x sigma                              */
//   	I32   BinMode;
//   	I32   MinContr;
//   	I32   Thresh;
//
//   	//////////////////////////////////////////////////
//   	// hough parameters
//   	//////////////////////////////////////////////////
//
//   	HoughControl  Control;  /**<  see Hough transform for descr.          */
//
//   	//////////////////////////////////////////////////
//   	// line selection filters
//   	//////////////////////////////////////////////////
//
//   	I32   DeviceCol;        /**<  0=Black 1=White                         */
//   	I32   MinLen;           /**<  min length in pixel                     */
//   	I32   MaxLen;           /**<  max length in pixel                     */
//   	I32   PropOpp;          /**<  proportion of opposite lines in percent */
//   	I32   PropNext;         /**<  proportion of adjacent lines in percent */
//   	I32   DeltaAngle;       /**<  max delta angle for 90 degree corners   */
//
//   	//////////////////////////////////////////////////
//   	// subpixel parameters
//   	//////////////////////////////////////////////////
//
//   	I32   EdgeFilterP;      /**<  must be 4                               */
//   	I32   EdgeFilterV;      /**<  must be 2                               */
//
//   	//////////////////////////////////////////////////
//   	// defect parameters
//   	//////////////////////////////////////////////////
//
//   	I32   DefectWidth;      /**<  maximum defect width                    */
//
//   	//////////////////////////////////////////////////
//   	// output parameters
//   	//////////////////////////////////////////////////
//
//   	//////////////////////////////////////////////////
//   	//
//   	//                line 0
//   	// Point 0 ------------------- Point 1
//   	//         |                 |
//   	//         |                 |
//   	//         |                 |
//   	//  line 3 |                 | line 1
//   	//         |                 |
//   	//         |                 |
//   	//         |                 |
//   	// Point 3 ------------------- Point 2
//   	//                 line 2
//   	//
//   	//////////////////////////////////////////////////
//
//   	vcline Line    [4];     /**<  BestLine parameters                     */
//
//   	I32    PointsXY[4];     /**<  number of line points in LineXY         */
//   	F32 *LineXY  [4];     /**<  accurate X and Y positions of the line  */
//
//   	F32  EndX0   [4];     /**<  start of line, x-coordinate             */
//   	F32  EndY0   [4];     /**<  start of line, y-coordinate             */
//   	F32  EndX1   [4];     /**<  end of line, x-coordinate               */
//   	F32  EndY1   [4];     /**<  end of line, y-coordinate               */
//
//   	F32  Dmin    [4];     /**<  min line deviation (negative)           */
//   	F32  Dmax    [4];     /**<  max line deviation (positive)           */
//   	F32  DSigma  [4];     /**<  standard deviation from line            */
//
//   	// result
//   	I32    Error;
//   } MR_Par;
//
//   I32   MeasureRectangle (MR_Par *Par, image *SearchArea);
//   void  DeinitMR (MR_Par *Par);

/**@}@endcond*//*---End of Rectangle Measurement--------------------------------*/


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup BarcodeReader   Barcode Reader
*
* @{
*/

 #define   BARCODE_25       0
 #define   BARCODE_39       1
 #define   BARCODE_EAN128   2
 #define   BARCODE_EAN13    3
 #define   PHARMACODE       4
 #define   BARCODE_128      5
 #define   MAX_BARCODE     16

 /*--*STRUCT*--------------------------------------------------------------*/
 /**
 *
 *  @brief  Barcode Reader Parameter Settings.
 *
 *    This structure contains the settings for the barcode reader.
 *    @p i32BinThr is the binarization threshold for the barcode finder.
 *    Reasonable values for this parameter are in the range of [2..20] with
 *    a default value of 4. A low value results in more edges in the internal
 *    detection image with a higher chance for finding low-contrast codes
 *    at the price of a slightly higher processing time.
 *
 *    @p i32MinContr is the detection contrast for barcodes or similar structures.
 *    A value too high for this parameter may result in a mal-detection of
 *    low-contrast barcode, a value too low can result in non-barcode structures
 *    like text falsely detected as potential barcode regions which may increase
 *    processing time. A reasonable value for this parameter is 80.
 *
 *    @p i32BCMaxCount is the maximum number of potential barcode regions searched.
 *    Each region can contain a barcode, so this parameter also limits the maximum
 *    number of barcodes that can be found in an image.
 *
 *    @p ReadBarCode[] defines the types of the barcode being searched for. There
 *    can be any number of different barcodes in an image. As an example, if
 *    a barcode of type EAN13 needs to be decoded, the parameter ReadBarCode[BARCODE_EAN13]
 *    must be set to 1.
 *
 *   @p ReadBarCode | Barcode Type
 *   --------------:|:----------------
 *   BARCODE_25     | 2/5 interleaved
 *   BARCODE_39     | Code 39
 *   BARCODE_EAN128 | EAN 128
 *   BARCODE_EAN13  | EAN 13
 *   PHARMACODE     | Pharmacode
 *   BARCODE_128    | Code 128
 *
 *   @p ISO_15416_Check defines af an ISO 15416 is required to check the print quality
 *   of the code. Set this flag to 1 only if necessary since it slightly slows down
 *   processing.
 */

 typedef struct
 {
     I32    i32BinThr;					/*!< binarization threshold for gradient image (4)											  */
     I32    i32MinContr;				/*!< minimum detection contrast (80), lower value detects more barcodes at lower contrast     */
     I32    i32BCMaxCount;				/*!< number of barcodes (regions) that are searched in the image							  */
     I32    i32BCMinWidth;				/*!< minimum width  = perpendicularly to the bars                                             */
     I32    i32BCMinHeight;				/*!< minimum height = size of bars															  */
     I32    i32MinEdgePercent; 			/*!< reserved, must be set to 10															  */
     I32    ReadBarCode[MAX_BARCODE];	/*!< define barcode types																	  */
     I32    FindMode;					/*!< 0=fast 1=standard 2=precise                                                              */
     I32    ISO_15416_Check;            /*!< Bar Code Print Quality ISO 15416 0: No barcode check  1: check active					  */
 } SBCSettings;
 #define NULL_SBCSETTINGS {0,0,0,0,0,0}
 
 /**  @brief  Barcode Reader Result Struct.  */
 typedef struct
 {
     I32   PosX[4];		/*!< bounding quadrangle x (counting clockwise)				*/
     I32   PosY[4];		/*!< bounding quadrangle y									*/
 
     I32   BarcodeX;	/*!< barcode window center x								*/
     I32   BarcodeY;	/*!< barcode window center y								*/
 
    I32   SizeMajor;	/*!< barcode size perpendicular to bars, mostly larger		*/
    I32   SizeMinor;	/*!< barcode size along bars, mostly smaller				*/
 
     F32   fAngle;		/*!< barcode angle (angle of bars)							*/
     I32   magnitude;	/*!< reserved for future use					            */
 
     char *Text;		 /*!< decoded text                                          */
     I32   TextLength;	 /*!< length of decoded text								*/
 
     I32   BarCodeTyp1;	 /*!< read barcode types e.g. BARCODE_25, BARCODE_39, ...	*/
 //  I16   BarCodeTyp2; was ist BarCodeTyp2 ???
 
   // Bar Code Print Quality ISO 15416 (formally ANSI X3.182-1990)
     I32   ISO_15416_Decode;               /*!< Pass/Fail							*/
     I32   ISO_15416_ReflectanceMin;       /*!< Pass/Fail							*/
     I32   ISO_15416_EdgeContrastMin;      /*!< Pass/Fail							*/
     I32   ISO_15416_SymbolContrast;       /*!< 4 (A) through 0 (F)					*/
     I32   ISO_15416_Modulation;           /*!< 4 (A) through 0 (F)					*/
     I32   ISO_15416_Defects;              /*!< 4 (A) through 0 (F)					*/
     I32   ISO_15416_Decodability;         /*!< 4 (A) through 0 (F)					*/
     I32   ISO_15416_OverallSymbolGrade;   /*!< 4 (A) through 0 (F)					*/
 
 } SBCRegion;
 #define NULL_SBCREGION {0,0, {0,0,0,0}, {0,0,0,0} ,0.0 ,0,0,0,0, 0,0,0,0,0,0,0,0}
 
 
 I32 barcode_search_omnidirectional(image *src, SBCSettings *psBCSet, SPtrList *psLstBCRegion);
 void barcode_search_deallocate(SPtrList *psLstBCRegion);

/**@}@endcond*//*---End of Barcode Reader-------------------------------------*/





/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup SmartFinder   Smart Finder
*
* @{
*/


 /*--*STRUCT*--------------------------------------------------------------*/
 /**
 *
 *  @brief  Smart Finder Parameter Settings.
 *
 *    This structure contains the settings for the smart finder.
 */
 typedef struct
 {
	 I32    angleAng8Start; /*!< Start Angle of Pattern in Search Image  [0..255]            */
	 I32    angleAng8End;   /*!< End   Angle of Pattern in Search Image  [0..255]            */
	 I32    angleAng8Step;  /*!< Angle Search Stepping                                       */

	 F32    scaleMin;       /*!< Minimum Scale of Pattern in Search Image  100% => 1.0       */
	 F32    scaleMax;       /*!< Maximum Scale of Pattern in Search Image  100% => 1.0       */
	 F32    scaleStep;      /*!< Scale Search Stepping                                       */

	 // Expert Options follow.

	 I32    rasters;        /*!< number of successive search stages with refined resolution  */
	 I32   *patRasterSize;  /*!<  array with pattern raster sizes for each stage             */
	 I32   *imgRasterSize;  /*!<  array with image   raster sizes for each stage             */
	 I32   *rasterSizePow;  /*!<  Raster resolution is 2^val pixels.                         */
	 U8     patSegments;    /*!<  number of different pattern segments                       */
	 U8    *patSegWeight;   /*!<  array with weights for each individual segment             */

 } SSFSettings;
 #define NULL_SSFSETTINGS {0,0,0, 0.0f,0.0f,0.0f, 0,NULL, NULL,NULL,0,NULL}



 /*--*STRUCT*--------------------------------------------------------------*/
 /**
 *
 *  @brief  Smart Finder Pattern.
 *
 *    This structure contains a pattern for a smart finder matching operation.
 */
 typedef struct
 {
	 U32    maxSpreadDist; 

	 I32    pxFull;        
	 I16   *xyFull;        
	 U8    *dirFull;       

	 I32    rasters;       
	 I32   *rasterSize;    
	 U8     segments;      
	 U8    *segment;       
	 U8    *segWeight;     

	 I32  **pxCount;       
	 I16 ***xy;            
	 U8  ***dir;           
	 I32 ***dirStart;      

 } SSFPattern;
 #define NULL_SSFPATTERN {0, 0,NULL,NULL, 0,NULL,0,NULL,NULL, NULL,NULL,NULL,NULL}


 /*--*STRUCT*--------------------------------------------------------------*/
 /**
 *
 *  @brief  Smart Finder Result.
 *
 *    This structure contains the result of a smart finder match.
 */
 typedef struct
 {
	 I32    x;             /*!< x-coordinate for match     */
	 I32    y;             /*!< y-coordinate for match     */
	 U8     dir;           /*!< direction for match        */
	 F32    scale;         /*!< scale for match            */
	 I32    strength;      /*!< relative matching strength */

 } SSFResult;
 #define NULL_SSFRESULT {0,0,0,0.0,0}


 I32  sf_init(SSFSettings *psSet);
 I32  sf_deinit(SSFSettings *psSet);
 I32  sf_init_nondefault(SSFSettings *psSet, I32 *rasterSizePow, I32 rasters, I32 *patRasterSize, U8 patSegments, U8 *patSegWeight, I32 *imgRasterSize);

 I32  sf_match_image(image *img, SSFPattern *psPat, SSFSettings *psSet, SSFResult *psRes);
 I32  sf_pattern_from_img(image *vec, SSFPattern *psPat, SSFSettings *psSet);
 I32  sf_pattern_from_file(char *path, SSFPattern *psPat, SSFSettings *psSet);
 I32  sf_pattern_from_xylist_I16(U32 pxCount, I16 *xy, U8 *dir, SSFPattern *psPat, SSFSettings *psSet);
 I32  sf_pattern_to_file(char *path, SSFPattern *psPat);
 I32  sf_pattern_free(SSFPattern *psPat);

 void sf_result_print(SSFResult *psResult);


/**@}@endcond*//*---End of Smartfinder ---------------------------------------*/





/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  testing   Testing
*
*  Functions in this group provide test images and do syntax checking
*
*
* @{
*/
 void  impulse(image *src, U32 value);
 void  TestPattern1(image *a);
 I32   TestPattern2(image *a);
 I32   BayerTest(image *bayer, U32 r, U32 gr, U32 gb, U32 b);
 void  wedge(image *a);

 I32  ColorBar(image *rgb, U32 amplitude, U32 saturation);
 I32  ColorGraph(image *dst, I32 y, I32 sat, I32 mode, F32 start_color);

 void  ChangeValue (I32 *ParValue, I32 Min, I32 Max, I32 Step, I8 *text);

 void  print_image(image *src);
 void  print_imgvar(image *a);

 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib )
 * @internal
 * @defgroup vcmenu  VC Menu (SVCMenu)
 *
 *  @brief  (Beta!) Menu driven Value Modification for Development Purposes.
 *
 *   The main structure SVCMenu can be used to store access and limit
 *   information for parameters that could be changed while the program is
 *   running e.g. a function displays a menu via telnet. Furthermore it is
 *   possible to display the menu 'on screen'.
 *
 *   These are the steps to be done when working with this framework:
 *   1. _Initialize_ each SVCMenu by calling @ref vcmenu_init().
 *   2. _Add Variables_ to those structs using the function @ref vcmenu_add_var().
 *       Remember that the memory area (the parameter) must be writable all
 *       the time to prevent memory corruption by these functions!
 *   3. _Display a menu_ via telnet (ansi or later) by calling @ref vcmenu_show().
 *   4. _Deinitialize_ each initialized SVCMenu if not needed anymore by
 *       calling @ref vcmenu_deinit().
 *
 *
 *
 * @{
 */

  /*
  * @example vcmenu_example.c
  *
  *  Sample usage of the VC Menu.
  */

/*--*GROUP*---------------------------------------------------------------------*/
/**
 * @internal
*@defgroup varTypesEnum  Variable Types Enum (EVarType)
*
*  @brief  Identify C Types.
*
*
* @{
*/

   /*--*ENUM*---------------------------------------------------------------------*/
   /**
   * @internal
   *  Identifier for a Variable Type.
   */
   typedef enum
   {
    TYPE_U8     =   8,  /**< The Identifier for the Type 'U8'.             */
    TYPE_U16    =  16,  /**< The Identifier for the Type 'U16'.            */
    TYPE_U32    =  32,  /**< The Identifier for the Type 'U32'.            */
    TYPE_U40    =  40,  /**< The Identifier for the Type 'U40'.            */
    TYPE_I8     = 108,  /**< The Identifier for the Type 'I8'.             */
    TYPE_I16    = 116,  /**< The Identifier for the Type 'I16'.            */
    TYPE_I32    = 132,  /**< The Identifier for the Type 'I32'.            */
    TYPE_I40    = 140,  /**< The Identifier for the Type 'I40'.            */
    TYPE_F32    = 232,  /**< The Identifier for the Type 'F32'.            */
    TYPE_F64    = 264,  /**< The Identifier for the Type 'F64'.            */
    TYPE_STRING = 300,  /**< The Identifier for the Type 'char *'.         */
    TYPE_MENU   = 500,  /**< The Identifier for the Struct 'SVCMenu'.      */
    TYPE_COMMAND= 501   /**< The Identifier for a Command used by SVCMenu. */
   } EVarType;


   /*--*DEFINE*------------------------------------------------------------------*/
   /**
 * @internal
   *  \def strEnumVarType(e)
   *
   *  'Returns' a String describing the Variable Type enum @a e.
   *
   */
   #define strEnumVarType(e)\
      (e == TYPE_U8     )?("U8"     )\
     :(e == TYPE_U16    )?("U16"    )\
     :(e == TYPE_U32    )?("U32"    )\
     :(e == TYPE_U40    )?("U40"    )\
     :(e == TYPE_I8     )?("I8"     )\
     :(e == TYPE_I16    )?("I16"    )\
     :(e == TYPE_I32    )?("I32"    )\
     :(e == TYPE_I40    )?("I40"    )\
     :(e == TYPE_F32    )?("F32"    )\
     :(e == TYPE_F64    )?("F64"    )\
     :(e == TYPE_STRING )?("STRING" )\
     :(e == TYPE_MENU   )?("MENU"   )\
     :(e == TYPE_COMMAND)?("COMMAND")\
    :"INVALID"


   /*--*DEFINE*----------------------------------------------------------------*/
   /**
 * @internal
   *  \def bytesEnumVarType(e)
   *
   *  'Returns' the Number of Bytes of the Variable Type enum @a e.
   *
   */
   #define bytesEnumVarType(e)\
      (e == TYPE_U8     )?(sizeof(U8     ))\
     :(e == TYPE_U16    )?(sizeof(U16    ))\
     :(e == TYPE_U32    )?(sizeof(U32    ))\
     :(e == TYPE_U40    )?(sizeof(U40    ))\
     :(e == TYPE_I8     )?(sizeof(I8     ))\
     :(e == TYPE_I16    )?(sizeof(I16    ))\
     :(e == TYPE_I32    )?(sizeof(I32    ))\
     :(e == TYPE_I40    )?(sizeof(I40    ))\
     :(e == TYPE_F32    )?(sizeof(F32    ))\
     :(e == TYPE_F64    )?(sizeof(F64    ))\
     :(e == TYPE_STRING )?(0xFFFFFFFF     )\
     :(e == TYPE_MENU   )?(sizeof(SVCMenu))\
     :(e == TYPE_COMMAND)?(sizeof(I32    ))\
    :(0xFFFFFFFF) /* Memory Allocation should fail on unknown type */

  /**@}@endcond*//*---End of Variable Types Enum (EVarType)---------------------*/


  /*--*STRUCT*----------------------------------------------------------------*/
  /**@internal
  *   Used by SVCMenu per Variable stored therein.
  *
  */
  typedef struct _SVCMenuVar
  {
    struct _SVCMenuVar  *psNext;   /**< Next Variable or NULL                                                   */
    U8        u8UnchangeableIff1;  /**< Set to 1 for Readonly Variables                                         */
    void     *pvVar;               /**< Variable                                                                */
    void     *pvBackup;            /**< Backup of Variable Content                                              */
    char     *pcVarName;           /**< Name of the Variable                                                    */
    EVarType  eVarType;            /**< Byte Size 8,16,32,...                                                   */
    F64       dMinVal;             /**< Minimum Value                                                           */
    F64       dDefVal;             /**< Default Value                                                           */
    F64       dMaxVal;             /**< Maximum Value                                                           */
    F64       dStepSize;           /**< Step Size                                                               */
    U32       u32AccessID;         /**< (dummy) May later be used for identifying the Variable                  */
    U32       u32TextID;           /**< (dummy) Text ID for international Language Support, or help texts, etc. */

  } SVCMenuVar;
  #define  NULL_SVCMENUVAR   {NULL, 0, NULL, NULL, NULL, 0, 0, 0, 0, 0, 0, 0}



  /*--*STRUCT*----------------------------------------------------------------*/
  /**@internal
  *  Bundles Information of Parameters to access their Memory for
  *  Changing Purposes by displaying a Menu.
  *
  */
  typedef struct _SVCMenu
  {
    char       *pcMenuTopic;     /**< Topic will be displayed.                            */
    I32         i32VarCount;     /**< Count of Variables that can be changed in the Menu  */
    I32         i32VarSelected;  /**< Holds the Variable which was selected before        */
    SVCMenuVar *psVar;           /**< Each Variable inserts such a struct                 */
    I32       (*pFBeforeDisplaying)(struct _SVCMenu *, void *); /**< Function to execute before displaying anything, or NULL; Return Value != ERR_NONE will quit the Menu Display returning that Return Value.  */
    void       *pvBeforeDisplayingArg;  /**< Will be passed to the same named function.   */
    I32       (*pFAfterValueChange)(struct _SVCMenu *, void *); /**< Function to execute after each Change, or NULL; Return Value != ERR_NONE will quit the Menu Display returning that Return Value.  */
    void       *pvAfterValueChangeArg;  /**< Will be passed to the same named function.   */
    char       *pcTextForSpace;  /**< Will be written on bottom of the menu as Text, should contain information about what's happening if the user presses 'space'; should not be long. If set to NULL, the standard text is used 'go back'. */
    U8          u8TextSize;      /**< Text Pixel Height (*8) of Menu Text if displayed.   */
    U8          u8ColMenuBg;     /**< Background Color of Menu.                           */
    U8          u8ColMenuText;   /**< Text Color of Menu.                                 */
    U32         u32AccessID;     /**< (dummy) May later be used for identifying the Menu  */
    U32         u32TextID;       /**< (dummy) Text ID for international Language Support, or help texts, etc. */

  } SVCMenu;
  #define  NULL_SVCMENU   {NULL, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, 0, 0}

  /**
   * @internal
   * @{
   */

  I32  vcmenu_add_var(SVCMenu *psVCMenu, U32 u32AccessID, char *pcVarName, void *pvVar, EVarType eVarType, F64 dDefVal, F64 dMinVal, F64 dMaxVal, F64 dStepSize, U8 u8UnchangeableIff1, U32 u32TextID);
//  I32  vcmenu_init(SVCMenu *psVCMenu, char *pcMenuTopic, char *pcTextForSpaceOrNULL, U32 u32AccessID, U32 u32TextID);
  I32  vcmenu_init(SVCMenu *psVCMenu, char *pcMenuTopic, U32 u32AccessID, U32 u32TextID);
  void vcmenu_deinit(SVCMenu *psVCMenu);
  I32  vcmenu_access_backup_varnr(SVCMenu *psVCMenu, I32 i32VarNr, U8 u8RevertFromBackupIff1);
  I32  vcmenu_access_backup(SVCMenu *psVCMenu, U8 u8RevertFromBackupIff1);
//  I32  vcmenu_show(SVCMenu *psVCMenu, image *psDispOrNULL);
  I32  vcmenu_show(SVCMenu **ppsVCMenu, image *psDisp, U8 u8Change_ppsVCMenuIff1);
  I32  vcmenu_set_default_number_values(SVCMenu *psVCMenu);
  I32  vcmenu_get_var_pointer_by_id(SVCMenu *psVCMenu, U32 u32AccessID, SVCMenuVar **ppsMenuVarOut);
  I32  vcmenu_get_var_content_by_id(SVCMenu *psVCMenu, U32 u32AccessID, void *pBufferOut, U32 u32BufferByteCount);
  I32  vcmenu_set_var_content_by_id(SVCMenu *psVCMenu, U32 u32AccessID, void *pBufferSameTypeIn, EVarType eBufferVarType, U32 u32BufferByteCount);

  /**@} */
 /**@}@endcond *//*---End of VC Menu (SVCMenu)---------------------------------*/



 /*--*GROUP*------------------------------------------------------------------*/
 /**
 *@cond ( vclib )
 *
 * @defgroup  debug_display_environment  Debug Display Environment
 *
 *  @brief  Output several labelled images to an image (display screen) for Development Purposes.
 *
 *  The functions in this group allow the display of a large number of images
 *  on the screen in an ordered way. The images are resized by request to the
 *  desired format. Each image can be assigned a text string which is displayed
 *  underneath the image.
 *
 * @image  html  debug_display_raster.png
 * @image  latex debug_display_raster.pdf
 *
 *   You configure a raster containing rectangular regions of indices, where
 *   the images will later be drawn to. This is done by passing numbers starting
 *   at '0' and raising up to the count of needed images. For convinience, it is
 *   the easiest way to define them in an array and provide it as pointer to the
 *   specific information holder.
 *
 *   First Initialisation is made by calling the function @ref init_debug_display().
 *   To free allocated memory, one has to call @ref deinit_debug_display() at the end.
 *
 *   Access to the images is easy:
 *   - Either by direct image access (SDebugDisplay.psImg + IndexNr) as a
 *     pointer to an image struct,
 *   - or by calling the function @ref debug_display_insert() for setting labels and
 *     scale input images to the debug image size.
 *
 *  @code
 *      SDebugDisplay  sDD = NULL_DEBUGDISPLAY;
 *      U16    au16[2][5] =
 *      {
 *          { 0, 0, 1, 1, 2 },
 *          { 0, 0, 3, 4, 2 }
 *      };
 *
 *    init_debug_display(&sDD, "Title Text", 2, 5, &(au16[0][0]), &dispImgOut, NULL);
 *    debug_display_draw_environment(&sDD);
 *
 *    // ...
 *    debug_display_insert(&capt,"mc",0,"Image0 Label",1,1,&sDD,0);
 *    debug_display_insert(&capt,"mc",0,"Image1 Label",1,0,&sDD,1);
 *    debug_display_insert(&capt,"mc",0,"Image2 Label",1,0,&sDD,2);
 *    debug_display_insert(&capt,"mc",0,"Image3 Label",1,0,&sDD,3);
 *    debug_display_insert(&capt,"mc",0,"Image4 Label",1,1,&sDD,4);
 *    // ...
 *
 *    deinit_debug_display(&sDD);
 *
 *  @endcode
 *
 *
 * @{
 */


  /*--*STRUCT*----------------------------------------------------------------*/
  /**
  * @brief  Data Keeper and Management for the Debug Display Output Routine.
  */
  typedef struct _SDebugDisplay
  {
    U8  u8InactiveIff1;    /**< A switch that turns the processing off, useful for deactivating debug displaying. */

    image  sDisp;          /**< The whole Output Image.                                                           */
    image  sOvl;           /**< The whole Output Overlay Image (optional).                                        */
    char  *pcTitle;        /**< The Title of the Debug Display.                                                   */
    U32    u32RasterRows;  /**< Main Raster Row Count of the Image Gallery.                                       */
    U32    u32RasterCols;  /**< Main Raster Column Count of the Image Gallery.                                    */
    U16    u16ImageCount;  /**< The Maximum valid Image Nr available for Destination Request.                     */
    U16  **ppu16MeldIdx;   /**< Each Image Area will be going over the raster cells with the same number therein. */
    image *psImg;          /**< The Images accessable by the MeldIdx as Index.                                    */
    image *psLabel;        /**< The Label Text Area for the Images.                                               */
    image *psOvlImg;       /**< The Images accessable by the MeldIdx as Index.                                    */
    image *psOvlLabel;     /**< The Label Text Area for the Images.                                               */

    U8  u8TitleSize;       /**< Size in 8* is Px Height of Title Chars.                                           */
    U8  u8TitleHeightPx;   /**< Height for the Title.                                                             */
    U8  u8TextHeightPx;    /**< Height for the Subtext.                                                           */
    U8  u8PadColsPx;       /**< Horizontal Space between Images.                                                  */
    U8  u8PadRowsPx;       /**< Vertical   Space between Images.                                                  */

    U8  u8ImgColorBg;      /**< Color Value for the Image Background.                                             */
    U8  u8ImgColorText;    /**< Color Value for the Label Text.                                                   */
    U8  u8ImgColorTextBg;  /**< Color Value for the Label Areas.                                                  */
    U8  u8ImgColorTitle;   /**< Color Value for the Title Text.                                                   */
    U8  u8ImgColorTitleBg; /**< Color Value for the Title Area.                                                   */
    U8  u8ImgColorBorder;  /**< Color Value for the Border around the Images.                                     */
    U8  u8OvlColorBg;      /**< Color Value for the Overlay Background (mostly 0).                                */
    U8  u8OvlColorText;    /**< Color Value for the Overlay Label Text.                                           */
    U8  u8OvlColorTextBg;  /**< Color Value for the Overlay Label Areas.                                          */
    U8  u8OvlColorTitle;   /**< Color Value for the Overlay Title Text.                                           */
    U8  u8OvlColorTitleBg; /**< Color Value for the Overlay Title Area.                                           */
    U8  u8OvlColorBorder;  /**< Color Value for the Overlay Border around the Images.                             */

  } SDebugDisplay;
  #define  NULL_DEBUGDISPLAY   {0,  NULL_IMAGE, NULL_IMAGE, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL,   0,0,0,0,0,   0,0,0,0,0,0,0,0,0,0,0,0}


  I32   init_debug_display(SDebugDisplay *psDD, char *pcTitle, U32 u32RasterRows, U32 u32RasterCols, U16 *pu16Meld, image *psDisplay, image *psOvlOrNULL);
  I32   deinit_debug_display(SDebugDisplay *psDD);
  void  debug_display_draw_environment(SDebugDisplay *psDD);
  I32   debug_display_insert(image *psImgIn, char *pcImgAnchor, U8 u8ToOverlayIff1, char *pcText, U8 u8ScaleContentIff1, U8 u8FitWillCropIff1, SDebugDisplay *psDD, U16 u16ImageIdx);

 /**@}@endcond*//*---End of Debug Display Environment-------------------------*/
/**@}@endcond*//*---End of Testing--------------------------------------------*/










/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vclib || extlib )
*
* @defgroup  cplusplusWrapper  C++ Wrapper Functions.
*
*  Functions in this group provide Access to C defines which are not allowed in C++ code.
*  The naming convention is  cpp_<Macro Name>  where also the macro is documented.
*
*
* @{
*/

void  cpp_binarize(  image *src, image *dest, I32 thr, I32 bl, I32 wt);
void  cpp_PaintWhite(image *src, image *dest, I32 thr,         I32 wt);
void  cpp_PaintBlack(image *src, image *dest, I32 thr, I32 bl        );
void  cpp_OR_image(  image *src, image *dest, I32 c                  );


void  cpp_add2( image *a, image *b, image *c, I32 sh  );
void  cpp_sub2( image *a, image *b, image *c          );
void  cpp_subx2(image *a, image *b, image *c, I32 offs);
void  cpp_suby2(image *a, image *b, image *c          );
void  cpp_mul2( image *a, image *b, image *c, I32 sh  );
void  cpp_max2( image *a, image *b, image *c          );
void  cpp_min2( image *a, image *b, image *c          );
void  cpp_and2( image *a, image *b, image *c          );
void  cpp_or2(  image *a, image *b, image *c          );
void  cpp_xor2( image *a, image *b, image *c          );


void  cpp_sobel(  image *a, image *b);
void  cpp_sobelh( image *a, image *b);
void  cpp_sobelv( image *a, image *b);
void  cpp_laplace(image *a, image *b);

void  cpp_mx(     image *a, image *b);
void  cpp_mn(     image *a, image *b);
void  cpp_avg_3x3(image *a, image *b);


I32  cpp_avgm( image *a, image *b, I32 kx, I32 ky        );
I32  cpp_maskx(image *a, image *b, I32 kx, I32 ky, I32 v );
I32  cpp_masky(image *a, image *b, I32 kx, I32 ky        );

I32  cpp_avgm2( image *a, image *b, I32 kx, I32 ky       );
I32  cpp_maskx2(image *a, image *b, I32 kx, I32 ky, I32 v);
I32  cpp_masky2(image *a, image *b, I32 kx, I32 ky       );

I32  cpp_avgm_1xN( image *a, image *b, I32 ky            );
I32  cpp_maskx_1xN(image *a, image *b, I32 ky, I32 v     );
I32  cpp_masky_1xN(image *a, image *b, I32 ky            );

I32  cpp_avgm2_1xN( image *a, image *b, I32 ky           );
I32  cpp_maskx2_1xN(image *a, image *b, I32 ky, I32 v    );
I32  cpp_masky2_1xN(image *a, image *b, I32 ky           );



I32  cpp_BayerToRGB(         image*  bayer, image* rgb              );
I32  cpp_BayerToRGB_Centred( image*  bayer, image* rgb              );
I32  cpp_BayerToGrey(        image*  bayer, image* grey             );
I32  cpp_BayerToGrey_Centred(image*  bayer, image* grey             );
I32  cpp_BayerToYCbCr(       image*  bayer, image* ycbcr            );
I32  cpp_BayerToYCbCr411(    image*  bayer, image* ycbcr            );
I32  cpp_BayerToYCbCr422(    image*  bayer, image* ycbcr            );
I32  cpp_BAYER_RGBO(         image*  bayer, image* ovl,  image* rgbo);
I32  cpp_BAYER_GREY_RGBO(    image*  bayer, image* ovl,  image* rgbo);


I32  cpp_pyramid(image *a, image *b);
I32  cpp_pyr_max(image *a, image *b);
I32  cpp_pyr_min(image *a, image *b);
I32  cpp_pyr_col(image *a, image *b);

I32  cpp_resample_bilinear( image *src, image *dst);
I32  cpp_resample_quadratic(image *src, image *dst);

I32  cpp_draw_lined(  image *a, F32 cx, F32 cy, F32 b, I32 c);
I32  cpp_draw_linex(  image *a, F32 cx, F32 cy, F32 b, I32 c);
I32  cpp_draw_vclined(image *a, vcline *line,           I32 c);
I32  cpp_draw_vclinex(image *a, vcline *line,           I32 c);

I32  cpp_lined(image *a, I32 x1, I32 y1, I32 x2, I32 y2, I32 c);
I32  cpp_linex(image *a, I32 x1, I32 y1, I32 x2, I32 y2, I32 c);
I32  cpp_lineo(image *a, I32 x1, I32 y1, I32 x2, I32 y2       );
I32  cpp_linez(image *a, I32 x1, I32 y1, I32 x2, I32 y2       );


void  cpp_MarkBox(      image *img, I32 x, I32 y, I32 dx, I32 dy, I32 col);
void  cpp_MarkBoxd(     image *img, I32 x, I32 y, I32 dx, I32 dy, I32 col);
void  cpp_MarkBoxx(     image *img, I32 x, I32 y, I32 dx, I32 dy, I32 col);
void  cpp_MarkSolidBox( image *img, I32 x, I32 y, I32 dx, I32 dy, I32 col);
void  cpp_MarkSolidBoxd(image *img, I32 x, I32 y, I32 dx, I32 dy, I32 col);
void  cpp_MarkSolidBoxx(image *img, I32 x, I32 y, I32 dx, I32 dy, I32 col);

void  cpp_MarkCross(    image *img, I32 x, I32 y, I32 col, I32 size);
void  cpp_MarkCrossd(   image *img, I32 x, I32 y, I32 col, I32 size);
void  cpp_MarkCrossx(   image *img, I32 x, I32 y, I32 col, I32 size);

void  cpp_draw_polygond( image *a, U32 cornercount, I32 *pi32PxLst, I32 col);
void  cpp_draw_polygonx( image *a, U32 cornercount, I32 *pi32PxLst, I32 col);
void  cpp_draw_polylined(image *a, U32 cornercount, I32 *pi32PxLst, I32 col);
void  cpp_draw_polylinex(image *a, U32 cornercount, I32 *pi32PxLst, I32 col);

I32  cpp_draw_circled(image *a, I32 px, I32 py, I32 r, I32 c);
I32  cpp_draw_circlex(image *a, I32 px, I32 py, I32 r, I32 c);

void  cpp_ellipsed(image *a, I32 c  );
void  cpp_ellipsex(image *a, I32 c  );
void  cpp_ellipseo(image *a         );
void  cpp_ellipsez(image *a         );

I32  cpp_draw_histod(image *Graph, U32 *Histo, I32 size, I32 BackgndColor, I32 Color, I32 i32Mode);
I32  cpp_draw_histox(image *Graph, U32 *Histo, I32 size, I32 BackgndColor, I32 Color, I32 i32Mode);


void  cpp_framed(  image *a, I32 col);
void  cpp_framex(  image *a, I32 col);
void  cpp_frameo(  image *a         );
void  cpp_framez(  image *a         );
void  cpp_dframed( image *a, I32 col);
void  cpp_dframex( image *a, I32 col);
void  cpp_dframeo( image *a         );
void  cpp_dframez( image *a         );
void  cpp_markerd( image *a, I32 col);
void  cpp_markerx( image *a, I32 col);
void  cpp_markero( image *a         );
void  cpp_markerz( image *a         );
void  cpp_dmarkerd(image *a, I32 col);
void  cpp_dmarkerx(image *a, I32 col);
void  cpp_dmarkero(image *a         );
void  cpp_dmarkerz(image *a         );



U8  *cpp_cjpeg_d(image *img, I32 qual, U8* addr, U32 maxlng);
U8  *cpp_cjpeg_f(image *img, I32 qual, U8* addr, U32 maxlng);

U8 *cpp_xjpeg_d(image *img, I32 qual, U8 *addr, U32 maxlng);
U8 *cpp_xjpeg_f(image *img, I32 qual, U8 *addr, U32 maxlng);

U8 *cpp_djpeg_d(image *img, U8 *addr);
U8 *cpp_djpeg_f(image *img, U8 *addr);

U16 *cpp_rlcand(    U16 *a, U16 *b, U16 *c);
U16 *cpp_rlcor(     U16 *a, U16 *b, U16 *c);
U16 *cpp_rlcxor(    U16 *a, U16 *b, U16 *c);
U16 *cpp_rlcnand(   U16 *a, U16 *b, U16 *c);
U16 *cpp_rlcnor(    U16 *a, U16 *b, U16 *c);
U16 *cpp_rlcequiv(  U16 *a, U16 *b, U16 *c);
U16 *cpp_rlcandnot( U16 *a, U16 *b, U16 *c);
U16 *cpp_rlcornot(  U16 *a, U16 *b, U16 *c);
U16 *cpp_rlcnandnot(U16 *a, U16 *b, U16 *c);
U16 *cpp_rlcnornot( U16 *a, U16 *b, U16 *c);

U16 *cpp_erode(  U16 *a, U16 *b);
U16 *cpp_dilate( U16 *a, U16 *b);

U16 *cpp_erode2( U16 *a, U16 *b);
U16 *cpp_dilate2(U16 *a, U16 *b);

void cpp_cdisp_d(image *a, U32 *src, I32 col);
void cpp_cdisp_x(image *a, U32 *src, I32 col);
void cpp_cdisp_o(image *a, U32 *src, I32 col);
void cpp_cdisp_z(image *a, U32 *src, I32 col);


/**@}@endcond*//*---End of C++ Wrapper --------------------------------------------*/


















#ifdef __cplusplus
 }
#endif


#endif
/* EOF */

