/**********************************************************************//**
***************************************************************************
*** @file   vcrt.h
***
*** @brief  Main VCRT header file
***
*** @author  Copyright (c) 2004-2014 Vision Components
*** @author  All rights reserved.
*** @author  This software embodies materials and concepts which are
***          confidential to Vision Components.
***
***
*** @revisionHistory
***
***    Date          Version    Author  Changes
***    Jul 05, 2002  5.00       ME      Initial Version for VCRT V5.00
***    Apr 29, 2003  5.05       ME      update
***    Jul 19, 2004  5.17       ME      cosmetic update
***    Oct 04, 2005  5.23       MS      added image timestamps,
***                                     added io_fseek(), io_ftell()
***    Dec 19, 2005  5.23       MS      added io_fprintf(), io_fscanf()
***    Jan 18, 2006  5.23       MS      move old functions in vcrt_compat.h
***    Feb 03, 2006  5.23       MS      added display sync functions
***    Feb 21, 2006  5.24       MS      added Encoder Macros
***    Oct 13, 2006  5.25       MS      added Timer2 Macros
***    Nov 06, 2006  5.25       MS      added Beckhoff support
***    Dec 22, 2006  5.25.4     MS      change layout
***    Jan 12, 2007  5.25.4     MS      added exec3
***    Apr 25, 2007  5.26       MS      added TCP/IP error codes
***    Jun 27, 2007  5.26       MS      added DNS resolver routines
***    Mar 11, 2011  5.26       KS      new defines for MSVC compatibility
***    Oct 17, 2012  5.30.9     MS      added new hardware platforms  
***
*** @endRevisionHistory
***************************************************************************
***************************************************************************/
#ifndef _VCRT_h
#define _VCRT_h

#define VCRTLIB_VERSION    531
#define VCRTLIB_SUBVERSION   0 

/* VCRTLIB version functions */

void GetLibVer_VCRTLIB (int *Version, int *SubVersion);
char *GetLibVerString_VCRTLIB (void);






#include <vctypes.h>
/*  
typedef unsigned char  uint_08;
typedef unsigned short uint_16;
typedef unsigned int   uint_32;
typedef int            int_32;
typedef short          int_16;
typedef void *         pointer;

typedef unsigned char  U8;
typedef unsigned short U16;
typedef unsigned int   U32;
typedef char           I8;
typedef short          I16;
typedef int            I32;

typedef float          F32;
typedef double         F64;

#if defined __GNUC__
  #define VC_BASICTYPES
  typedef signed   long long I40;
  typedef unsigned long long U40;
  typedef unsigned long long U64;
  typedef signed   long long I64;

  #define   VCRT_NO_COMPAT
  #define  _nassert
  #define  _abs            abs
#endif

#if defined __TI_COMPILER_VERSION__
#define VC_BASICTYPES
typedef long           I40;
typedef unsigned long  U40;
  #if __TI_COMPILER_VERSION__ >= 5000000
  typedef unsigned long long U64;
  typedef   signed long long I64;
  #endif
#endif

#ifndef VC_BASICTYPES
#define VC_BASICTYPES
typedef unsigned __int64 U40;           // Microsoft Visual C++ 6.0 definitions 
typedef          __int64 I40;
typedef unsigned __int64 U64;
typedef          __int64 I64;

#define   VCRT_NO_COMPAT
#define  _nassert
#define  _abs            abs

#endif

#ifdef  NULL
   #undef  NULL
#endif

#ifdef __cplusplus
   #define NULL (0)
#else
   #define NULL ((pointer)0)
#endif
*/

#ifdef TRUE
   #undef TRUE
#endif

#ifdef FALSE
   #undef FALSE
#endif

#define TRUE    (1)
#define FALSE   (0)

/*
#ifdef __cplusplus
   #define malloc(nwords)   sysmalloc((unsigned) nwords, 0)
   #define malloc(nbytes)   byte_malloc(nbytes)
   #define free(ap)         sysfree(ap)
#endif
*/

#ifdef __cplusplus
   extern "C" {
#endif

/*
 * define VCRT_COMPAT 
 * if you do want to use deprecated functions !!!
 * define VCRT_NO_COMPAT
 * if you do NOT want to use deprecated functions !!!
 */
#ifndef VCRT_NO_COMPAT
 #define VCRT_COMPAT
#endif
#ifdef VCRT_COMPAT
 #include <vcrt_compat.h> 
#endif

/* **** */


/*--*GROUP*-------------------------------------------------------------------*/
/**
*@cond ( vcrtlib )
*
* @defgroup videoControlFunctions   Video Control Functions
*
*   Functions in this group apply to the sensor of the camera.
*
* @{
*/

 int  capture_request(int exp, int xgain, int *start, int mode);
 /* capture_request mode bits */
 #define EXTTRIG 1   /* external trigger     */
 #define BINNING 8   /* binning mode         */
 int  cancel_capture_rq(void);
 
 void vmode(int);
 void tpict(void);
 int  tpp(void);
 int  tpstart(void);
 int  tenable(void);
 int  trdy(void);
 long shutter(long);
 
 /* tpwait() could use IMAGE_READY event in any process since VCRT/VCRTLIB 5.26
    because events were connected automatically when linking with vcrt4_526.lib */
 #define tpwait() \
   if( ( (getvar(VERSION) >  525) && (getvar(TESTVERSION) > 3) ) || (getvar(VERSION) >= 526) ) \
     while(!trdy()) wait(IMAGE_READY,1000); \
   else \
     while(!trdy()) 
 
 
/**@}@endcond*//*---End of Video Control Functions---------------------------*/


/* terminal functions */

void rs232snd(char); // see macro putchar
char rs232rcv(void); // see macro getchar
int sbready(void);
int rbready(void);
char kbdrcv(void);
int kbready(void);

/* LUT functions */

void set_ovlmask(int mask);
int set_overlay_bit(int bit, int r, int g, int b);
int set_translucent(int table, int r, int g, int b);
void init_LUT(void);
void init_LUT_gamma(float gamma);

/* utility functions */

int hextoi(char *s);                        /* hexadecimal to integer     */
char tohex(int val);                        /* integer to hexadecimal     */
void setbaud(long baud);                    /* set baud rate              */
int  getvar(int);
float getfvar(int);
long getlvar(int);
void setvar(int, int);
void setlvar(int, long);
void setfvar(int, float);
 
/*
void *sysmalloc(unsigned nwords, int type);
int sysfree(void *ap);

// macros   

#if defined __TI_COMPILER_VERSION__
 #define getchar rs232rcv
 #define putchar rs232snd

 #define printf print

 void pstr(char *);
 void _pstr(char *);
 void print(char *, ...);
 void _print(char *, ...);

 #define sprintf io_sprintf
 #define sprint io_sprintf
 void sprint(char *s, char *pfmt,...);
 void io_sprintf(char *s, char *pfmt,...);

 #define fprintf io_fprintf
 #define fscanf io_fscanf

#else
 #define print printf
 #define pstr  printf
 #define rs232rcv   getchar
 #define rs232snd   putchar 
#endif
*/


#define setRTS() {int p=1; io_ioctl(stdout,IO_IOCTL_SERIAL_SET_HW_SIGNAL,&p);}
#define resRTS() {int p=1; io_ioctl(stdout,IO_IOCTL_SERIAL_CLR_HW_SIGNAL,&p);}

#define setPLC0() outPLC(getvar(PLCOUT) | 1)
#define setPLC1() outPLC(getvar(PLCOUT) | 2)
#define setPLC2() outPLC(getvar(PLCOUT) | 4)
#define setPLC3() outPLC(getvar(PLCOUT) | 8)
#define resPLC0() outPLC(getvar(PLCOUT) & ~1)
#define resPLC1() outPLC(getvar(PLCOUT) & ~2)
#define resPLC2() outPLC(getvar(PLCOUT) & ~4)
#define resPLC3() outPLC(getvar(PLCOUT) & ~8)

void outPLC(int x);
void set_rts(int x);

#define inPLC() (getvar(PLCIN)&0x0f)

/* time function macros                     */

void c_time(long zsec, int tz, int *sec, int *min, int *hour);
void RTC_set_time(void);

#define get_local_timedate(a,b,c,d,e,f,g)   get_timedate(getvar(TIMEZONE)+ \
                                            getvar(DAYLIGHT),a,b,c,d,e,f,g)
#define get_gmt_timedate(a,b,c,d,e,f,g)     get_timedate(0,a,b,c,d,e,f,g)

#define get_local_time(a,b,c,d)             get_time(getvar(TIMEZONE)+     \
                                            getvar(DAYLIGHT),a,b,c,d)
#define get_gmt_time(a,b,c,d)               get_time(0,a,b,c,d)

#define get_local_date(a,b,c)               get_date(getvar(TIMEZONE)+     \
                                            getvar(DAYLIGHT),a,b,c)
#define get_gmt_date(a,b,c)                 get_date(0,a,b,c)

#define ltimedate(a,b,c,d,e,f)              c_timedate(getlvar(SEC),       \
                                            getvar(TIMEZONE)+              \
                                            getvar(DAYLIGHT),a,b,c,d,e,f)

#define ltime(a,b,c)                        c_time(getlvar(SEC),           \
                                            getvar(TIMEZONE)+              \
                                            getvar(DAYLIGHT),a,b,c)

#define ldate(a,b,c)                        c_date(getlvar(SEC),           \
                                            getvar(TIMEZONE)+              \
                                            getvar(DAYLIGHT),a,b,c)

#define gtimedate(a,b,c,d,e,f)              c_timedate(getlvar(SEC),       \
                                            0,a,b,c,d,e,f)

#define gtime(a,b,c)                        c_time(getlvar(SEC),0,a,b,c)

#define gdate(a,b,c)                        c_date(getlvar(SEC),0,a,b,c)

void c_timedate(long z, int tz, int *s, int *m, int *h, int *d, int *mo, int *yr);
unsigned long x_timedate(int tz, int s, int m, int h, int d, int mo, int yr);

#define xtimedate(a,b,c,d,e,f)              setlvar(SEC,x_timedate(        \
                                            getvar(TIMEZONE)+              \
                                            getvar(DAYLIGHT),a,b,c,d,e,f))

/* utilities                                */

#define vcsetup()
//#define vcmalloc(x)     sysmalloc((x),MDATA)
//#define vcfree(x)       sysfree((x))
#define prtfree()       sysprtfree(MDATA)
//#define byte_malloc(x)  sysmalloc(((x)+3)/4,MDATA)
//#define byte_free(x)    sysfree((x))

int getstptr(void);
int getdp(void);
int getbss(void);

void disints(void); /* disable interrupts */
void enaints(void); /* enable  interrupts */

void set_ctrl_reg (unsigned int);
void res_ctrl_reg (unsigned int);
void set_ctrl_reg2(unsigned int);
void res_ctrl_reg2(unsigned int);

void reset(void);

int set_binning(int on); /* VC4008E high speed binning */

// new in VCRT 5.29.18 HAL wrapper routines - DO NOT use directly !!! use MACROS instead !!!
int set_trigout_polarity(int pol);
int set_trigout_mode(int mode);
int set_trigout(int pol);

int set_trigin_int(int onoff);
int set_trigin_pol(int pol);
int set_trigin_sticky(int onoff);
int get_trigin();

int vcrt_get_hw_status();
int vcrt_uart_installed();
int vcrt_ledcon_installed();
int vcrt_led_out(int x);
int vcrt_ttl_out(int x);
int vcrt_get_ttl_in();

int vcrt_sensor_reset(int x); // x=0 (normal reset - if capturing returns -1 else 0   )
                              // x=1 (forced reset - always returns 0 - since 5.30.13 )

/* VIRTX_CTRL register macros                   */

/* rs232 interface VC20XX only */
#define SET_RTS()           set_ctrl_reg(RTS_OUT)                           /* set RTS signal       */
#define RES_RTS()           res_ctrl_reg(RTS_OUT)                           /* reset RTS signal     */

/* trigger output macros */
#define SET_TRIGOUT()       set_trigout(1)                                  /* set trigger output   */
#define RES_TRIGOUT()       set_trigout(0)                                  /* reset trigger output */
#define TRIGOUT_POS()       set_trigout_polarity(1)                         /* trigger out positive */
#define TRIGOUT_NEG()       set_trigout_polarity(0)                         /* trigger out negative */
#define TRIGOUT_EXP()       set_trigout_mode(1)                             /* trigger hw. exposure */
#define TRIGOUT_USR()       set_trigout_mode(0)                             /* trigger out user def */

/* trigger input macros */
#define SET_trig_sticky()   set_trigin_sticky(1)                            /* trigger input sticky */
#define SET_trig_lossy()    set_trigin_sticky(0)                            /* trigger input lossy  */
#define TRIGINP_POS()       set_trigin_pol(1)                               /* trigger input pos.   */
#define TRIGINP_NEG()       set_trigin_pol(0)                               /* trigger input neg.   */
/* new for VCRT 5.29 trigger interrupt */
#define TRIG_INT_ENABLE()   set_trigin_int(1)                               /* enable trigger interupt */
#define TRIG_INT_DISABLE()  set_trigin_int(0)                               /* disable trigger interupt*/
/* new for VCRT 5.30 TRIGINP_PIN is now function which returns trigin pin state */
#define TRIGINP_PIN         get_trigin()                                    /* trigger input pin state */     

/* SBC400X, SBC401X, SBC42XX, SBC60XX, SBC62XX, SBC72XX functions */

/* new for VCRT 5.30.4 MACROS point now to wrapper functions      */
#define SET_BINNING()       set_ctrl_reg(HIGH_binn)                         /* highspeed binning VC4008E */
#define RES_BINNING()       res_ctrl_reg(HIGH_binn)                         /* highspeed binning VC4008E */
#define SET_BOOST()         set_ctrl_reg(BOOST)                             /* boost / SBC/VC4018E  */
#define RES_BOOST()         res_ctrl_reg(BOOST)                             /* boost / SBC/VC4018E  */
#define ILLU_POS()          res_ctrl_reg(ILLU_pol)                          /* illumination polarity*/
#define ILLU_NEG()          set_ctrl_reg(ILLU_pol)                          /* illumination polarity*/
#define ILLU_USR()          set_ctrl_reg(ILLU_usr)                          /* illumination user    */
#define ILLU_EXP()          res_ctrl_reg(ILLU_usr)                          /* illumination normal  */
#define GET_HW_STATUS()     vcrt_get_hw_status()                            /* CPLD release number  */
#define GET_TRIG_SIGNAL()   get_trigin()                                    /* trigger signal       */
#define UART_INSTALLED()    vcrt_uart_installed()                           /* 1 if UART installed  */
#define TTL_OUT(x)          vcrt_ttl_out(x)                  /* SBC/VC6210 and SBC/VC4018 TTL output*/
#define LED_OUT(x)          vcrt_led_out(x)                  /* SBC/VC6210 and SBC/VC4018 LED output*/
#define GET_TTL_IN()        vcrt_get_ttl_in()                /* SBC/VC6210 and SBC/VC4018 TTL input */
#define LEDCON_INSTALLED()  vcrt_ledcon_installed()                         /* 1 if LEDCON installed*/

/* VCRT 5.0 defines     */

/* Seek parameters */
#define IO_SEEK_SET (1) /* Seek from start              */
#define IO_SEEK_CUR (2) /* Seek from current location   */
#define IO_SEEK_END (3) /* Seek from end                */

/* IOCTL commands */
#define IO_BAUD_SET      (0x0104) /* set baud rate                */
#define IO_BAUD_GET      (0x0103) /* get baud rate                */
#define IO_RTS_SET       (0x010A) /* set RTS to 1                 */
#define IO_RTS_CLR       (0x010B) /* set RTS to 0                 */
#define IO_PACK          (0x1001) /* pack files / directory       */
#define FLASH_IOCTL_PACK IO_PACK 
#define IO_ERASE         (0x1002) /* erase                        */
#define IO_READDIR       (0x1003) /* read directory               */
#define IO_CHKSYS        (0x1004) /* check system                 */
#define IO_DEL           (0x1005) /* delete file / directory      */
#define IO_MKDIR         (0x1006) /* make directory               */
#define IO_PIPE_CHMOD    (0x1003) /* change mode                  */
#define IO_PIPE_CHSIZ    (0x1004) /* change size and reset pipe   */
#define IO_PIPE_RDFLAGS  (0x1005) /* read out mode flags          */
#define IO_PIPE_GETCOUNT (0x1001) /* get number of characters     */
#define IO_PIPE_SIZE     (0x1002) /* size of pipe                 */
#define IO_REMAIN        (0x1007) /* remaining device space       */

/* serial io */
#define IO_IOCTL_SERIAL_GET_FLAGS     (0x0101)
#define IO_IOCTL_SERIAL_SET_FLAGS     (0x0102)
#define IO_IOCTL_SERIAL_GET_BAUD      (0x0103)
#define IO_IOCTL_SERIAL_SET_BAUD      (0x0104)
#define IO_IOCTL_SERIAL_GET_STATS     (0x0105)
#define IO_IOCTL_SERIAL_CLEAR_STATS   (0x0106)
#define IO_IOCTL_SERIAL_TRANSMIT      (0x0107)
#define IO_IOCTL_SERIAL_GET_HW_SIGNAL (0x0108)
#define IO_IOCTL_SERIAL_SET_HW_SIGNAL (0x0109)
#define IO_IOCTL_SERIAL_CLR_HW_SIGNAL (0x010A)
#define IO_IOCTL_SERIAL_SET_DATA_BITS (0x010C)
#define IO_IOCTL_SERIAL_GET_DATA_BITS (0x010D)
#define IO_IOCTL_SERIAL_SET_PARITY    (0x0110)
#define IO_IOCTL_SERIAL_GET_PARITY    (0x0111)

/* flags for serial io */
#define IO_SERIAL_RAW_IO          (0)
#define IO_SERIAL_XON_XOFF        (1) /* Perform xon/xoff processing                      */
#define IO_SERIAL_TRANSLATION     (2) /* Perform translation :     outgoing \n to CR\LF   */
                                      /* incoming backspace erases previous character     */
#define IO_SERIAL_ECHO            (4) /* echo incoming characters                         */
#define IO_SERIAL_HW_FLOW_CONTROL (8) /* RTS/CTS hardware handshaking                     */

/* 
 * standard I/O streams (stdio_types)
 */
#define IO_STDIN       (0L) /* current task stream indicators          */
#define IO_STDOUT      (1L)
#define IO_STDERR      (2L)
#define IO_PROC_STDIN  (3L) /* default stream indicators for new tasks */
#define IO_PROC_STDOUT (4L)
#define IO_PROC_STDERR (5L)

#if defined __TI_COMPILER_VERSION__

/* functions to get and set the handle for above stdio_types */
void *io_get_handle(U32 stdio_type);                    /* returns current_handle */
void *io_set_handle(U32 stdio_type, void * new_handle); /* returns old_handle     */

/*
** FILE STRUCTURE
**
** This structure defines the information kept in order to implement
** ANSI 'C' standard I/O stream.
*/
#ifdef _STDIO
#define FILE FILEX
#endif

typedef struct file_struct
{
  void       *DEV_PTR;              /* The address of the Device for this stream    */
  void       *DEV_DATA_PTR;         /* Device Driver specific information           */
  uint_32     FLAGS;                /* General control flags for this stream        */
  uint_32     ERROR;                /* The current error for this stream            */
  uint_32     LOCATION;             /* The current position in the stream           */
  uint_32     SIZE;                 /* The current size of the file                 */
  /*  BOOLEAN */
  int         HAVE_UNGOT_CHARACTER; /* used to implement undelete                   */
  int_32      UNGOT_CHARACTER;      /* used to implement undelete                   */

} FILE, * FILE_PTR;

typedef struct vc_readdir_struct               /* ME                                       */
{
  unsigned             MAXNUM;                 /* the maximum number of directory items    */
  char *               FPATH;                  /* the file path                            */
  struct vc_dirent  *  DIRENT;                 /* pointer to structure for the directory   */
} VC_READDIR_STRUCT, *  VC_READDIR_STRUCT_PTR;

#define FS_MAXPATH  80                         /* MAXPATH incl. final '\0'                 */

typedef struct vc_dirent                       /* ME                                       */
{
  char                     D_NAME[FS_MAXPATH]; /* the file name string including full path */
  unsigned                 LENGTH;             /* file size                                */
  long                     CR_DATE;            /* date of creation                         */
  long                     WR_DATE;            /* date last modified                       */
  int                      ATTR;               /* since VCRT 5.18 file attributes          */
  int                      LINK_COUNT;
  int                      DUMMY1;
  int                      DUMMY2;
} VC_DIRENT_STRUCT, * VC_DIRENT_STRUCT_PTR;

#define MAX_FILES   (1024)

#undef  stdin /* undefine stdio.h settings */
#undef  stdout
#undef  stderr

#define stdin  (FILE *)io_get_handle(IO_STDIN)
#define stdout (FILE *)io_get_handle(IO_STDOUT)
#define stderr (FILE *)io_get_handle(IO_STDERR)

#define read(a,b,c)  io_read(a,b,c)
#define write(a,b,c) io_write(a,b,c)
#define fopen(a,b)   io_fopen(a,b)
#define fclose(a)    io_fclose(a)

int sscanf(const char *_str, const char *_fmt, ...); /* from TI rtsXXXX.lib */

/* pipe device */

int_32 io_pipe_install(char *name, uint_32 size);

#define FIFO 0
#define CIRC 1

#define PIPE_READ   1
#define PIPE_WRITE  2
#define PIPE_MODE   8


/* file functions */

FILE *io_fopen(char *name, char *flags);
int io_fgetc(FILE *fp);
int io_fungetc(int c,FILE *fp);
int io_fputc(int c, FILE *fp);
int io_read(FILE *fp, char *dp, int count);
int io_write(FILE *fp, char *dp, int count);
int io_fclose(FILE *fp);
int io_ioctl(FILE *fp, unsigned cmd, unsigned *param);
int io_ftell(FILE *fp);
int io_fseek(FILE *fp, int offset, unsigned whence);
int io_fprintf(FILE *fp, char *pfmt, ...);
int io_fscanf(FILE *fp, char *pfmt, ...);
U32 io_fstatus (FILE *fp);
U32 io_status  (void);

int loadf(FILE *fp, int *stksiz);

#endif  /* __TI_COMPILER_VERSION__  */

int _FsFnSplit(const char *pathP, char *driveP, char *dirP, char *nameP, char *extP);
void _FsFnMerge(char *pathP, const char *driveP, const char *dirP, const char *nameP, const char *extP);

#define VCRT_OK      (0)
#define VCRT_ERROR  (-1)


/* TCP/IP ***********************************************************************************************/

#if defined __TI_COMPILER_VERSION__

/*
** Special IP addresses
*/
#define INADDR_ANY               0
#define INADDR_LOOPBACK          0x7F000001L
#define INADDR_ALLHOSTS_GROUP    0xE0000001L
#define INADDR_ALLROUTERS_GROUP  0xE0000002L
#define INADDR_RIP_GROUP         0xE0000009L
#define INADDR_NTP_GROUP         0xE0000101L
#define INADDR_BROADCAST         0xFFFFFFFFL

/*
** Successful return code.  No errors.
*/
#define VCRT_OK                   (0)
#define VCRT_ERROR               (-1)
#define VCRT_HANDLE_ERROR        (0xffffffffL)
#define VCRT_SOCKET_ERROR        (0xffffffffL)

/*
** Generic Error codes
*/
#define VCRTERR_SEND_FAILED             (0x1102)
#define VCRTERR_CREATE_POOL_FAILED      (0x1105)
#define VCRTERR_OPEN_QUEUE_FAILED       (0x1106)
#define VCRTERR_CREATE_FAILED           (0x1107)
#define VCRTERR_RECEIVE_FAILED          (0x1108)
#define VCRTERR_DEALLOC_FAILED          (0x1182)
#define VCRTERR_OUT_OF_MEMORY           (0x102)
#define VCRTERR_TIMEOUT                 (0x103)
#define VCRTERR_INVALID_ADDRESS         (0x104)
#define VCRTERR_INVALID_PARAMETER       (0x105)
#define VCRTERR_READ_ABORTED            (0x11f)
#define VCRTERR_OUT_OF_BUFFERS          (0x120)
#define VCRTERR_CREATE_PARTITION_FAILED (0x121)
#define VCRTERR_OUT_OF_SOCKETS          (0x122)
#define VCRTERR_FOPEN_FAILED            (0x123)

/*
** Protocol and Address families
*/
#define PF_INET         1
#define AF_UNSPEC       0
#define AF_INET         1

/*
** Validity check for socket structures
*/
#define SOCKET_VALID       0x52544353

#define VCRT_SO_IGMP_ADD_MEMBERSHIP (0)

/* 
 * Socket options
 */
#define OPT_CONNECT_TIMEOUT            (0x01)
#define OPT_RETRANSMISSION_TIMEOUT     (0x02)
#define OPT_SEND_TIMEOUT               (0x03)
#define OPT_RECEIVE_TIMEOUT            (0x04)
#define OPT_RECEIVE_PUSH               (0x05)
#define OPT_SEND_NOWAIT                (0x06)
#define OPT_SEND_PUSH                  (0x08)
#define OPT_RECEIVE_NOWAIT             (0x09)
#define OPT_CHECKSUM_BYPASS            (0x0A)
#define OPT_ACTIVE                     (0x0B)
#define OPT_TBSIZE                     (0x0C)
#define OPT_RBSIZE                     (0x0D)
#define OPT_MAXRTO                     (0x0E)
#define OPT_MAXRCV_WND                 (0x0F)
#define OPT_KEEPALIVE                  (0x10)
#define OPT_NO_NAGLE_ALGORITHM         (0x12)
#define OPT_NOSWRBUF                   (0x13)
#define OPT_SOCKET_TYPE                (0x15)
#define OPT_CALLBACK                   (0x16)
#define OPT_TIMEWAIT_TIMEOUT           (0x17)
#define OPT_SOCKET_ERROR               (0x19) /* since VCRT 5.26 */

/*
 * Flags for send[to]/recv[from]
 */
#define VCRT_MSG_O_NONBLOCK   0x0001
#define VCRT_MSG_S_NONBLOCK   0x0002
#define VCRT_MSG_BLOCK        (VCRT_MSG_O_NONBLOCK)
#define VCRT_MSG_NONBLOCK     (VCRT_MSG_O_NONBLOCK | VCRT_MSG_S_NONBLOCK)
#define VCRT_MSG_O_NOCHKSUM   0x0004
#define VCRT_MSG_S_NOCHKSUM   0x0008
#define VCRT_MSG_CHKSUM       (VCRT_MSG_O_NOCHKSUM)
#define VCRT_MSG_NOCHKSUM     (VCRT_MSG_O_NOCHKSUM | VCRT_MSG_S_NOCHKSUM)
#define VCRT_MSG_PEEK         0x0010
#define VCRT_MSG_NOLOOP       0x0020

/*
 * VCRT_level option levels 
 */
#define SOL_IGMP    0
#define SOL_LINK    1
#define SOL_SOCKET  2
#define SOL_TCP     3
#define SOL_UDP     4

/*
 * Socket error codes
 */
#define VCRTERR_SOCK_INVALID        0x704
#define VCRTERR_SOCK_INVALID_AF     0x702
#define VCRTERR_SOCK_SHORT_ADDRESS  0x706
#define VCRTERR_SOCK_NOT_SUPPORTED  0x719
#define VCRTERR_SOCK_INVALID_OPTION 0x71A
#define VCRTERR_SOCK_SHORT_OPTION   0x71B
#define VCRTERR_SOCK_NOT_BOUND      0x709
#define VCRTERR_SOCK_IS_BOUND       0x705
#define VCRTERR_SOCK_NOT_LISTENING  0x70C
#define VCRTERR_SOCK_IS_LISTENING   0x70B
#define VCRTERR_SOCK_NOT_CONNECTED  0x70D
#define VCRTERR_SOCK_IS_CONNECTED   0x70A
#define VCRTERR_SOCK_NOT_OWNER      0x70E

/*
 * ICMP error codes
 */
#define VCRTERR_ICMP_ECHO_TIMEOUT   0x580 /* timed out waiting for echo reply */
#define VCRTERR_ICMP_BAD_HEADER     0x581 /* error in header                  */
#define VCRTERR_ICMP_BAD_CHECKSUM   0x582 /* invalid checksum                 */

/*
 * UDP error codes
 */
#define VCRTERR_UDP_UCB_ALLOC    0x660    /* couldn't allocate UCB */
#define VCRTERR_UDP_UCB_FREE     0x661    /* couldn't free UCB     */
#define VCRTERR_UDP_UCB_CLOSE    0x662    /* UCB not open          */
#define VCRTERR_UDP_PORT_OPEN    0x663    /* port already open     */
#define VCRTERR_UDP_PORT_ALLOC   0x664    /* no more ports         */
#define VCRTERR_UDP_PORT_ZERO    0x665    /* can't send to port 0  */
#define VCRTERR_UDP_BAD_HEADER   0x666    /* error in header       */
#define VCRTERR_UDP_BAD_CHECKSUM 0x667    /* invalid checksum      */

/*
 * TCP error codes
 */
#define VCRTERR_TCP_OPEN_FAILED         (0x630)  /* TcpOpen failed              */ 
#define VCRTERR_TCP_INVALID_OPTION      (0x631)  /* Option was invalid          */ 
#define VCRTERR_TCP_IN_PROGRESS         (0x632)  /* Operation already in        */ 
                                                 /*    progress                 */ 
#define VCRTERR_TCP_ADDR_IN_USE         (0x633)  /* Address already in use      */ 
#define VCRTERR_TCP_ADDR_NA             (0x634)  /* Can't assign requested      */ 
                                                 /*    address                  */ 
#define VCRTERR_TCP_CONN_ABORTED        (0x635)  /* Software caused connection  */ 
                                                 /*    abort                    */ 
#define VCRTERR_TCP_CONN_RESET          (0x636)  /* Connection reset by peer    */ 
#define VCRTERR_TCP_HOST_DOWN           (0x637)  /* Host is down                */ 
#define VCRTERR_TCP_CONN_CLOSING        (0x638)  /* Connection closing          */ 
#define VCRTERR_TCP_CONN_RLSD           (0x639)  /* Connection/TCB released     */ 
#define VCRTERR_TCP_MISSING_OPEN        (0x63A)  /* TCB exists in LISTEN state  */ 
                                                 /*    with no associated Listn */ 
                                                 /*    request from upper layer */ 
#define VCRTERR_TCP_CTR_ZERO_RSIZE      (0x63B)  /* TCP_Copy_to_ring got 0      */ 
                                                 /*    ring buf size            */ 
#define VCRTERR_TCP_SP_BAD_SEND_STATE   (0x63C)  /* Attempted to TCP_Send_packet*/ 
                                                 /*    in invalid state         */ 
#define VCRTERR_TCP_SP_OUT_OF_PCBS      (0x63D)  /* Could not get a PCB in      */ 
                                                 /*    TCP_Send_packet          */ 
#define VCRTERR_TCP_SRR_OUT_OF_PCBS     (0x63E)  /* Could not get a PCB in      */ 
                                                 /*    TCP_Send_reply_reset     */ 
#define VCRTERR_TCP_SR_OUT_OF_PCBS      (0x63F)  /* Could not get a PCB in      */ 
                                                 /*    TCP_Send_reset           */ 
#define VCRTERR_TCP_SHRINKER_HOST       (0x640)  /* TCP detected a 'shrinker'   */ 
                                                 /*    peer host                */ 
#define VCRTERR_TCP_PA_BUFF_CORRUPT     (0x642)  /* TCP send buffer corruption  */ 
                                                 /*    detected in TCP_Process_ack */ 
#define VCRTERR_TCP_PS_FAILED_GET_SBUF  (0x643)  /* Could not get an SbufNode   */ 
                                                 /*    in TCP_Process_send      */ 
#define VCRTERR_TCP_PR_OUT_OF_MEM       (0x644)  /* Could not get a Rcvchunk    */ 
                                                 /*    buffer in Setup Receive  */ 
#define VCRTERR_TCP_PP_OUT_OF_MEM       (0x645)  /* Could not add a host IpList */ 
                                                 /*    node in TCP_Process_packet */ 
#define VCRTERR_TCP_TIMER_FAILURE       (0x646)  /* Could not start a TCP timer */ 
#define VCRTERR_TCP_NOT_CONN            (0x647)  /* Socket is not connected     */ 
#define VCRTERR_TCP_SHUTDOWN            (0x648)  /* Can't send after socket     */ 
                                                 /*    shutdown                 */ 
#define VCRTERR_TCP_TIMED_OUT           (0x649)  /* Connection timed out        */ 
#define VCRTERR_TCP_CONN_REFUSED        (0x64A)  /* Connection refused          */ 
#define VCRTERR_TCP_NO_MORE_PORTS       (0x64B)  /* No more ports available     */ 
                                                 /*    for connections          */ 
#define VCRTERR_TCP_BAD_STATE_FOR_CLOSE (0x64C)  /* Attempted to call           */ 
                                                 /*    TCP_Process_effective_close or */ 
                                                 /*    TCP_Process_close in invalid  */ 
                                                 /*    state                    */ 
#define VCRTERR_TCP_BAD_STATE_FOR_REL   (0x64D)  /* Attempted to release a TCB  */ 
                                                 /*    in an invalid state      */ 
#define VCRTERR_TCP_REXMIT_PROBLEM      (0x64E)  /* A retransmission timer      */ 
                                                 /*    timed out while in IDLE  */ 
                                                 /*    mode (i.e. invalid)      */ 
#define VCRTERR_TCP_URGENT_DATA_PROB    (0x64F)  /* Urgent data pointer         */ 
                                                 /*    was corrupted in the tcb */ 
#define VCRTERR_TCP_DEALLOC_FAILED      (0x650)  /* A call to release memory failed */ 
#define VCRTERR_TCP_HOST_UNREACH        (0x651)  /* No route to host            */ 
#define VCRTERR_TCP_BAD_HEADER          (0x652)  /* error in header             */ 
#define VCRTERR_TCP_BAD_CHECKSUM        (0x653)  /* invalid checksum            */ 

/*
** Close methods for shutdown()
*/

#define FLAG_ABORT_CONNECTION       (0x0010)
#define FLAG_CLOSE_TX               (0x0001)

/*
** IP address definition
*/
typedef uint_32 ip_address;

typedef struct in_addr {
   ip_address s_addr;
} in_addr;

/*
** IP multicast group
*/
typedef struct ip_mreq {
   in_addr  imr_multiaddr;
   in_addr  imr_interface;
} ip_mreq;


/*
** Socket Address Structure
*/
typedef struct sockaddr_in {
   uint_16  sin_family;
   uint_16  sin_port;
   in_addr  sin_addr;
} sockaddr_in;

/* 
 * DNS resolver 
 */
typedef struct hostent
{
  char *            h_name;
  char **           h_aliases;
  int_16            h_addrtype;
  int_16            h_length;
  char **           h_addr_list;

} HOSTENT_STRUCT, *  HOSTENT_STRUCT_PTR;

/* prototypes for socket interface  */

uint_32 accept(uint_32 socket, sockaddr_in * peeraddr, uint_16 * addrlen);
uint_32 bind(uint_32 socket, sockaddr_in * localaddr, uint_16 addrlen);
uint_32 connect(uint_32 socket, sockaddr_in * destaddr, uint_16 addrlen);
uint_32 getpeername(uint_32 socket, sockaddr_in * name, uint_16 * namelen);
uint_32 getsockname(uint_32 socket, sockaddr_in * name, uint_16 * namelen);
uint_32 getsockopt(uint_32 socket, uint_32 VCRT_level, uint_32 optname, pointer optval, uint_32 * optlen);
uint_32 listen(uint_32 socket, uint_16 backlog);
int_32  recvfrom(uint_32 socket, char * buffer, uint_32 buflen, uint_32 flags, 
            sockaddr_in * fromaddr, uint_16 * fromlen);
int_32  recv(uint_32 socket, char * buffer, uint_32 buflen, uint_32 flags);
int_32  sendto(uint_32 socket, char * buffer, uint_32 buflen, uint_16 flags,
            sockaddr_in * destaddr, uint_16 addrlen);
int_32  send(uint_32 socket, char * buffer, uint_32 buflen, uint_32 flags);
uint_32 setsockopt(uint_32 socket, uint_32 VCRT_level, uint_32 optname, pointer optval, uint_32 optlen);
uint_32 socket_dgram(void);
uint_32 socket_stream(void);
uint_32 shutdown (uint_32 socket, uint_32 method);

uint_32 VCRT_ping(ip_address address, uint_32 *timeout, uint_16 id);
uint_32 VCRT_geterror(uint_32 socket);
uint_32 VCRT_selectall(uint_32 timeout);
uint_32 VCRT_selectset(pointer socketset, uint_32 count, uint_32 timeout);

/* new for VCRT 5.25 */
int_32  VCRT_inet_aton(const char *cp, struct in_addr *inp); /* convert dotted IP string in in_addr */
char *  VCRT_inet_ntoa(struct in_addr in);                   /* convert in_addr in dottet IP string */

/* new for VCRT 5.26 */
HOSTENT_STRUCT_PTR gethostbyname(char *);                                     /* resolve host ip by name */
HOSTENT_STRUCT_PTR gethostbyaddr(char * addr_ptr, uint_32 len, uint_32 type); /* resolve host name by ip */

/*
 * Flags for the FTP client - see FTP_command_data()
 */
#define FTPMODE_DEFAULT 1
#define FTPMODE_PORT    2
#define FTPMODE_PASV    3
#define FTPDIR_RECV     0
#define FTPDIR_SEND     4
#define FTPDIRECT_MEM   8

typedef struct {

  char *  buffer; /* buffer address to read/write */
  uint_32 length; /* memory length in bytes       */
  uint_32 dummy1;
  uint_32 dummy2;

} FTP_DIRECT_STRUCT, * FTP_DIRECT_STRUCT_PTR;

/* new for VCRT 5.28 */
int_32  FTP_open           (pointer * handle, ip_address ip , FILE_PTR ctrl_fd);
int_32  FTP_command        (pointer   handle, char * command, FILE_PTR ctrl_fd);
int_32  FTP_command_data   (pointer   handle, char * command, FILE_PTR ctrl_fd, FILE_PTR data_fd, uint_32 flags);
int_32  FTP_close          (pointer   handle, FILE_PTR ctrl_fd);
int_32  FTP_receive_message(pointer   handle, FILE_PTR ctrl_fd);

#endif  /*  __TI_COMPILER_VERSION__ */

/* end TCP/IP *******************************************************************************************/

/* tasks */

int  task_del(int id);                /* kill task with ID <id>         */
int  task_get_next_id(int id);        /* get next ID in process table   */
int  task_get_id(void);               /* get own task ID                */
char *task_get_name_from_id(int id);  /* get name of task with ID <id>  */
int  task_get_priority(int id,int *prio); /* get priority of task       */
int  task_set_priority(int id,int new_prio,int *old_prio); /* set prio  */
void time_delay(int ms);              /* task waits <ms> milliseconds   */

int  exec (char *fname,...);          /* execute program                */
int  exec1(char *fname, void *arg);   /* deprecated use exec2() instead */
int  exec2(char *fname, ...);         /* execute program as a new task  */
int  exec3(unsigned int priority,     /* set priority of task at start  */
           unsigned int timeslice,    /* set timeslice of task at start */
           char *fname, ...);         /* execute program as a new task  */

/* new for VCRT 5.25 */
int  sysmemfree();                    /* returns free memory in bytes   */

/* events */

void event_connect_to_task(void);
void event_disconnect(void);
int  set_evt(int id);
int  wait(int id, int timeout);

/* internal events      */

#define     TIMER           0
#define     MM_CARD         1
#define     MM_CARD_RW      2
#define     MM_CARD_DMA     3
#define     IMAGE_READY     4
#define     EXP_READY       5
/* new for VCRT 5.18 */
#define     DHCP_READY      6
#define     TRIG_READY      7
#define     PLC_INT         8
/* new for VCRT 5.23 */
#define     I2C_INT         9
/* new for VCRT 5.25 */
#define     TIMER2          10
#define     DISPLAY_EVT     11
#define     IO_EVT          12
#define     LB_INT          13
#define     COMP1_INT       14
#define     COMP2_INT       15
#define     COMP3_INT       16
#define     SEN_ERR_INT     17

/* new for VCRT 5.26 */
#define VC_MAX_EVENTS       (64) 

/* new for VCRT 5.23: timestamps on exposed images table  
 *
 * imgts ts_table[IMGTS_SIZE]; 
 * ts_table = getvar(TIMESTAMP); 
 *
 */

#define IMGTS_SIZE  20  /* number of timestamp table entries */

/* structure of timestamp entry */ 
typedef struct
{
  U64 exptimestamp; /* time stamp of image  */
  I32 imageno;      /* image number         */ 
} imgts;


/* display modes for color camera   */

#define DISP_NO_COLOR   (-1)    /* value for non color cameras                          */
#define DISP_IDLE       (0)     /* no display update, no overhead                       */
#define DISP_GREY       (1)     /* display of grey images, like black-and-white camera  */
#define DISP_RGB        (2)     /* display of RGB images, image var type = IMAGE_RGB    */
#define DISP_BAYER      (3)     /* display of Bayer images, image var type = IMAGE_BAYER*/
#define DISP_BAYERGREY  (4)     /* same as before, image is shown as grey image         */
#define DISP_YCBCR      (5)     /* display of YCbCr images, var type = IMAGE_CBCR444    */

#define DISP_MODE_MAX   (5)    /* highest display mode                                  */


/* sync functions for unbuffered display of VC4XXX cameras   */ 

void display_update(int type); /* update display immediately */
void wait_vsync();             /* wait on next vsync         */

/* display update types */

#define UPDATE_DISP      (0)
#define UPDATE_OVLY      (1)
#define UPDATE_DISP_OVLY (2)

/*
 * new in 5.30.7  debounce function for VC4XXX cameras                           
 * plc_port:      range 0..3
 * debounce_time: range could be N * 40.960 microseconds (N=1..256) on VC40XX                                  
 * debounce_time: range could be N * 32.768 microseconds (N=1..256) on VC44XX                                  
 * return value:  actually set time value in microseconds 
 *                -1 if error or not supported                     
 * */
int debounce(unsigned int plc_port, unsigned int debounce_time_in_micro_seconds);

/* new in 5.28.8 IO PORT functions VC64XX */
int io_port_init(void);         /* intialize io ports to all input               */ 
int io_port_in(void);           /* returns result - lowest 8 bits - 0=LOW 1=HIGH */ 
int io_port_dir(int direction); /* returns result - lowest 8 bits - 0=INP 1=OUT  */
int io_port_out(int out);       /*                - lowest 8 bits - 0=LOW 1=HIGH */ 

/* new in 5.28.9 IO PORT functions VC64XX */
int io_port_int_enable();       /* IOPORT interrupt enable                       */
int io_port_int_disable();      /* IOPORT interrupt disable                      */

/* new in 5.29.7 sysvar VC_HW_TYPE */
int get_VC_HW_TYPE(void);

/* VC_HW_TYPE platforms */
#define VC_DM6435_4_CPU 0x7400
#define VC_DM6435_3_CPU 0x7300
#define VC_DM6435_2_CPU 0x7200
#define VC_DM6431_2_CPU 0x8200
#define VC_DM6431_CPU   0x8000
#define VC_DM6435_CPU   0x7000
#define VC_DM642_CPU    0x6000
#define VC_DM648_CPU    0x5000
#define VC_C6414_CPU    0x4000
#define VC_C6410_CPU    0x3000
#define VC_DM640_CPU    0x2000
#define VC_C6211_CPU    0x1000

/* new in 5.30.4 set sensor ROI - currently only VC6010/6210/7210 */

/* hpos,vpos:   position of ROI
 * hsize,vsize: width and height of ROI
 * shutter:     shutter value in microseconds
 * mode:        0=b/w 1=color
 * the variables pointed to will be filled with the real settings in sensor, which
 * could be different due to sensor restrictions !!! 
 * * */
int vcrt_set_image_format(int hpos, int vpos, int hsize, int vsize,
                          long shutter, int mode,
                          int * sen_hpos_p, int * sen_vpos_p, int * sen_hsize_p, int * sen_vsize_p,
                          long * sen_shutter_p);

/* new in 5.30.8 get serial numver of camera */
int VcrtlibGetCameraID (int *CameraID);

/*
 * new for VCRT 5.25:   
 *
 * VC4x Encoder Macros (Revision 21.06.2006)
 *
 ************** IMPORTANT!!! ********************************************************
 * Encoder Macros do require VCRT Version > 5.24.7
 * AND PWR PCB Hardware Revision 1.4
 * AND FPGA file dated after 02.04.2006 !!!
 ************** IMPORTANT!!! ********************************************************/

/* bits */ 

/* Ctrl_reg */
#define USE_ENCODER          (uint_32)(1<<24)

/* CAM_enc_ctrl write only */
#define ENC_WR_TO_RELOAD     (uint_32)(1<<31) /* write to comparator                */
#define ENC_WR_TO_CNT        (uint_32)(1<<30) /* write to counter                   */
#define ENC_SET_N_MODE       (uint_32)(1<<29) /* set N mode                         */
#define ENC_RESET_N_MODE     (uint_32)(1<<28) /* reset N mode                       */
#define ENC_CLR_TIL_N_ONCE   (uint_32)(1<<27) /* clear counter immediately until N pulse for one time */
#define ENC_RESET_ENC        (uint_32)(1<<26) /* reset encoder                      */
#define ENC_DIS_ABN          (uint_32)(1<<25) /* disable inputs ABN                 */
#define ENC_ENA_ABN          (uint_32)(1<<24) /* enable  inputs ABN                 */
                                              /* bits 23:0 counter value            */
                                              
#define ENC_CNT_MASK         (uint_32)(0xFFFFFF) /* counter or comparator bits 23:0 */

/* CAM_enc_status read only */
#define ENC_ENABLED          (uint_32)(1<<31) /* 0=RS232 enabled 1=encoder in use   */
#define ENC_CNT_CLEAR        (uint_32)(1<<30) /* 0=nowait        1=wait for N pulse */
#define ENC_N_MODE           (uint_32)(1<<29) /* 0=N mode off    1=N mode on        */
#define ENC_LOCKED           (uint_32)(1<<28) /* 0=unlocked      1=locked           */
#define ENC_ABN_ENABLED      (uint_32)(1<<27) /* 0=ABN disabled  1=ABN enabled      */
                                              /* bits 26:24 not used                */
                                              /* bits 23:0 counter value            */
/* helpers */
#define set_enc_reg(x)       (*(volatile uint_32 *)VIRTX_TDIV = x)
#define get_cnt_reg()        (*(volatile uint_32 *)VIRTX_TCNT) 

#define ENC_GP10_0()         {  disints(); /* ATTENTION order of register setting is IMPORTANT! */ \
                                *((volatile int *)(GPDIR)) |=  (1<<10); /* set GP10 as output */ \
                                *((volatile int *)(GPVAL)) &= ~(1<<10); /* set GP10 = 0 */ \
                                *((volatile int *)(GPEN))  &= ~(1<<10); /* dis GP10 = 0 */ \
                                set_ctrl_reg( USE_ENCODER ); \
                                enaints(); }

#define ENC_GP10_1()         {  disints(); /* ATTENTION order of register setting is IMPORTANT! */ \
                                *((volatile int *)(GPDIR)) |=  (1<<10); /* set GP10 as output */ \
                                *((volatile int *)(GPVAL)) |=  (1<<10); /* set GP10 = 1 */ \
                                *((volatile int *)(GPEN))  |=  (1<<10); /* ena GP10 = 1 */ \
                                res_ctrl_reg( USE_ENCODER ); \
                                enaints(); }
/* functions */

#define ENCODER (0)
#define SERIAL  (1)
#define INTERFACE_MODE(x)     { switch(x) \
                                { case SERIAL:  ENC_GP10_1(); break; \
                                  case ENCODER: ENC_GP10_0(); break; } \
                              }

/* set */
#define ENC_DISABLE_CNT()     set_enc_reg( ENC_DIS_ABN )  
#define ENC_ENABLE_CNT()      set_enc_reg( ENC_ENA_ABN )  
#define ENC_WAIT_N()          set_enc_reg( ENC_CLR_TIL_N_ONCE )
#define ENC_WAIT_N_TRIG()     set_enc_reg( ENC_SET_N_MODE )
#define ENC_NOWAIT_N_TRIG()   set_enc_reg( ENC_RESET_N_MODE )
#define ENC_RESET()           set_enc_reg( ENC_RESET_ENC )

/* write */
#define ENC_WRITE_CNT(x)      set_enc_reg( ENC_WR_TO_CNT    | (x & ENC_CNT_MASK) )
#define ENC_WRITE_RELOAD(x)   set_enc_reg( ENC_WR_TO_RELOAD | (x & ENC_CNT_MASK) )

/* read  */
#define ENC_READ_CNT()            (get_cnt_reg() & ENC_CNT_MASK)
#define ENC_WAITING_FOR_N_PULSE() (get_cnt_reg() & ENC_CNT_CLEAR)


/******************************************************************************/
/*                                                                            */
/*  MACRO FUNCTIONS for REGISTERS:                                            */
/*     REG_READ          - Read register at specified address                 */
/*     REG_WRITE         - Write to register at specified address             */
/*     RESET_BIT         - Clears bit in register.                            */
/*     GET_BIT           - Returns bit value in register.                     */
/*     SET_BIT           - Sets bit in register.                              */
/*     MASK_BIT          - Create (1's) mask for specified bit.               */
/*     ASSIGN_BIT_VAL    - Assign bit to specified value                      */
/*                                                                            */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* MACRO FUNCTIONS                                                            */
/*----------------------------------------------------------------------------*/

#define CONTENTS_OF(addr) \
        (*((volatile unsigned int *)(addr)))

#define REG_READ(addr) \
        (CONTENTS_OF(addr))

#define REG_WRITE(addr,val) \
        (CONTENTS_OF(addr) = (val))

#define MASK_BIT(bit) \
        (1 << (bit))

#define RESET_BIT(addr,bit) \
        (CONTENTS_OF(addr) &= (~MASK_BIT(bit)))

#define GET_BIT(addr,bit) \
        ((CONTENTS_OF(addr) & MASK_BIT(bit)) ? 1 : 0)

#define SET_BIT(addr,bit) \
        (CONTENTS_OF(addr) = (CONTENTS_OF(addr)) | (MASK_BIT(bit)))

#define ASSIGN_BIT_VAL(addr,bit,val) \
        ( (val) ? SET_BIT(addr,bit) : RESET_BIT(addr,bit) )


/******************************************************************************/
/*                                                                            */
/*  MACRO FUNCTIONS for TIMER2:                                               */
/*    TIMER2_RESET()                                                          */
/*    TIMER2_INIT(ctrl,per,cnt)                                               */
/*    TIMER2_START()                                                          */
/*    TIMER2_STOP()                                                           */
/*                                                                            */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* DEFINES AND MACROS                                                         */
/*----------------------------------------------------------------------------*/

/**************************** TIMER REGISTERS *********************************/
#define TIMER2_CTRL_ADDR      0x01AC0000
#define TIMER2_PERIOD_ADDR    0x01AC0004
#define TIMER2_COUNTER_ADDR   0x01AC0008

#define TIMER2_CTRL     *(volatile unsigned int *)(TIMER2_CTRL_ADDR)
#define TIMER2_PERIOD   *(volatile unsigned int *)(TIMER2_PERIOD_ADDR)
#define TIMER2_COUNTER  *(volatile unsigned int *)(TIMER2_COUNTER_ADDR)

/* Timer Control Register Bitfield */
#define GO       6
#define HLD      7


/*----------------------------------------------------------------------------*/
/* TIMER2_RESET - reset timer to conditions defined by device reset           */
/*----------------------------------------------------------------------------*/
#define TIMER2_RESET() \
        { (*(unsigned int *)(TIMER2_CTRL_ADDR) = 0);       \
          (*(unsigned int *)(TIMER2_PERIOD_ADDR) = 0);     \
          (*(unsigned int *)(TIMER2_COUNTER_ADDR) = 0);    \
        }

/*----------------------------------------------------------------------------*/
/* TIMER2_INIT - initialize timer registers                                   */
/*----------------------------------------------------------------------------*/
#define TIMER2_INIT(ctrl,per,cnt) \
        { (*(unsigned int *)(TIMER2_CTRL_ADDR)    = ctrl); \
          (*(unsigned int *)(TIMER2_PERIOD_ADDR)  = per);  \
          (*(unsigned int *)(TIMER2_COUNTER_ADDR) = cnt);  \
        }

/*----------------------------------------------------------------------------*/
/* TIMER2_START - Sets both GO and HOLD bits in Timer Control Register which  */
/*                resets the Timer Counter Register and enables counting on   */
/*                on the next clock.  (GO bit autoclears)                     */
/*----------------------------------------------------------------------------*/
#define TIMER2_START() \
        REG_WRITE(TIMER2_CTRL_ADDR,(REG_READ(TIMER2_CTRL_ADDR) | 0xc0))

/*----------------------------------------------------------------------------*/
/* TIMER2_STOP - Asserts (clears) the HOLD bit in the Timer Control Register  */
/*----------------------------------------------------------------------------*/
#define TIMER2_STOP() \
        RESET_BIT(TIMER2_CTRL_ADDR,HLD)

#define T2CTRL    (0x200)
#define T2CLK     ((getvar(CPUCLK))/8)
#define T2MICROS  ((U32)((U40)T2CLK/(U40)1000000)) 
#define T2MILLIS  ((U32)((U40)T2CLK/(U40)1000)) 
#define T2SECONDS ((U32)((U40)T2CLK/(U40)1))

/* misc functions for VC4012E i2c bus, VC4008E binning, etc */

/* VC4012E/VC6210 i2c  -  edma read/write error return values  -        */
#define ERR_I2CBB      (-1)   /* i2c bus busy error condition           */
#define ERR_I2CPAR     (-2)   /* i2c parameter error                    */
#define ERR_I2CSND     (-3)   /* i2c send error                         */
#define ERR_I2CNOIN    (-4)   /* i2c no init done                       */
#define ERR_I2CTIMEOUT (-5)   /* i2c timeout                            */

I32 i2c_edma_write(I32 count, I32 i2c_addr, char *i2c_sendbuf);
I32 i2c_edma_read(I32 count, I32 i2c_addr, char *i2c_rcvbuf);

I32 set_binning(int on);        /* VC4008 binning, returns 0 on success */

/* SPI WORD LENGTH - used for reg_width */
#define WDLEN_8BIT  0
#define WDLEN_12BIT 1
#define WDLEN_16BIT 2
#define WDLEN_20BIT 3
#define WDLEN_24BIT 4
#define WDLEN_32BIT 5

I32 spi_edma_write(U32 reg_addr, U32 *spi_sendbuf, U32 reg_width, U32 count);
I32 spi_edma_read (U32 reg_addr, U32 *spi_readbuf, U32 reg_width, U32 count);

/* NEW since VCRT 5.30.8 light weight semaphores */
U32 lwsem_msize();
U32 lwsem_create ( void * lwsem, I32 initial_number );
U32 lwsem_destroy ( void * lwsem );
U32 lwsem_post ( void * lwsem );
U32 lwsem_wait ( void * lwsem );
U32 lwsem_test ( void** lwsem_error, void** td_error);

#ifdef __cplusplus
}
#endif

/*----------------------------------------------------------------------*/
/**
*       BK9000 library header (Bus coupler for Ethernet)
* 
*       type definitions, macros and function
*       prototypes for the Beckhoff I/O box library.
*
*/
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*                           Macros and typedefs                        */
/*----------------------------------------------------------------------*/

#define BK_VERS      100    /* library version (100 for 1.00)           */
#define BK_VERS_STR "1.00"  /* library version string                   */

/*............. Bus terminal (module) types */
typedef enum
{
    BK_WORD_OUT=0,  /* word-oriented output terminal (16-bit words)     */
    BK_WORD_IN,     /* word-oriented input terminal (16-bit words)      */
    BK_BIT_OUT,     /* bit-oriented output terminal (1 bit/chan)        */
    BK_BIT_IN,      /* bit-oriented input terminal (1 bit/chan)         */
    BK_MODTYPE_CNT  /* # of module (terminal) types                     */
} BK_MOD_TYPES;

/*............. BL_LIB handle */
typedef void * BK_HND;

/*............. Transfer modes */
#define BK_WRITE_MODE 0x0001    /* write data to output modules         */
#define BK_READ_MODE  0x0002    /* read data from input modules         */

/*............. Memory allocation */
#ifndef cam_alloc
    #define cam_alloc(size)     vcmalloc(((size)+3)/4)  /* size in bytes  */
    #define cam_alloc_int(size) vcmalloc(size)          /* size in ints   */
    #define cam_free            vcfree
#endif

/*............. General purpose macros */
#define HIGH_BYTE(a)          ((a)>>8 & 0xFF)
#define LOW_BYTE(a)           ((a) & 0xFF)
#define BYTES_TO_WORD(hi,lo)  (((unsigned short)(hi)<<8) | ((lo) & 0xFF))

/*----------------------------------------------------------------------*/
/*                              Error codes                             */
/*----------------------------------------------------------------------*/
#ifndef R_OK
  #define R_OK              0   /* Success                              */
#endif
#define BK_R_NO_MEMORY    2120  /* Memory allocation error              */
#define BK_R_NO_CONFIG    2121  /* Configuration file not found         */
#define BK_R_INV_FORMAT   2122  /* Invalid format of config. file       */
#define BK_R_MODBUF_OVF   2123  /* Module buffer overflow: # of config. */
                                /* file modules > max allowed modules   */
#define BK_R_DATABUF_OVF  2124  /* Out/in data buffer overflow          */
#define BK_R_INV_MODNUM   2125  /* Invalid module number for data access*/
#define BK_R_INV_HANDLE   2126  /* Invalid library handle               */
#define BK_R_TCP_INITERR  2127  /* TCP initialization error             */
#define BK_R_TCP_SENDERR  2128  /* TCP query send error                 */
#define BK_R_TCP_RECVERR  2129  /* TCP response receive error           */
#define BK_R_SLAVE_ERR    2130  /* MosbusTCP slave error                */
#define BK_R_RESP_LEN     2131  /* Invalid response message length      */
#define BK_R_RESP_ERR     2132  /* Invalid response message format      */
#define BK_R_CNT_MISMATCH 2133  /* Response byte count mismatch         */

/*----------------------------------------------------------------------*/
/*                           Function prototypes                        */
/*----------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

int  bk_open (
    BK_HND *bk_hnd,         /* output BK9000 library handle             */
    char   *cfg_file,       /* input BK9000 library configuration file  */
    char   *ip_addr,        /* input IP address of BK9000 bus coupler   */
    int     port,           /* input connection port (0=default 502)    */
    int     con_timeout,    /* input connection timeout in ms           */
    int     io_timeout,     /* input I/O timeout in ms                  */
    int     wd_time );      /* input watchdog time in ms (0=disable)    */

void bk_close (
    BK_HND *bk_hnd );       /* input/output BK9000 library handle       */

int  bk_xfer (
    BK_HND  bk_hnd,         /* input BK9000 library handle              */
    int     mode,           /* input transfer mode                      */
    int     timeout );      /* input response timeout in ms             */

int  bk_write (
    BK_HND   bk_hnd,        /* input BK9000 library handle              */
    unsigned int mod_num,   /* input module number (0,1,...)            */
    short   *dbuf );        /* input 16-bit short buffer                */

int  bk_read (
    BK_HND   bk_hnd,        /* input BK9000 library handle              */
    unsigned int mod_num,   /* input module number (0,1,...)            */
    short   *dbuf );        /* output 16-bit short buffer               */

int  bk_mod_count (
    BK_HND  bk_hnd );       /* input BK9000 library handle              */

int  bk_mod_info (
    BK_HND   bk_hnd,        /* input BK9000 library handle              */
    unsigned int mod_num,   /* input module number: 0=POS1,1=POS2,...   */
    int     *mod_type,      /* output module type                       */
    int     *chan_cnt );    /* output # of module channels (words/bitd) */

int  bk_get_stat (
    BK_HND  bk_hnd );       /* input BK9000 library handle              */

int  bk_watchdog_reset (
    BK_HND  bk_hnd,         /* input BK9000 library handle              */
    int     wd_time );      /* input watchdog time in ms (0=disable)    */

unsigned long sys_time ();

#ifdef __cplusplus
}
#endif

#endif /* _VCRT_h */

/* EOF */

